/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_ADC_InnerTempture.c
* @author    : FMD AE
* @brief     : bsp_ADC_InnerTempture program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "bsp_ADC_InnerTempture.h"

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

#define TS_CAL1_ADR  0x1FFFF7B8
#define TS_CAL2_ADR  0x1FFFF7C2

#define TS_CAL1_TEMPTURE  25 //25°c
#define TS_CAL2_TEMPTURE  110//110°c

/**
  * @brief  Configure ADC .
  * @param  None 
  *          
  *           
  *            
  *           
  *            
  * @retval None
  */
void ADC_Config(void)
{
  ADC_InitTypeDef     ADC_InitStructure;
  GPIO_InitTypeDef    GPIO_InitStructure;
  
  /* GPIOC Periph clock enable */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
  
  /* ADC1 Periph clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
  
  /* Configure ADC Channel11 as analog input */
	 GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3 ;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
	 GPIO_Init(GPIOA, &GPIO_InitStructure);
  
  /* ADCs DeInit */  
  ADC_DeInit(ADC1);
  
  /* Initialize ADC structure */
  ADC_StructInit(&ADC_InitStructure);
  
  /* Configure the ADC1 in continuous mode with a resolution equal to 12 bits  */
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
  ADC_InitStructure.ADC_ContinuousConvMode = ENABLE; 
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_ScanDirection = ADC_ScanDirection_Upward;
  ADC_Init(ADC1, &ADC_InitStructure); 


	ADC_ChannelConfig(ADC1, ADC_Channel_TempSensor , ADC_SampleTime_239_5Cycles);
  /* ADC Calibration */
  ADC_GetCalibrationFactor(ADC1);
  
  /* Enable the ADC peripheral */
  ADC_Cmd(ADC1, ENABLE);     
  
  /* Wait the ADRDY flag */
  while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_ADRDY)); 
  
	ADC_TempSensorCmd(ENABLE);
	ADC_VrefintCmd(ENABLE);
	
	/* ADC1 regular Software Start Conv */ 
  ADC_StartOfConversion(ADC1);
}


/**
  * @brief  Delay_ms .
  * @param  delay 
  *          
  *           
  *            
  *           
  *            
  * @retval None
  */
static void Delay_ms(uint32_t delay)
	{//主时钟为48MHz
	uint32_t count,cnt;

  count = delay;
	while (--count)
	{
		cnt = 5963;
		while (--cnt)
		{
			__ASM("");
		}
	}
	
}

/**
  * @brief  Delay_ms .
  * @param  None 
  *          
  *           
  *            
  *           
  *            
  * @retval None
  */
float GetTempture(void)
{
	uint16_t TS_CAL1,TS_CAL2,CurrAdcValue;
	float tempValue,CurrTempture;
	
	TS_CAL1 = *(uint16_t *)(TS_CAL1_ADR);	//获取温度校准值1
	TS_CAL1 &= 0x0fff;
	TS_CAL2 = *(uint16_t *)(TS_CAL2_ADR);//获取温度校准值2
	TS_CAL2 &= 0x0fff;
	
	/* Test EOC flag */
	while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
			
			/* Get ADC1 converted data */
	CurrAdcValue =ADC_GetConversionValue(ADC1);
	
	tempValue = (TS_CAL2_TEMPTURE - TS_CAL1_TEMPTURE)*(CurrAdcValue - TS_CAL1);
	tempValue /=(TS_CAL2 - TS_CAL1);
	tempValue += TS_CAL1_TEMPTURE;
	CurrTempture = tempValue;
	
	return CurrTempture;
}

/**
  * @brief  ADC_Process .
  * @param  None 
  *          
  *           
  *            
  *           
  *            
  * @retval None
  */
void ADC_Process(void)
{
	static float tempture;
		 
	while(1)
	{
		 tempture = GetTempture();
		 Delay_ms(1000);
	}
}
