/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : main program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
__IO uint16_t  ADC1ConvertedValue1 = 0, ADC1ConvertedVoltage1 = 0;

/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
static void TIM3_Config(void);
static void Buttons_Initialize(void);

/* Public function ------ --------------------------------------------------------*/
/* Private function ------ -------------------------------------------------------*/
/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
    /*  Configures LEDs GPIO */
    LedConfig(LED1);
    LedConfig(LED2);
    Buttons_Initialize();

    /* ADC Configuration */
    ADC_Config();

    /* TIM3 Configuration */
    TIM3_Config();

    /* ADC1 regular Software Start Conv */
    ADC_StartOfConversion(ADC1);

    /* Infinite loop */
    while (1)
    {
        while (GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_13) == RESET)
        {
            LedOn(LED1);
            LedOff(LED2);
        }

        LedOn(LED2);
        LedOff(LED1);

        /* Get ADC1 converted data */
        ADC1ConvertedValue1 = ADC_GetConversionValue(ADC1);

        /* Compute the voltage */
        ADC1ConvertedVoltage1 = (ADC1ConvertedValue1 * 3300) / 0xFFF;
    }
}


/**
  * @brief  TIM3 configuration
  * @param  None
  * @retval None
  */
static void TIM3_Config(void)
{
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
    TIM_OCInitTypeDef        TIM_OCInitStructure;

    /* ADC1 Peripheral clock enable */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

    TIM_DeInit(TIM3);

    TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);

    TIM_OCStructInit(&TIM_OCInitStructure);

    /* Time base configuration */
    TIM_TimeBaseStructure.TIM_Period = 0xFF;
    TIM_TimeBaseStructure.TIM_Prescaler = 0x0;
    TIM_TimeBaseStructure.TIM_ClockDivision = 0x0;
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

    /* TIM3 TRGO selection */
    TIM_SelectOutputTrigger(TIM3, TIM_TRGOSource_Update);

    /* TIM3 enable counter */
    TIM_Cmd(TIM3, ENABLE);
}

/**
  * @brief  Buttons_Initialize program.
  * @param  None
  * @retval None
  */
static void  Buttons_Initialize(void)
{
    GPIO_InitTypeDef    GPIO_InitStructure;

    /* BUTTONS GPIO Periph clock enable */
    RCC_AHBPeriphClockCmd(RCC_AHBENR_GPIOCEN, ENABLE);

    /* Configure BUTTON1_GPIO_Pin in input pull down mode */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;

    GPIO_Init(GPIOC, &GPIO_InitStructure);
}

/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
