/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_ADC_SamleAndHold.c
* @author    : FMD AE
* @brief     : bsp_ADC_SamleAndHold program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "bsp_ADC_SamleAndHold.h"

/* Private macro -------------------------------------------------------------*/

#define SAMPLEANDHOLD
#define MAX_CNT     400

/* Private variables ---------------------------------------------------------*/

__IO uint16_t  ADC1ConvertedValue[MAX_CNT], ADC1ConvertedVoltage[MAX_CNT];

/**
  * @brief  Configure ADC .
  * @param  None
  *
  *
  *
  *
  *
  * @retval None
  */

void ADC_Config(void)
{
    ADC_InitTypeDef     ADC_InitStructure;
    GPIO_InitTypeDef    GPIO_InitStructure;

    /* GPIOC Periph clock enable */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    /* ADC1 Periph clock enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

    /* Configure ADC Channel11 as analog input */

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    /* ADCs DeInit */
    ADC_DeInit(ADC1);

    /* Initialize ADC structure */
    ADC_StructInit(&ADC_InitStructure);

    /* Configure the ADC1 in continuous mode with a resolution equal to 12 bits  */
    ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
    ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;//ENABLE;
    ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
    ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
    ADC_InitStructure.ADC_ScanDirection = ADC_ScanDirection_Upward;
    ADC_Init(ADC1, &ADC_InitStructure);

    /* Convert the ADC1 Channel 11 with 239.5 Cycles as sampling time */
#ifdef SAMPLEANDHOLD
    ADC_ChannelConfig(ADC1, ADC_Channel_8 | ADC_Channel_18, ADC_SampleTime_28_5Cycles);
#else
    ADC_ChannelConfig(ADC1, ADC_Channel_8 | ADC_Channel_9, ADC_SampleTime_28_5Cycles);
#endif

//  ADC_VrefselConfig(ADC_Vrefsel_2_5V);
    ADC_VrefintCmd(ENABLE);

    ADC_DiscModeCmd(ADC1, DISABLE);
    /* ADC Calibration */
    ADC_GetCalibrationFactor(ADC1);

//  ADC1->CFGR1 &= ~(0x0001EC04);

    /* Enable the ADC peripheral */
    ADC_Cmd(ADC1, ENABLE);

    /* Wait the ADRDY flag */
    while (!ADC_GetFlagStatus(ADC1, ADC_FLAG_ADRDY));


}



#ifdef SAMPLEANDHOLD
/**
  * @brief  ADC_Process .
  * @param  None
  *
  *
  *
  *
  *
  * @retval None
  */
void ADC_Process(void)
{
    static uint16_t cnt = 0;

    /* ADC1 regular Software Start Conv */
    ADC->CR2 |= 0x00000300;
    ADC_StartOfConversion(ADC1);
    /* Infinite loop */
    while (1)
    {
        while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOSMP == RESET));
        /*Set SMPEN = 0 to maintain the voltag of PB1 port*/
        ADC->CR2 &= ~0x00000200;
        ADC->CR2 |= 0x00000100;
        /* Test EOC flag */
        while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
        /* Get ADC1 converted data */
        ADC1ConvertedValue[cnt] = ADC_GetConversionValue(ADC1);

        /* Test EOC flag */
        while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
//      /* Test EOSEQ flag */

        /* Get ADC1 converted data */
        ADC1ConvertedValue[cnt + 1] = ADC_GetConversionValue(ADC1);
        /*clear EOS 、EOSMP、ADRDY zero*/
        ADC1->ISR |= (ADC_FLAG_EOSEQ | ADC_FLAG_EOSMP | ADC_FLAG_ADRDY);

        /* Compute the voltage */
        ADC1ConvertedVoltage[cnt] = (ADC1ConvertedValue[cnt] * 3300) / 0xFFF;
        ADC1ConvertedVoltage[cnt + 1] = (ADC1ConvertedValue[cnt + 1] * 3300) / 0xFFF;

        cnt += 2;
        if (cnt >= MAX_CNT)
            cnt = 0;

        /*Start IO sample and hold circuit*/
        ADC->CR2 |= 0x00000300;
        /* ADC1 regular Software Start Conv */
        ADC_StartOfConversion(ADC1);
    }
}

#else
/**
  * @brief  ADC_Process .
  * @param  None
  *
  *
  *
  *
  *
  * @retval None
  */
void ADC_Process(void)
{
    static uint16_t cnt = 0;

    /* ADC1 regular Software Start Conv */
    ADC_StartOfConversion(ADC1);
    /* Infinite loop */
    while (1)
    {
        /* Test EOC flag */
        while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);

        /* Get ADC1 converted data */
        ADC1ConvertedValue[cnt] = ADC_GetConversionValue(ADC1);
        /* Test EOC flag */
        while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);

        /* Get ADC1 converted data */
        ADC1ConvertedValue[cnt + 1] = ADC_GetConversionValue(ADC1);
//      ADC1->ISR &= ~ADC_FLAG_EOSMP;
        ADC1->ISR |= (ADC_FLAG_EOSEQ | ADC_FLAG_EOSMP | ADC_FLAG_ADRDY); //clear EOS zero

        while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOSEQ) == SET);
        /* Compute the voltage */
        ADC1ConvertedVoltage[cnt] = (ADC1ConvertedValue[cnt] * 3300) / 0xFFF;
        ADC1ConvertedVoltage[cnt + 1] = (ADC1ConvertedValue[cnt + 1] * 3300) / 0xFFF;

        cnt += 2;
        if (cnt >= MAX_CNT)
            cnt = 0;

        /* ADC1 regular Software Start Conv */
        ADC_StartOfConversion(ADC1);
    }
}

#endif
