/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : Main program body
* @version   : V1.1.0
* @data      : 2022-05-30
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/

#include "main.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
void Test_IO(void);

/* Public function ------ --------------------------------------------------------*/
/* Private function ------ -------------------------------------------------------*/


/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
    /*!< At this stage the microcontroller clock setting is already configured,
       this is done through SystemInit() function which is called from startup
       file (startup_ft32f0xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_ft32f0xx.c file
     */
    /* ADC Config */
    ADC_Config();
    
    /* TIM1 Config*/
    TIM1_Config();
    
    /* Test IO */
    Test_IO();
    
    while (1)
    {

    }
}
/**
  * @brief  Test_IO program.
  * @param  None
  * @retval None
  */
void Test_IO(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOC,ENABLE);
    
    /* Test io config */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_3;
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_3;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource11, GPIO_AF_2);
    
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_3;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
}
/**
  * @brief  DelayNus program.
  * @param  time: This value can range from 0 to 0xffFFFFFF.
  * @retval None
  */
void DelayNus(uint32_t time)
{
    uint32_t i, j;
    /*when sysclk equal 48M ,the delay is about 1us*/
    for (i = 0; i < time; i++)
    {
        for (j = 0; j < 4; j++)
        {
            __NOP();
        }
    }
}

/**
  * @brief  DelayNms program.
  * @param  time: This value can range from 0 to 0xffFFFFFF.
  * @retval None
  */
void DelayNms(uint32_t time)
{
    for (uint32_t i = 0; i < time; i++)
    {
        DelayNus(1000);
    }
}

/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
