/**
    ******************************************************************************
    * @file     bsp_ADC_BasicExample.c
    * @author   FMD AE
    * @brief    bsp_ADC_BasicExample program body
    * @version  V1.0.0
    * @data     2021-09-14
    ******************************************************************************
    * @attention
    * COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
    * This software is provided by the copyright holders and contributors,and the
    *   software is believed to be accurate and reliable. However, Fremont Micro
    *   Devices (SZ) Corporation assumes no responsibility for the consequences of
    *   use of such software or for any infringement of patents of other rights
    *   of third parties, which may result from its use. No license is granted by
    *   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
    *   Corporation.
    ******************************************************************************
    */

/* Includes ------------------------------------------------------------------*/
#include "bsp_adc.h"

/* Private Constant --------------------------------------------------------------*/
#define VREFINT_CAL  ((*(uint16_t *)0x1FFFF7BA) & 0x0FFF)

/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/

/* Private variables -------------------------------------------------------------*/
__IO uint16_t  ADC1ConvertedValue = 0, ADC1ConvertedVoltage = 0;
__IO uint16_t  VDDA_Voltage = 0, Vref_Int_Value = 0;

/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/

/**
  * @brief  ADC_Config program.
  * @param  None
  * @retval None
  */
void ADC_Config(void)
{
    ADC_InitTypeDef     ADC_InitStructure;
    GPIO_InitTypeDef    GPIO_InitStructure;

    /* GPIOC Periph clock enable */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);

    /* ADC1 Periph clock enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

    /* ADC Clock = HSI14 */
    RCC_HSI14ADCRequestCmd(ENABLE);
    ADC_ClockModeConfig(ADC1, ADC_ClockMode_AsynClk);
    
    /* Config the VDDA as the Vref of ADC1 */
    ADC->CR2 = 0;
    /* Configure ADC Channel11 as analog input */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 ;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
    GPIO_Init(GPIOC, &GPIO_InitStructure);

    /* ADCs DeInit */
    ADC_DeInit(ADC1);

    /* Initialize ADC structure */
    ADC_StructInit(&ADC_InitStructure);

    /* Configure the ADC1 in continuous mode with a resolution equal to 12 bits  */
    ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
    ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
    ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
    ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
    ADC_InitStructure.ADC_ScanDirection = ADC_ScanDirection_Upward;
    ADC_Init(ADC1, &ADC_InitStructure);

    /* Configure the VDDA as the ADC reference voltage   */
    ADC_VrefselConfig(ADC_Vrefsel_VDDA);

    /* Enable Vref */
    ADC_VrefintCmd(ENABLE);

    /* ADC Calibration */
    ADC_GetCalibrationFactor(ADC1);

    /* Use discontinuous mode */
    ADC_ContinuousModeCmd(ADC1,DISABLE);
    
    /* Enable the ADC peripheral */
    ADC_Cmd(ADC1, ENABLE);

    /* Wait the ADRDY flag */
    while (!ADC_GetFlagStatus(ADC1, ADC_FLAG_ADRDY));

}
/**
  * @brief  ADC_Process program.
  * @param  None
  * @retval None
  */
void ADC_Process(void)
{
    /* Infinite loop */
    while (1)
    {
        /* Clear Channel */
        ADC1->CHSELR = 0;
        
        /* Convert the ADC1 Channel 17 with 239.5 Cycles as sampling time */
        ADC_ChannelConfig(ADC1, ADC_Channel_17, ADC_SampleTime_7_5Cycles);

        /* ADC1 regular Software Start Conv */
        ADC_StartOfConversion(ADC1);

        /* Test EOC flag */
        while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);

        /* Get ADC1 converted data */
        Vref_Int_Value = ADC_GetConversionValue(ADC1);

        /* Compute the VDDA voltage */
        VDDA_Voltage = ( ((uint32_t)VREFINT_CAL) * 3300) / Vref_Int_Value;
        
        /* Clear Channel */
        ADC1->CHSELR = 0;
        /* Convert the ADC1 Channel 10 with 239.5 Cycles as sampling time */
        ADC_ChannelConfig(ADC1, ADC_Channel_10, ADC_SampleTime_7_5Cycles);
        
        ADC_StartOfConversion(ADC1);
        
        /* Test EOC flag */
        while (ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);

        /* Get ADC1 converted data */
        ADC1ConvertedValue = ADC_GetConversionValue(ADC1);

        /* Compute the voltage */
        ADC1ConvertedVoltage = ( ((uint32_t)ADC1ConvertedValue) * VDDA_Voltage) / 0xFFF;
        
    }
}
/* Private function ------ -------------------------------------------------------*/
/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
