/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : ft32f0xx_it.c
* @author    : FMD AE
* @brief     : Main Interrupt Service Routines.
*              This file provides template for all exceptions handler and
*              peripherals interrupt service routine.
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "FT32F0xx_it.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/
/* Private function ------ -------------------------------------------------------*/
/******************************************************************************
  * @brief  This function handles NMI exception.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void NMI_Handler(void)
{
}

/******************************************************************************
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void HardFault_Handler(void)
{
    while (1)
    {
    }
}
/******************************************************************************
  * @brief  This function handles SVCall exception.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void SVC_Handler(void)
{
}

/******************************************************************************
  * @brief  This function handles PendSVC exception.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void PendSV_Handler(void)
{
}

/******************************************************************************
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void SysTick_Handler(void)
{
    g_cTime10msFlag = 1;
}

/******************************************************************************
  * @brief  This function handles USART1_IRQHandler.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void USART1_IRQHandler(void)
{
    uint8_t Len;

    /* Detects whether a USART idle interrupt has occurred or not */
    if (USART_GetFlagStatus(USART1, USART_FLAG_IDLE) != RESET)
    {
        /* Clear the IDLE interrupt flag bit */
        USART_ClearFlag(USART1, USART_FLAG_IDLE);

        /* DMA disable */
        DMA_Cmd(DMA1_Channel3, DISABLE);

        /* Clear DMA receive completion interrupt flag*/
        DMA_ClearFlag(DMA1_FLAG_TC3);

        /* Gets the length of a received frame of data */
        Len = RX_SIZE - DMA_GetCurrDataCounter(DMA1_Channel3);

        /* Send the received data back */
        for (uint8_t i = 0; i < Len; i++)
        {
            USART_SendData(USART1, g_cUsartRxBuff[i]);
            while (USART_GetFlagStatus(USART1, USART_FLAG_TC) != SET);
        }

        /* Reset the DMA counter value */
        DMA_SetCurrDataCounter(DMA1_Channel3, RX_SIZE);

        /* DMA enable */
        DMA_Cmd(DMA1_Channel3, ENABLE);
    }
}
/******************************************************************************
  * @brief  This function handles TIM1_BRK_UP_TRG_COM_IRQHandler.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void TIM1_BRK_UP_TRG_COM_IRQHandler(void)
{
    /* Checks whether the TIM update Interrupt has occurred or not */
    if (TIM_GetITStatus(TIM1, TIM_IT_Update) != RESET)
    {
        /* Clear the update interrupt flag bit */
        TIM_ClearITPendingBit(TIM1, TIM_IT_Update);
    }
}
/************************ (C) COPYRIGHT FMD *****END OF FILE****/
