/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : main program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private Constant --------------------------------------------------------------*/
/* LED flash time :unit ms */
const uint8_t   LED_FLASH_TIME = 200;

/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* USART reception Buff */
__IO uint8_t g_cUsartRxBuff[RX_SIZE];

/* For timing */
__IO uint32_t g_wTimeCount;

/* 10 ms timing flag */
uint8_t g_cTime10msFlag;

/* Private function prototypes ---------------------------------------------------*/
static void SystemClkSet(void);

/* Public function ------ --------------------------------------------------------*/
/* Private function ------ -------------------------------------------------------*/

/******************************************************************************
  * @brief  main program.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
int main(void)
{
    /*!< At this stage the microcontroller clock setting is already configured,
       this is done through SystemInit() function which is called from startup
       file (startup_ft32f0xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_ft32f0xx.c file
     */

    /* Set SYSCLK to 72M  */
    SystemClkSet();

    /* Configuring SysTick 10 ms generates an interrupt */
    SysTick_Configuration();

    /* Initialize the four leds on the board */
    LED_Init();

    /* Initializes 2 keys on the board */
    KEY_Init();

    /* Initialize the USART1 */
    USART_Config();

    /* Initialize the ADC and begin conversion */
    ADC_Config();

    /* Configure IWDG, 200ms not feeding the dog will cause a reset */
    IWDG_Init();

    /* Configure TIM1 Channel 1 and 1N */
    TIM_Config();

    /* Print SYSRESET */
    printf("SYSRESET!!!!\r\n");

    while (1)
    {
        if (g_cTime10msFlag == 1)
        {
            g_cTime10msFlag = 0;

            g_wTimeCount++;

            /* Feed dog */
            IWDG_ReloadCounter();

            /* Key detection and processing */
            KEY_Pro();

            /* LED3 Flash:  */
            LED_Flash(LED_FLASH_TIME);

            /* ADC voltage acquisition and processing  */
            ADC_Measure();

        }
    }
}

/******************************************************************************
  * @brief  SystemClkSet program.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
static void SystemClkSet(void)
{
    __IO uint32_t StartUpCounter = 0;

    /* Enable HSI */
    RCC_HSICmd(ENABLE);

    /* Wait till HSI is ready and if Time out is reached exit */
    do
    {
        StartUpCounter++;
    }
    while ((RCC_GetFlagStatus(RCC_FLAG_HSIRDY) == RESET) && (StartUpCounter != HSI_STARTUP_TIMEOUT));

    if (RCC_GetFlagStatus(RCC_FLAG_HSIRDY) == SET)
    {

        /* Select HSI as system clock source */
        RCC_SYSCLKConfig(RCC_SYSCLKSource_HSI);

        /* Wait till HSI is used as system clock source */
        while (RCC_GetSYSCLKSource() != RCC_CFGR_SWS_HSI)
        {

        }

        /* Disable the PLL */
        RCC_PLLCmd(DISABLE);

        /* Enable Prefetch Buffer */
        FLASH_PrefetchBufferCmd(ENABLE);

        /* set Flash Latency */
        FLASH_SetLatency(FLASH_Latency_2);

        /* HCLK = SYSCLK */
        RCC_HCLKConfig(RCC_SYSCLK_Div1);

        /* PCLK = HCLK */
        RCC_PCLKConfig(RCC_HCLK_Div1);

        /* PLL configuration */
        RCC_PLLConfig(RCC_PLLSource_HSI, RCC_PLLMul_9);

        /* Enable the PLL */
        RCC_PLLCmd(ENABLE);

        /* Wait till PLL is ready */
        while (RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
        {

        }

        /* Select PLL as system clock source */
        RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

        /* Wait till PLL is used as system clock source */
        while (RCC_GetSYSCLKSource() != RCC_CFGR_SWS_PLL)
        {

        }
    }

    /* Update SystemCoreClock according to Clock Register Values */
    SystemCoreClockUpdate();
}
/************************ (C) COPYRIGHT FMD *****END OF FILE****/
