/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_comp.c
* @author    : FMD AE
* @brief     : bsp_comp program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/

#include "bsp_comp.h"

uint32_t    comp1out = 0, comp2out = 0;
/**
  * @brief  Configure COMP1 COMP2
    * @note  Compare the voltage and output the result to comp1out and comp2out
  * @param  None
  *
  *
  * @retval None
  */
void Bsp_COMP_Config(void)
{

    COMP_InitTypeDef COMP_InitStruct = {0};
    GPIO_InitTypeDef GPIO_InitStruct = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB, ENABLE);

    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AN;
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_2 | GPIO_Pin_3;

    GPIO_Init(GPIOA, &GPIO_InitStruct);

    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_12;
    GPIO_Init(GPIOB, &GPIO_InitStruct);
    /*configure compare1 */
    COMP_InitStruct.COMP_VipSel = NCOMP_VIP_SEL_PAD_PB12;
    COMP_InitStruct.COMP_VinSel = NCOMP_VIN_SEL_PAD_PA0;
    COMP_InitStruct.COMP_OutputSel = COMP_OUTPUT_NO_SELECTION;
    COMP_InitStruct.COMP_Pol = NCOMP_POL_NOT_INVERT;
    COMP_Init(COMP_Selection_COMP1, &COMP_InitStruct);
    /*configure compare2 */
    COMP_InitStruct.COMP_VipSel = PCOMP_VIP_SEL_PAD_PA3;
    COMP_InitStruct.COMP_VinSel = PCOMP_VIN_SEL_PAD_PA2;
    COMP_InitStruct.COMP_OutputSel = COMP_OUTPUT_NO_SELECTION;
    COMP_InitStruct.COMP_Pol = NCOMP_POL_NOT_INVERT;
    COMP_Init(COMP_Selection_COMP2, &COMP_InitStruct);

    COMP_WindowCmd(ENABLE);
    /*compare1 enable*/
    COMP_Cmd(COMP_Selection_COMP1, ENABLE);
    /*compare2 enable*/
    COMP_Cmd(COMP_Selection_COMP2, ENABLE);

    COMP_LockConfig(COMP_Selection_COMP1);
    COMP_LockConfig(COMP_Selection_COMP2);

    while (1)
    {
        /*get the value of the compare1 output*/
        comp1out = COMP_GetOutputLevel(COMP_CSR_COMP1OUT);
        /*get the value of the compare2 output*/
        comp2out = COMP_GetOutputLevel(COMP_CSR_COMP2OUT);
    }
}

/**
  * @brief  reset for COMP1 and COMP2
    * @note
  * @param  None
  *
  *
  * @retval None
  */
void Bsp_COMP_ResetConfig(void)
{
    COMP_InitTypeDef COMP_InitStruct = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

    COMP_DeInit();
    COMP_StructInit(&COMP_InitStruct);
}
/**
  * @brief test COMP1 COMP2
    * @note
  * @param  None
  *
  *
  * @retval None
  */
void Bsp_COMP_Test(void)
{
    Bsp_COMP_ResetConfig();
    Bsp_COMP_Config();
}

