/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_spi.c
* @author    : FMD AE
* @brief     : bsp_spi program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "bsp_spi.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
const uint16_t spi1_tx_buff[RX_SIZE] =
{
    0x5588, 0x331e, 0x222d, 0x113c, 0xff4b,
    0xee5a, 0xdd69, 0xcc78, 0xbb87, 0xaa76,
    0x9985, 0x8894, 0x77a3, 0x66b2, 0x55c1,
    0x44d0, 0x33e1, 0x22ff, 0x11aa
};

const uint16_t spi2_tx_buff[RX_SIZE] =
{
    0x6598, 0x331e, 0x222d, 0x113c, 0xff4b,
    0xee5a, 0xdd69, 0xcc78, 0xbb87, 0xaa76,
    0x9985, 0x8894, 0x77a3, 0x66b2, 0x55c1,
    0x44d0, 0x33e1, 0x22ff, 0x11ab
};

/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
uint8_t data_addr = 0;
uint8_t rece_cnt2 = 0;
uint16_t spi1_rx_buff[RX_SIZE];
uint16_t spi2_rx_buff[RX_SIZE];


/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/
/**
  * @brief  Bsp_SPI1_GPIO_Config program.
  * @param  None
  * @retval None
  */
void Bsp_SPI1_GPIO_Config(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    /* Enable the clock of GPIOA */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    /* GPIO Init */
    GPIO_InitStructure.GPIO_Pin = SPI1_NSS | SPI1_SCK | SPI1_MISO | SPI1_MOSI;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(SPI1_PORT, &GPIO_InitStructure);

    /* Pin AF config */
    GPIO_PinAFConfig(SPI1_PORT, SPI1_NSS_PIN_SOURCE, SPI1_NSS_AF);
    GPIO_PinAFConfig(SPI1_PORT, SPI1_SCK_PIN_SOURCE, SPI1_SCK_AF);
    GPIO_PinAFConfig(SPI1_PORT, SPI1_MISO_PIN_SOURCE, SPI1_MISO_AF);
    GPIO_PinAFConfig(SPI1_PORT, SPI1_MOSI_PIN_SOURCE, SPI1_MOSI_AF);
}



/**
  * @brief  Bsp_SPI1_IT_Config program.
  * @param  None
  * @retval None
  */
void Bsp_SPI1_IT_Config(void)
{
    SPI_InitTypeDef SPI_InitStructure;

    /* Enable the clock of SPI1 */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);

    /* GPIO Init */
    Bsp_SPI1_GPIO_Config();

    /* SPI Init */
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Hard;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_Init(SPI1, &SPI_InitStructure);

    /* Set RxFIFO */
    SPI_RxFIFOThresholdConfig(SPI1, SPI_RxFIFOThreshold_QF);

    /* Output enable */
    SPI_SSOutputCmd(SPI1,ENABLE);
		
		SPI_LastDMATransferCmd(SPI1,SPI_LastDMATransfer_TxOddRxOdd);
		SPI_I2S_DMACmd(SPI1,SPI_I2S_DMAReq_Tx | SPI_I2S_DMAReq_Rx,ENABLE);
		Bsp_DMA_SPI1_TX_RX_Config();
    
    /* SPI1 enable*/
    SPI_Cmd(SPI1, ENABLE);

	while(1)
	{
		DMA_Cmd(DMA1_Channel3,ENABLE);
			Delay(100000);
		DMA_Cmd(DMA1_Channel3,DISABLE);
			Delay(100000);
	}
}


/**
  * @brief  Delay program.
  * @param  time:0~0xFFFFFFFF
  * @retval None
  */
void Delay(uint32_t time)
{
    uint32_t i = 0;

    for (i = 0; i < time; i++)
    {
        __NOP();
    }
}

/**
	* @ SPI1 DMA
	*/
void Bsp_DMA_SPI1_TX_RX_Config(void)
{
	DMA_InitTypeDef DMA_InitStruct;
	NVIC_InitTypeDef NVIC_InitStruct;
	
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1,ENABLE);
	
	/*SPI1 RX*/
	DMA_InitStruct.DMA_BufferSize = RX_SIZE;
	DMA_InitStruct.DMA_DIR = DMA_DIR_PeripheralSRC;
	DMA_InitStruct.DMA_M2M = DMA_M2M_Disable;
	DMA_InitStruct.DMA_MemoryBaseAddr = (uint32_t)&spi1_rx_buff[0];
	DMA_InitStruct.DMA_MemoryDataSize = 8;
	DMA_InitStruct.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStruct.DMA_Mode = DMA_Mode_Normal;
	DMA_InitStruct.DMA_PeripheralBaseAddr = (uint32_t)&SPI1->DR;
	DMA_InitStruct.DMA_PeripheralDataSize = 8;
	DMA_InitStruct.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStruct.DMA_Priority = DMA_Priority_Low;
	
	DMA_Init(DMA1_Channel2,&DMA_InitStruct);	
	
	DMA_Cmd(DMA1_Channel2,ENABLE);
	
	/*SPI1 TX*/
	DMA_InitStruct.DMA_BufferSize = RX_SIZE;
	DMA_InitStruct.DMA_DIR = DMA_DIR_PeripheralDST;
	DMA_InitStruct.DMA_M2M = DMA_M2M_Disable;
	DMA_InitStruct.DMA_MemoryBaseAddr = (uint32_t)&spi1_tx_buff[0];
	DMA_InitStruct.DMA_MemoryDataSize = 8;
	DMA_InitStruct.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStruct.DMA_Mode = DMA_Mode_Normal;
	DMA_InitStruct.DMA_PeripheralBaseAddr = (uint32_t)&SPI1->DR;
	DMA_InitStruct.DMA_PeripheralDataSize = 8;
	DMA_InitStruct.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStruct.DMA_Priority = DMA_Priority_High;
	
	DMA_Init(DMA1_Channel3,&DMA_InitStruct);	
	
	DMA_Cmd(DMA1_Channel3,ENABLE);
	
	DMA_ITConfig(DMA1_Channel2,DMA_IT_TC,ENABLE);
	DMA_ITConfig(DMA1_Channel3,DMA_IT_TC,ENABLE);
	
	NVIC_InitStruct.NVIC_IRQChannel = DMA1_Channel2_3_IRQn;
	NVIC_InitStruct.NVIC_IRQChannelPriority = 1;
	NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStruct);
}



