/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_flash.c
* @author    : FMD AE
* @brief     : bsp_flash program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "bsp_flash.h"

/* Private typedef -----------------------------------------------------------*/
typedef enum {FAILED = 0, PASSED = !FAILED} TestStatus;
/* Private define ------------------------------------------------------------*/
#define FLASH_PAGE_SIZE         ((uint32_t)0x00000200)   /* FLASH Page Size */
#define FLASH_USER_START_ADDR   ((uint32_t)0x08006000)   /* Start @ of user Flash area */
#define FLASH_USER_END_ADDR     ((uint32_t)0x08007000)   /* End @ of user Flash area */
#define DATA_32                 ((uint32_t)0x12345678)

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint32_t erase_counter = 0x00, address = 0x00;
uint32_t data = 0x3210ABCD;
uint32_t number_of_page = 0x00;
__IO FLASH_Status flash_status = FLASH_COMPLETE;
__IO TestStatus memory_program_status = PASSED;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Write Flash.
  * @param  None
  * @retval None
  */
void FlashWrite(void)
{
    /* Unlock the Flash to enable the flash control register access *************/
    FLASH_Unlock();

    /* Erase the user Flash area
      (area defined by FLASH_USER_START_ADDR and FLASH_USER_END_ADDR) ***********/

    /* Define the number of page to be erased */
    number_of_page = (FLASH_USER_END_ADDR - FLASH_USER_START_ADDR) / FLASH_PAGE_SIZE;

    for (erase_counter = 0; (erase_counter < number_of_page) && (flash_status == FLASH_COMPLETE); erase_counter++)
    {
        if (FLASH_ErasePage(FLASH_USER_START_ADDR + (FLASH_PAGE_SIZE * erase_counter)) != FLASH_COMPLETE)
        {
            /* Error occurred while sector erase.
                User can add here some code to deal with this error  */
            while (1)
            {

            }
        }

    }
    /* Program the user Flash area word by word
      (area defined by FLASH_USER_START_ADDR and FLASH_USER_END_ADDR) ***********/

    address = FLASH_USER_START_ADDR;

    while (address < FLASH_USER_END_ADDR)
    {
        if (FLASH_ProgramWord(address, DATA_32) == FLASH_COMPLETE)
        {
            address = address + 4;
        }
        else
        {
            /* Error occurred while writing data in Flash memory.
               User can add here some code to deal with this error */
            while (1)
            {

            }
        }
    }

    /* Lock the Flash to disable the flash control register access (recommended
       to protect the FLASH memory against possible unwanted operation) *********/
    FLASH_Lock();

    /* Check if the programmed data is OK
         MemoryProgramStatus = 0: data programmed correctly
         MemoryProgramStatus != 0: number of words not programmed correctly ******/
    address = FLASH_USER_START_ADDR;
    memory_program_status = PASSED;

    while (address < FLASH_USER_END_ADDR)
    {
        data = *(__IO uint32_t *)address;

        if (data != DATA_32)
        {
            memory_program_status = FAILED;
        }

        address = address + 4;
    }

}

/**
  * @brief  Read Flash.
        * @param  StartAdr:
                            ReadBuffer:
                            Size:
  * @retval None
  */
void FlashRead(uint32_t StartAdr, uint32_t *ReadBuffer, uint32_t Size)
{
    uint32_t *pt, i;

    pt = ReadBuffer;

    for (i = 0; i < Size; i++)
    {
        *pt++ = *(uint32_t *)(StartAdr + i * 4);
    }
}

