/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_flashWriteProtection.c
* @author    : FMD AE
* @brief     : bsp_flashWriteProtection program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "bsp_flashWriteProtection.h"


/* Private Constant ---------------------------------------------------------------*/
const uint32_t RX_SIZE = 20;

/* Public Constant ---------------------------------------------------------------*/
uint32_t TICK_RATE = 100;

/* Private typedef ---------------------------------------------------------------*/
typedef enum {FAILED = 0, PASSED = !FAILED} TestStatus;

/* Private define ----------------------------------------------------------------*/
#define FLASH_PAGE_SIZE                ((uint16_t)0x200)  //0X200=0.5K
#define FLASH_PAGES_TO_BE_PROTECTED    (OB_WRP_Pages0to7| OB_WRP_Pages8to15|\
                                          OB_WRP_Pages72to79 | OB_WRP_Pages80to87 | OB_WRP_Pages88to95|\
                                          OB_WRP_Pages96to103| OB_WRP_Pages104to111| OB_WRP_Pages112to119|\
                                          OB_WRP_Pages120to127/*|OB_WRP_AllPages*/)
#define BANK1_WRITE_START_ADDR         ((uint32_t)0x08006000)
#define BANK1_WRITE_END_ADDR           ((uint32_t)0x08008000)

/* Uncomment this line to program the Flash pages */
#define FLASH_PAGE_PROGRAM

/* Uncomment this line to Enable Write Protection */
//#define WRITE_PROTECTION_ENABLE

/* Uncomment this line to Disable Write Protection */
#define WRITE_PROTECTION_DISABLE


/* Private variables -------------------------------------------------------------*/
uint32_t EraseCounter = 0x0, Address = 0x0;
uint16_t Data = 0x1753;
uint32_t WRPR_Value = 0xFFFFFFFF, ProtectedPages = 0x0;
uint32_t NbrOfPage;
__IO FLASH_Status FLASHStatus = FLASH_COMPLETE;
__IO TestStatus MemoryProgramStatus = PASSED;


/* Public variables --------------------------------------------------------------*/


/* Private function prototypes ---------------------------------------------------*/
void LED_Toggle(GPIO_TypeDef *GPIOx, uint16_t GPIO_Pin);


/* Public function ------ --------------------------------------------------------*/

/******************************************************************************
  * @brief  delay time  program
  * @param  dlx
  * @note
  * @retval None
  *****************************************************************************
*/
void delaym(uint16_t dlx)
{
    while (dlx--);
}

/**
  * @brief  WriteFlashProcess program.
  * @param  None
  * @retval None
  */

void WriteFlashProcess(void)
{
    /* Unlock the Flash Program Erase controller */
    FLASH_Unlock();

    FLASH_OB_Unlock();

    FLASH_ClearFlag(FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPERR);

    /* Get pages write protection status */
    WRPR_Value = FLASH_OB_GetWRP();


    /*TO PROTECT*/
#ifdef WRITE_PROTECTION_ENABLE
    /*when PC13==H*/
//if(Bit_SET==GPIO_ReadInputDataBit(GPIOC,GPIO_Pin_13))
    {
        /* Get current write protected pages and the new pages to be protected */
        ProtectedPages = (~WRPR_Value) | FLASH_PAGES_TO_BE_PROTECTED;

        /* Check if desired pages are not yet write protected */
        if (((~WRPR_Value) & FLASH_PAGES_TO_BE_PROTECTED) != FLASH_PAGES_TO_BE_PROTECTED)
        {
            /* Erase all the option Bytes because if a program operation is
            performed on a protected page, the Flash memory returns a
            protection error */
            FLASHStatus = FLASH_OB_Erase();

            /* Enable the pages write protection */
            FLASHStatus = FLASH_OB_EnableWRP(ProtectedPages);

            /* Generate System Reset to load the new option byte values */
            FLASH_OB_Launch();
        }

    }
#endif



    /*TO PROTECT*/
#ifdef WRITE_PROTECTION_DISABLE
    /*when PC13==L*/
//if(Bit_RESET==GPIO_ReadInputDataBit(GPIOC,GPIO_Pin_13))
    {
        /* Get pages already write protected */
        ProtectedPages = ~(WRPR_Value | FLASH_PAGES_TO_BE_PROTECTED);

        /* Check if desired pages are already write protected */
        if ((WRPR_Value | (~FLASH_PAGES_TO_BE_PROTECTED)) != 0xFFFFFFFF)
        {
            /* Erase all the option Bytes */
            FLASHStatus = FLASH_OB_Erase();

            /* Check if there is write protected pages */
            if (ProtectedPages != 0x0)
            {
                /* Restore write protected pages */
                FLASHStatus = FLASH_OB_EnableWRP(ProtectedPages);
            }

            /* Generate System Reset to load the new option byte values */
            FLASH_OB_Launch();
        }
    }
#endif



#ifdef FLASH_PAGE_PROGRAM
    /* Get the number of pages to be erased */
    NbrOfPage = (BANK1_WRITE_END_ADDR - BANK1_WRITE_START_ADDR) / FLASH_PAGE_SIZE;

    /* The selected pages are not write protected */
    if ((WRPR_Value & FLASH_PAGES_TO_BE_PROTECTED) != 0x00)
    {
        /* Clear pending flags */
        FLASH_ClearFlag(FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPERR);

        /* erase the FLASH pages */
        for (EraseCounter = 0; (EraseCounter < NbrOfPage) && (FLASHStatus == FLASH_COMPLETE); EraseCounter++)
        {
            FLASHStatus = FLASH_ErasePage(BANK1_WRITE_START_ADDR + (FLASH_PAGE_SIZE * EraseCounter));
        }

        /* FLASH Half Word program of data 0x1753 at addresses defined by  BANK1_WRITE_START_ADDR and BANK1_WRITE_END_ADDR */
        Address = BANK1_WRITE_START_ADDR;

        while ((Address < BANK1_WRITE_END_ADDR) && (FLASHStatus == FLASH_COMPLETE))
        {
            FLASHStatus = FLASH_ProgramWord(Address, Data);
            Address = Address + 4;
        }

        /* Check the correctness of written data */
        Address = BANK1_WRITE_START_ADDR;

        while ((Address < BANK1_WRITE_END_ADDR) && (MemoryProgramStatus != FAILED))
        {
            if ((*(__IO uint32_t *) Address) != Data)
            {
                MemoryProgramStatus = FAILED;
            }
            Address += 4;
        }
    }
    else
    {
        /* Error to program the flash : The desired pages are write protected */
        MemoryProgramStatus = FAILED;
    }

#endif /* FLASH_PAGE_PROGRAM */

}
