/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : Main program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "main.h"

void MCO_Config(void);
void Set_Sysclk_72M(void);

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
    /*!< At this stage the microcontroller clock setting is already configured,
       this is done through SystemInit() function which is called from startup
       file (startup_ft32f0xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_ft32f0xx.c file
     */

    /* Config HSE as PLL Source */
    Set_Sysclk_72M();
    
    /* MCO config :Prescaler 64 */
    MCO_Config();
    
    /* Initialize LED-related GPIO */
    LedConfig(LED1);
    LedConfig(LED2);
    LedConfig(LED3);
    LedConfig(LED4);

    while (1)
    {
        LedToggle(LED1);
        DelayNms(100);

        LedToggle(LED2);
        DelayNms(100);

        LedToggle(LED3);
        DelayNms(100);

        LedToggle(LED4);
        DelayNms(100);
    }
}
/**
  * @brief  Set_Sysclk_72M program.
  * @param  None
  * @retval None
  */
void Set_Sysclk_72M(void)
{
    RCC_DeInit();
    
    /* Enable Prefetch Buffer and set Flash Latency */
    FLASH->ACR = FLASH_ACR_PRFTBE | ((uint32_t)0x00000002);

    RCC_HSEConfig(RCC_HSE_ON);
    while (RCC_GetFlagStatus(RCC_FLAG_HSERDY) != SET);

    RCC_PLLConfig(RCC_PLLSource_HSE, RCC_PLLMul_9);
    RCC_PLLCmd(ENABLE);
    while (RCC_GetFlagStatus(RCC_FLAG_PLLRDY) != SET);

    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
    /*PLL as sysclk*/
    while (RCC_GetSYSCLKSource() != 0x08);

    RCC_HCLKConfig(RCC_SYSCLK_Div1);
    RCC_PCLKConfig(RCC_HCLK_Div1);

    /* Update SystemCoreClock */
    SystemCoreClockUpdate();

}
/**
  * @brief  MCO_Config program.
  * @param  None
  * @retval None
  */
void MCO_Config(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    /* MCO pin configuration: PA8 */
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_8;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_0);

    /* Output System Clock on MCO pin */
    RCC_MCOConfig(RCC_MCOSource_SYSCLK, RCC_MCOPrescaler_64);
}

/**
  * @brief  DelayNus program.
  * @param  time: This value can range from 0 to 0xffFFFFFF.
  * @retval None
  */
void DelayNus(uint32_t time)
{
    uint32_t i, j;
    /*when sysclk equal 48M ,the delay is about 1us*/
    for (i = 0; i < time; i++)
    {
        for (j = 0; j < 4; j++)
        {
            __NOP();
        }
    }
}

/**
  * @brief  DelayNms program.
  * @param  time: This value can range from 0 to 0xffFFFFFF.
  * @retval None
  */
void DelayNms(uint32_t time)
{
    for (uint32_t i = 0; i < time; i++)
    {
        DelayNus(1000);
    }
}

/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
