/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : main program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "main.h"
#include "string.h"

/* Private Constant --------------------------------------------------------------*/

/* Private typedef ---------------------------------------------------------------*/
typedef enum {PASSED = 0, FAILED = !PASSED} TestStatus;
/* Private define ----------------------------------------------------------------*/
#define countof(a) (sizeof(a) / sizeof(*(a)))
#define sEE_WRITE_ADDRESS1        0x50
#define sEE_READ_ADDRESS1         0x50
#define BUFFER_SIZE1             (countof(Tx1Buffer)-1)
#define BUFFER_SIZE2             (countof(Tx2Buffer)-1)
#define sEE_WRITE_ADDRESS2       (sEE_WRITE_ADDRESS1 + BUFFER_SIZE1)
#define sEE_READ_ADDRESS2        (sEE_READ_ADDRESS1 + BUFFER_SIZE1)
/* Private variables -------------------------------------------------------------*/
uint8_t Tx1Buffer[] = "/* FT32f0xx I2C Firmware Library EEPROM driver example*/3";

uint8_t Tx2Buffer[] = "/* FT32f0xx I2C Firmware Library EEPROM driver example : I2C1 interfacing with EEPROM */";

uint8_t Rx1Buffer[BUFFER_SIZE1], Rx2Buffer[BUFFER_SIZE2];

volatile TestStatus TransferStatus1 = FAILED, TransferStatus2 = FAILED;

volatile uint16_t NumDataRead = 0;
/* Public variables --------------------------------------------------------------*/


/* Private function prototypes ---------------------------------------------------*/
void Delay(uint32_t ms);
/**********************************************************************************
  * @brief  main program.
  * @param  None
  * @note
  * @retval None
  *********************************************************************************
*/

int main(void)
{
    /* Initialize the I2C EEPROM driver ----------------------------------------*/
    sEE_Init();

    /* First write in the memory followed by a read of the written data --------*/
    /* Write on I2C EEPROM from sEE_WRITE_ADDRESS1 */
    sEE_WriteBuffer(Tx1Buffer, sEE_WRITE_ADDRESS1, BUFFER_SIZE1);

    /* Wait for EEPROM standby state */
    sEE_WaitEepromStandbyState();

    /* Set the Number of data to be read */
    NumDataRead = BUFFER_SIZE1;

    /* Read from I2C EEPROM from sEE_READ_ADDRESS1 */
    sEE_ReadBuffer(Rx1Buffer, sEE_READ_ADDRESS1, (uint16_t *)(&NumDataRead));

    /* Check if the data written to the memory is read correctly */
    TransferStatus1 = memcmp(Tx1Buffer, Rx1Buffer, BUFFER_SIZE1);

    /* Second write in the memory followed by a read of the written data -------*/
    /* Write on I2C EEPROM from sEE_WRITE_ADDRESS2 */
    sEE_WriteBuffer(Tx2Buffer, sEE_WRITE_ADDRESS2, BUFFER_SIZE2);

    /* Wait for EEPROM standby state */
    sEE_WaitEepromStandbyState();

    /* Set the Number of data to be read */
    NumDataRead = BUFFER_SIZE2;

    /* Read from I2C EEPROM from sEE_READ_ADDRESS2 */
    sEE_ReadBuffer(Rx2Buffer, sEE_READ_ADDRESS2, (uint16_t *)(&NumDataRead));


    /* Check if the data written to the memory is read correctly */
    TransferStatus2 = memcmp(Tx2Buffer, Rx2Buffer, BUFFER_SIZE2);

    while (1)
    {
    }
}
/* Private function ------ -------------------------------------------------------*/

/**********************************************************************************
  * @brief  Delay program.
  * @param  ms
  * @note
  * @retval None
  *********************************************************************************
*/
void Delay(uint32_t us)
{
    uint32_t i, j;
    for (i = us; i > 0; i--)
    {
        for (j = 0; j < 1000; j++)
        {
            __ASM("NOP");
        }
    }
}
/************************ (C) COPYRIGHT FMD *****END OF FILE****/
