/**
    ******************************************************************************
    * @file         bsp_i2c.c
    * @author       FMD AE
    * @brief        bsp_i2c program body
    * @version      V1.0.0
    * @data         2022-05-30
    ******************************************************************************
    * @attention
    * COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
    * This software is provided by the copyright holders and contributors,and the
    *   software is believed to be accurate and reliable. However, Fremont Micro
    *   Devices (SZ) Corporation assumes no responsibility for the consequences of
    *   use of such software or for any infringement of patents of other rights
    *   of third parties, which may result from its use. No license is granted by
    *   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
    *   Corporation.
    ******************************************************************************
    */

/* Includes ------------------------------------------------------------------*/
#include "bsp_i2c.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/

#define sEE_MAX_TRIALS_NUMBER     300
#define sEE_FLAG_TIMEOUT         ((uint32_t)0x1000)
#define sEE_LONG_TIMEOUT         ((uint32_t)(10 * sEE_FLAG_TIMEOUT))
#define sEE_OK                    0
#define sEE_FAIL                  1

__IO uint32_t  sEETimeout = sEE_LONG_TIMEOUT;
uint8_t DeviceAdr = 0x7A;
uint8_t send_buff[256] = {0x99, 0xaa, 0x55, 0x00, 0x69};
uint8_t send_buff2[256] = {0x11, 0xaa, 0x55, 0xEE, 0x77};
uint8_t receive_buff[256];
uint16_t recev_cnt = 0;
uint16_t I2C_SEND_SIZE = 5;
uint16_t match_addr = 0;
uint8_t smbus_mode = 0;
uint8_t CurrentAdrSlaveDev = 0;
uint8_t readNum;
uint8_t RegAdr;

/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/
/**
  * @brief  Bsp_I2C_INTERRUPT_Mode_Config program.
  * @param  None
  * @retval None
  */
void Bsp_I2C_INTERRUPT_Mode_Config(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    I2C_InitTypeDef I2C_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

    /*I2C1:SCL:PB8  SDA:PB9   AF1*/
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C2, ENABLE);

    /* Configure SYSCLK as the clock of IC21 */
    RCC_I2CCLKConfig(RCC_I2C1CLK_SYSCLK);

    /*GPIO Init*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_1);
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_1);

    I2C_DeInit(I2C1);
    I2C_StructInit(&I2C_InitStructure);
    /*Tclk = 8MHz,Timing is set to 0x3139*/
    I2C_InitStructure.I2C_Timing = 0x3139;
    I2C_Init(I2C1, &I2C_InitStructure);

    I2C_Cmd(I2C1, ENABLE);

    /*I2C2:SCL:PB10     SDA:PB11 */
    /*GPIO Init*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_PinAFConfig(GPIOB, GPIO_PinSource10, GPIO_AF_1);
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource11, GPIO_AF_1);

    I2C_DeInit(I2C2);

    /* Disable clock extension */
    I2C_StretchClockCmd(I2C2, DISABLE);

    I2C_ITConfig(I2C2, I2C_IT_RXI, ENABLE);


    I2C_StructInit(&I2C_InitStructure);
    /*Tclk = 8MHz,Timing is set to 0x3139*/
    I2C_InitStructure.I2C_Timing = 0x3139;
    I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
    I2C_InitStructure.I2C_OwnAddress1 = SLAVE_DEVICE_ADDR;
    I2C_Init(I2C2, &I2C_InitStructure);
		
		//config OAR2 for slave device
		I2C_OwnAddress2Config(I2C2, SLAVE_DEVICE_ADDR2, I2C_OA2_NoMask);
		I2C_DualAddressCmd(I2C2, ENABLE);

    NVIC_InitStructure.NVIC_IRQChannel = I2C2_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    I2C_Cmd(I2C2, ENABLE);

    while (1)
    {
				DeviceAdr = 0xA4;
				readNum = 10;
				RegAdr = 0x00;
			
				sEE_SendBuffer(send_buff2, RegAdr, (uint8_t*)(&readNum));
				Delay(5000);
				sEE_ReadBuffer(receive_buff, RegAdr, (uint16_t*)(&readNum));
				sEE_WaitEepromStandbyState();
				Delay(5000);
    }
}

/**
  * @brief  sEE_SendBuffer program.
  * @param  *pBuffer  WriteAddr *NumByteToWrite
  * @retval None
  */
uint32_t sEE_SendBuffer(uint8_t *pBuffer, uint16_t WriteAddr, uint8_t *NumByteToWrite)
{
    uint32_t DataNum = 0;

    /* Configure slave address, nbytes, reload and generate start */
    I2C_TransferHandling(I2C1, DeviceAdr, 1, I2C_Reload_Mode, I2C_Generate_Start_Write);

    /* Wait until TXIS flag is set */
    sEETimeout = sEE_LONG_TIMEOUT;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_TXIS) == RESET)
    {
        if ((sEETimeout--) == 0) return 1;
    }

    /* Send memory address */
    I2C_SendData(I2C1, (uint8_t)WriteAddr);
    /* Wait until TCR flag is set */
    sEETimeout = sEE_LONG_TIMEOUT;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_TCR) == RESET)
    {
        if ((sEETimeout--) == 0) return 2;
    }

    /* Update CR2 : set Slave Address , set write request, generate Start and set end mode */
    I2C_TransferHandling(I2C1, DeviceAdr, (uint8_t)(*NumByteToWrite), I2C_AutoEnd_Mode, I2C_No_StartStop);

    while (DataNum != (*NumByteToWrite))
    {
        /* Wait until TXIS flag is set */
        sEETimeout = sEE_LONG_TIMEOUT;
        while (I2C_GetFlagStatus(I2C1, I2C_ISR_TXIS) == RESET)
        {
            if ((sEETimeout--) == 0) return 3;
        }

        /* Write data to TXDR */
        I2C_SendData(I2C1, (uint8_t)(pBuffer[DataNum]));

        /* Update number of transmitted data */
        DataNum++;
    }

    /* Wait until STOPF flag is set */
    sEETimeout = sEE_LONG_TIMEOUT;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_STOPF) == RESET)
    {
        if ((sEETimeout--) == 0) return 4;
    }

    /* Clear STOPF flag */
    I2C_ClearFlag(I2C1, I2C_ICR_STOPCF);

    /* If all operations OK, return sEE_OK (0) */
    return sEE_OK;
}

/**
  * @brief  sEE_ReadBuffer program.
  * @param  *pBuffer  ReadAddr *NumByteToRead
  * @retval None
  */
uint32_t sEE_ReadBuffer(uint8_t *pBuffer, uint16_t ReadAddr, uint16_t *NumByteToRead)
{
    uint32_t NumbOfSingle = 0, DataNum = 0;

    
    NumbOfSingle = (*NumByteToRead) % 255;
    /* Configure slave address, nbytes, reload and generate start */
    I2C_TransferHandling(I2C1, DeviceAdr, 1, I2C_SoftEnd_Mode, I2C_Generate_Start_Write);

    /* Wait until TXIS flag is set */
    sEETimeout = sEE_LONG_TIMEOUT;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_TXIS) == RESET)
    {
        if ((sEETimeout--) == 0) return 1;
    }

    /* Send memory address */
    I2C_SendData(I2C1, (uint8_t)ReadAddr);
    /* Wait until TC flag is set */
    sEETimeout = sEE_LONG_TIMEOUT;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_TC) == RESET)
    {
        if ((sEETimeout--) == 0) return 2;
    }

    /* If number of Reload cycles is not equal to 0 */

        /* Update CR2 : set Slave Address , set read request, generate Start and set end mode */
        I2C_TransferHandling(I2C1, DeviceAdr, (uint32_t)(NumbOfSingle), I2C_AutoEnd_Mode, I2C_Generate_Start_Read);

        /* Reset local variable */
        DataNum = 0;

        /* Wait until all data are received */
        while (DataNum != NumbOfSingle)
        {
            /* Wait until RXNE flag is set */
            sEETimeout = sEE_LONG_TIMEOUT;
            while (I2C_GetFlagStatus(I2C1, I2C_ISR_RXNE) == RESET)
            {
                if ((sEETimeout--) == 0) return 5;
            }

            /* Read data from RXDR */
            pBuffer[DataNum] = I2C_ReceiveData(I2C1);

            /* Update number of received data */
            DataNum++;
            (*NumByteToRead)--;
        }


    /* Wait until STOPF flag is set */
    sEETimeout = sEE_LONG_TIMEOUT;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_STOPF) == RESET)
    {
        if ((sEETimeout--) == 0) return 6;
    }

    /* Clear STOPF flag */
    I2C_ClearFlag(I2C1, I2C_ICR_STOPCF);

    /* If all operations OK, return sEE_OK (0) */
    return sEE_OK;
}

/**********************************************************************************
  * @brief  Wait for EEPROM Standby state.
  *
  * @note  This function allows to wait and check that EEPROM has finished the
  *        last operation. It is mostly used after Write operation: after receiving
  *        the buffer to be written, the EEPROM may need additional time to actually
  *        perform the write operation. During this time, it doesn't answer to
  *        I2C packets addressed to it. Once the write operation is complete
  *        the EEPROM responds to its address.
  *
  * @param  None
  * @note
  * @retval sEE_OK (0) if operation is correctly performed, else return value
  *         different from sEE_OK (0) or the timeout user callback.
  *********************************************************************************
*/
uint32_t sEE_WaitEepromStandbyState(void)
{
    __IO uint32_t sEETrials = 0;

    /* Keep looping till the slave acknowledge his address or maximum number
    of trials is reached (this number is defined by sEE_MAX_TRIALS_NUMBER define
    in FT32373c_eval_i2c_ee.h file) */

    /* Configure CR2 register : set Slave Address and end mode */
    I2C_TransferHandling(I2C1, DeviceAdr, 0, I2C_AutoEnd_Mode, I2C_No_StartStop);

    do
    {
        /* Initialize sEETimeout */
        sEETimeout = sEE_FLAG_TIMEOUT;

        /* Clear NACKF */
        I2C_ClearFlag(I2C1, I2C_ICR_NACKCF | I2C_ICR_STOPCF);

        /* Generate start */
        I2C_GenerateSTART(I2C1, ENABLE);

        /* Wait until timeout elapsed */
        while (sEETimeout-- != 0);

        /* Check if the maximum allowed number of trials has bee reached */
        if (sEETrials++ == sEE_MAX_TRIALS_NUMBER)
        {
            /* If the maximum number of trials has been reached, exit the function */
            return 1;
        }
    }
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_NACKF) != RESET);

    /* Clear STOPF */
    I2C_ClearFlag(I2C1, I2C_ICR_STOPCF);

    /* Return sEE_OK if device is ready */
    return sEE_OK;
}

/**
  * @brief  Delay program.
  * @param  us :0~0xFFFFFFFF
  * @retval None
  */
void Delay(uint32_t us)
{
    uint32_t i, j;
    for (i = us; i > 0; i--)
    {
        for (j = 0; j < 1000; j++)
        {
            __NOP();
        }
    }
}
/* Private function ------ -------------------------------------------------------*/
/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
