/**
    ******************************************************************************
    * @file         bsp_i2c.c
    * @author       FMD AE
    * @brief        bsp_i2c program body
    * @version      V1.0.0
    * @data         2022-05-30
    ******************************************************************************
    * @attention
    * COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
    * This software is provided by the copyright holders and contributors,and the
    *   software is believed to be accurate and reliable. However, Fremont Micro
    *   Devices (SZ) Corporation assumes no responsibility for the consequences of
    *   use of such software or for any infringement of patents of other rights
    *   of third parties, which may result from its use. No license is granted by
    *   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
    *   Corporation.
    ******************************************************************************
    */

/* Includes ------------------------------------------------------------------*/
#include "bsp_i2c.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
uint8_t send_buff[256] = {0x88, 0xaa, 0x55, 0x00, 0x69, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x9, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F
                           , 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F
													  , 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF};

uint8_t Recev_buff[100] = {0xff, 0xaa, 0x55, 0x00, 0x69};
uint8_t receive_buff[256];
uint16_t recev_cnt = 0;
uint16_t I2C_SEND_SIZE = 100;
uint16_t match_addr = 0;
uint8_t smbus_mode = 0;
uint8_t CurrentAdrSlaveDev = 0;
uint8_t I2C_DIR,senddata=0;

/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/
/**
  * @brief  Bsp_I2C_INTERRUPT_Mode_Config program.
  * @param  None
  * @retval None
  */
void Bsp_I2C_INTERRUPT_Mode_Config(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    I2C_InitTypeDef I2C_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

    /*I2C1:SCL:PB8  SDA:PB9   AF1*/
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C2, ENABLE);

    /* Configure SYSCLK as the clock of IC21 */
    RCC_I2CCLKConfig(RCC_I2C1CLK_SYSCLK);

    /*GPIO Init*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_1);
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_1);

    I2C_DeInit(I2C1);
    I2C_StructInit(&I2C_InitStructure);
    /*Tclk = 8MHz,Timing is set to 0x3139*/
    I2C_InitStructure.I2C_Timing = 0x3139;
    I2C_Init(I2C1, &I2C_InitStructure);

    I2C_Cmd(I2C1, ENABLE);

    /*I2C2:SCL:PB10     SDA:PB11 */
    /*GPIO Init*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_PinAFConfig(GPIOB, GPIO_PinSource10, GPIO_AF_1);
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource11, GPIO_AF_1);

    I2C_DeInit(I2C2);

    /* Disable clock extension */
    I2C_StretchClockCmd(I2C2, DISABLE);

    I2C_ITConfig(I2C2, I2C_IT_RXI, ENABLE);
		I2C_ITConfig(I2C2, I2C_IT_ADDRI, ENABLE);
		I2C_ITConfig(I2C2, I2C_IT_TXI, ENABLE);
		I2C_ITConfig(I2C2, I2C_IT_STOPI, ENABLE);


    I2C_StructInit(&I2C_InitStructure);
    /*Tclk = 8MHz,Timing is set to 0x3139*/
    I2C_InitStructure.I2C_Timing = 0x3139;
    I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
    I2C_InitStructure.I2C_OwnAddress1 = SLAVE_DEVICE_ADDR;
    I2C_Init(I2C2, &I2C_InitStructure);
		
		//config OAR2 for slave device
		I2C_OwnAddress2Config(I2C2, SLAVE_DEVICE_ADDR2, I2C_OA2_NoMask);
		I2C_DualAddressCmd(I2C2, ENABLE);

    NVIC_InitStructure.NVIC_IRQChannel = I2C2_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    I2C_Cmd(I2C2, ENABLE);

    while (1)
    {
				CurrentAdrSlaveDev = (((uint32_t)I2C2->ISR & I2C_ISR_ADDCODE) >> 16);
				I2C_DIR = (((uint32_t)I2C2->ISR & I2C_ISR_DIR) >> 16);
    }
}
/**
  * @brief  I2C_Send_MutiByte program.
  * @param  None
  * @retval None
  */
void I2C_Send_MutiByte(uint8_t addr, uint8_t *dat, uint8_t len)
{
    /* Configure the slave IP address : addr,Number of bytes sent: len. Direction: Write data */
    I2C_SlaveAddressConfig(I2C1, addr);
    I2C_NumberOfBytesConfig(I2C1, len);
    I2C_MasterRequestConfig(I2C1, I2C_Direction_Transmitter);

    /* Generate start signal */
    I2C_GenerateSTART(I2C1, ENABLE);

    for (uint8_t i = 0; i < len; i++)
    {
        /* Send is empty? */
        while (I2C_GetFlagStatus(I2C1, I2C_FLAG_TXIS) != SET);

        I2C_SendData(I2C1, dat[i]);
    }
		
		for(uint8_t i = 0; i < len; i++)//reading
    {
			/* Wait until RXNE flag is set */
     
      while (I2C_GetFlagStatus(I2C1, I2C_ISR_RXNE) == RESET)
      {
				;
      }
			Recev_buff[i] = I2C_ReceiveData(I2C1);
		}
		
		while (I2C_GetFlagStatus(I2C1, I2C_ISR_STOPF) == RESET)
    {
        ;
    }

    /* Clear STOPF flag */
    I2C_ClearFlag(I2C1, I2C_ICR_STOPCF);
		
}

/**
  * @brief  Delay program.
  * @param  us :0~0xFFFFFFFF
  * @retval None
  */
void Delay(uint32_t us)
{
    uint32_t i, j;
    for (i = us; i > 0; i--)
    {
        for (j = 0; j < 1000; j++)
        {
            __NOP();
        }
    }
}
/* Private function ------ -------------------------------------------------------*/
/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
