/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_iwdg.c
* @author    : FMD AE
* @brief     : bsp_iwdg program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "bsp_iwdg.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/

/**
  * @brief  IWDG_Config program.
  * @param  None
  * @retval None
  */
void IWDG_Config(void)
{
    /********************* IWDG configuration ***********************************/
    /* Enable IWDG, set reload and window values to 2000 and 1000, respectively.
     In this case the refresh window is:
     (2000-1000) / (LSI/16) = ~400 ms < refresh window < 2000 / (LSI/16) = ~800 ms

        Note: the refresh window may varies due to LSI frequency dispersion
        ----
     */

    /* Enable IWDG (the LSI oscillator will be enabled by hardware) */
    IWDG_Enable();

    /* Enable write access to IWDG_PR and IWDG_RLR registers */
    IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);

    /* Set IWDG Prescaler value to 16 */
    IWDG_SetPrescaler(IWDG_Prescaler_16);

    /* Set IWDG Reload value to 2000 */
    IWDG_SetReload(2000);

    /* Wait until RVU flag is reset to be sure that the reload value
     update operation is completed */
    while (IWDG_GetFlagStatus(IWDG_FLAG_RVU) != RESET);

    /* Set the IWDG window value to 1000 */
    IWDG_SetWindowValue(1000);
}
/* Private function ------ -------------------------------------------------------*/
/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
