/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : main program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/

/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
static void  Buttons_Initialize(void);

/* Public function ------ --------------------------------------------------------*/
/* Private function ------ -------------------------------------------------------*/
/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
    /* Init LEDs and button */
    LedConfig(LED1);
    LedConfig(LED2);
    LedConfig(LED3);
    Buttons_Initialize();

    /* If the system generates IWDG reset event ,light up LED1.*/
    if (RCC_GetFlagStatus(RCC_FLAG_IWDGRST) != RESET)
    {
        LedOn(LED1);

        RCC_ClearFlag();
    }
    else
    {
        LedOff(LED1);
    }

    /* Setup SysTick Timer for 1 msec interrupts. */
    SystickInit();

    /* IWDG config with window mode */
    IWDG_Config();

    while (1)
    {
        /* Toggle LED2 */
        LedToggle(LED2);

        /* Insert 750 ms delay */
        Delay(750);
   

        /* Reload IWDG counter */
        IWDG_ReloadCounter();
    }

}

/**
  * @brief  Buttons_Initialize program.
  * @param  None
  * @retval None
  */
static void  Buttons_Initialize(void)
{
    GPIO_InitTypeDef    GPIO_InitStructure;
    EXTI_InitTypeDef    EXTI_InitStructure;
    NVIC_InitTypeDef    NVIC_InitStructure;

    /* BUTTONS GPIO Periph clock enable */
    RCC_AHBPeriphClockCmd(RCC_AHBENR_GPIOCEN, ENABLE);

    /* Configure BUTTON1_GPIO_Pin in input pull down mode */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;

    GPIO_Init(GPIOC, &GPIO_InitStructure);

    /* SYSCFG Config */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
    SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOC, EXTI_PinSource13);

    /* EXTI INIT */
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_InitStructure.EXTI_Line = EXTI_Line13;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;

    EXTI_Init(&EXTI_InitStructure);

    /* NVIC Init */
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannel = EXTI4_15_IRQn;
    NVIC_Init(&NVIC_InitStructure);
}

/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
