/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : main program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "main.h"


__IO uint32_t LowPowerMode = 0;

/* Private function prototypes -----------------------------------------------*/
void Delay(__IO uint32_t nCount);


/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
    /*!< At this stage the microcontroller clock setting is already configured,
       this is done through SystemInit() function which is called from startup
       file (startup_ft32f0xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_ft32f0xx.c file
     */

    LedConfig(LED2);
    EXTI4_15_Config();

    while (1)
    {
        if (LowPowerMode == 1)
        {
            /* Request to enter WFI mode */
            __WFI();
            LowPowerMode = 0;
        }

        Delay(0xFFFFF);
        LedToggle(LED2);
    }
}


/**
  * @brief  Inserts a delay time.
  * @param  nCount: specifies the delay time length.
  * @retval None
  */
void Delay(__IO uint32_t nCount)
{
    for (; nCount != 0; nCount--);
}
/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
