/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_PVD.c
* @author    : FMD AE
* @brief     : bsp_PVD program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "bsp_PVD.h"

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/


/**
  * @brief  Configure IO .
  * @param  None
  *
  *
  *
  *
  *
  * @retval None
  */

void Test_IO_Config(void)
{
	GPIO_InitTypeDef GPIO_InitStruct;
	
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA,ENABLE);
	
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_5;
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
	
	GPIO_Init(GPIOA,&GPIO_InitStruct);	
}

/**
  * @brief  Set PA5 output pulse wave.
  * @param  None
  *
  *
  *
  *
  *
  * @retval None
  */
void PA5_Test_Out(uint8_t num)
{
	Test_IO_Config();
	
	for(uint8_t i=0;i<num;i++)
	{
		GPIOA->BRR = GPIO_Pin_5;
		GPIOA->BSRR = GPIO_Pin_5;
		GPIOA->BRR = GPIO_Pin_5;
	}
}

/**
  * @brief  Configure PVD .
  * @param  None
  *
  *
  *
  *
  *
  * @retval None
  */
void Bsp_PWR_PVD_Test(void)
{
	EXTI_InitTypeDef EXTI_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;
	
	/* Enable PWR APB1 Clock */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
	
	PA5_Test_Out(1);
	
	PWR_DeInit();
	
	PWR_PVDLevelConfig(PWR_PVDLevel_15);
	
	EXTI_InitStructure.EXTI_Line = EXTI_Line16;
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
	EXTI_Init(&EXTI_InitStructure);
	
	NVIC_InitStructure.NVIC_IRQChannel = PVD_VDDIO2_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	
	/*Turn off SysTick*/
   SysTick->CTRL  &= ~SysTick_CTRL_ENABLE_Msk;                    /* Disable SysTick IRQ and SysTick Timer */

	PWR_PVDCmd(ENABLE);
	
	PWR_EnterSleepMode(PWR_SLEEPEntry_WFI);
	
	/*Turn on SysTick*/
   SysTick->CTRL  |= SysTick_CTRL_ENABLE_Msk;                    /* Enable SysTick IRQ and SysTick Timer */
	

	for(uint8_t i=0;i<2;i++)
	{
		GPIOA->BRR = GPIO_Pin_5;
		GPIOA->BSRR = GPIO_Pin_5;
		GPIOA->BRR = GPIO_Pin_5;
	}
	
	while(1)
	{
	
	}
	
}
