/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_comp.c
* @author    : FMD AE
* @brief     : bsp_comp program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/

#include "bsp_comp.h"

/**
  * @brief  Configure COMP1 
    * @note  Compare the voltage and output the result to 
  * @param  None
  *
  *
  * @retval None
  */
void Bsp_COMP_Config(void)
{
    COMP_InitTypeDef COMP_InitStruct = {0};
    GPIO_InitTypeDef GPIO_InitStruct = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB, ENABLE);

    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AN;
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_1;//

    GPIO_Init(GPIOA, &GPIO_InitStruct);

    /*configure compare1 */
    COMP_InitStruct.COMP_VipSel = NCOMP_VIP_SEL_PAD_PA1;
    COMP_InitStruct.COMP_VinSel = NCOMP_VIN_SEL_PAD_PA0;
    COMP_InitStruct.COMP_OutputSel = COMP_OUTPUT_NO_SELECTION;
    COMP_InitStruct.COMP_Pol = NCOMP_POL_NOT_INVERT;
    COMP_Init(COMP_Selection_COMP1, &COMP_InitStruct);

    COMP_WindowCmd(ENABLE);
    /*compare1 enable*/
    COMP_Cmd(COMP_Selection_COMP1, ENABLE);

    COMP_LockConfig(COMP_Selection_COMP1);
}

/**
  * @brief  reset for COMP1 
    * @note
  * @param  None
  *
  *
  * @retval None
  */
void Bsp_COMP_ResetConfig(void)
{
    COMP_InitTypeDef COMP_InitStruct = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

    COMP_DeInit();
    COMP_StructInit(&COMP_InitStruct);
}
/**
  * @brief  NVIC_Configuration 
    * @note  enable compare interrupt 
  * @param  None
  *
  *
  * @retval None
  */
void NVIC_Configuration(void)
{
    NVIC_InitTypeDef NVIC_InitStructure;
    EXTI_InitTypeDef    EXTI_InitStructure;

    /* Configure EXTI21 line */
    EXTI_InitStructure.EXTI_Line = EXTI_Line21;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    /* COMP IRQ Channel configuration */
    NVIC_InitStructure.NVIC_IRQChannel = ADC1_COMP_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 0x00;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
}
/**
  * @brief Bsp_COMP  
    * @note configueration compare 1 
  * @param  None
  *
  *
  * @retval None
  */
void Bsp_COMP(void)
{
    Bsp_COMP_ResetConfig();
    NVIC_Configuration();
    Bsp_COMP_Config();
}

