/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_rtc.c
* @author    : FMD AE
* @brief     : bsp_rtc program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "bsp_rtc.h"
#include "bsp_led.h"

//#define LSE //extertal crystal oscillator 32.768KHz

extern uint16_t LedCnt;

uint32_t sys_ccfgr1 = 0;
uint32_t sys_ccfgr2 = 0;
uint32_t sys_ccfgr3 = 0;
uint32_t save_Latency = 0;

static void Set_Sysclk_48M(void);
static void User_Sysclk_Recover(void);

/**
  * @brief  delay time function.
  * @param  cnt  delay cnt ms
  * @retval None
  */
void DelaymS(uint16_t cnt)
{
    LedCnt = 0;
    while (1)
    {
        if (LedCnt > cnt)
            break;
    }
}

/**
  * @brief  RTC NVIC Config
  * @param  NewState  Enable or Disabble
  * @retval None
  */
void RTC_NVIC_Config(FunctionalState NewState)
{
    NVIC_InitTypeDef NVIC_InitStruct;

    NVIC_InitStruct.NVIC_IRQChannel = RTC_IRQn;
    NVIC_InitStruct.NVIC_IRQChannelPriority = 0;
    NVIC_InitStruct.NVIC_IRQChannelCmd = NewState;
    NVIC_Init(&NVIC_InitStruct);
}

/**
  * @brief  config RTC Alarm wake stop
  * @param  None
  * @retval None
  */
void Bsp_RTC_ALARM_Config(void)
{
    RTC_InitTypeDef RTC_InitStruct;
    RTC_TimeTypeDef RTC_TimeStruct;
    RTC_DateTypeDef RTC_DateStruct;
    RTC_AlarmTypeDef RTC_AlarmStruct;
    EXTI_InitTypeDef EXTI_InitStruct;

    /* If the system clock is greater than 48MHz, the following 
    functions must be called, otherwise, you can not call. */
    Set_Sysclk_48M();
    
    /* PWR clock enable */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);

    PWR_BackupAccessCmd(ENABLE);
#ifdef LSE
    RCC_LSEConfig(RCC_LSE_ON);
    while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET);
    RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);
#else
    RCC_LSICmd(ENABLE);
    while (RCC_GetFlagStatus(RCC_FLAG_LSIRDY) == RESET);
    RCC_RTCCLKConfig(RCC_RTCCLKSource_LSI);
#endif

    RCC_RTCCLKCmd(ENABLE);

    RTC_DeInit();

    //RTC register write protection test
    RTC_WriteProtectionCmd(ENABLE);
    RTC->TR = (uint32_t)0x00000001;

    //calendar value source
    RTC_BypassShadowCmd(DISABLE);

    RTC_WaitForSynchro();

    RTC_StructInit(&RTC_InitStruct);

    RTC_InitStruct.RTC_HourFormat = RTC_HourFormat_24;
#ifdef LSE
    RTC_InitStruct.RTC_AsynchPrediv = 127;
    RTC_InitStruct.RTC_SynchPrediv = 255;
#else
    RTC_InitStruct.RTC_AsynchPrediv = 99;
    RTC_InitStruct.RTC_SynchPrediv = 399;
#endif

    RTC_Init(&RTC_InitStruct);

    //set an alarm 05:20:05
    RTC_AlarmStruct.RTC_AlarmDateWeekDay = 3;
    RTC_AlarmStruct.RTC_AlarmDateWeekDaySel = RTC_AlarmDateWeekDaySel_WeekDay;
    RTC_AlarmStruct.RTC_AlarmMask = RTC_AlarmMask_None;
    RTC_AlarmStruct.RTC_AlarmTime.RTC_H12 = RTC_H12_AM;
    RTC_AlarmStruct.RTC_AlarmTime.RTC_Hours = 5;
    RTC_AlarmStruct.RTC_AlarmTime.RTC_Minutes = 20;
    RTC_AlarmStruct.RTC_AlarmTime.RTC_Seconds = 5;
    RTC_SetAlarm(RTC_Format_BIN, RTC_Alarm_A, &RTC_AlarmStruct);

    RTC_AlarmStructInit(&RTC_AlarmStruct);
    RTC_GetAlarm(RTC_Format_BIN, RTC_Alarm_A, &RTC_AlarmStruct);

    RTC_AlarmSubSecondConfig(RTC_Alarm_A, 520, RTC_AlarmSubSecondMask_SS14_1);
    while (RTC_GetAlarmSubSecond(RTC_Alarm_A) != 520);

    RTC_AlarmCmd(RTC_Alarm_A, ENABLE);

    //PC13 output alarm status Generate a high-level pulse at the set alarm time
    RTC_OutputConfig(RTC_Output_AlarmA, RTC_OutputPolarity_High);
    RTC_OutputTypeConfig(RTC_OutputType_PushPull);


    //Alarm interrupt configure
    RTC_ClearITPendingBit(RTC_IT_ALRA);
    RTC_ITConfig(RTC_IT_ALRA, ENABLE);

    EXTI_InitStruct.EXTI_Line = EXTI_Line17;
    EXTI_InitStruct.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStruct.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_InitStruct.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStruct);

    RTC_NVIC_Config(ENABLE);

    RTC_TimeStructInit(&RTC_TimeStruct);
    //Set time 05:20:00
    RTC_TimeStruct.RTC_H12 = RTC_H12_AM;
    RTC_TimeStruct.RTC_Hours = 5;
    RTC_TimeStruct.RTC_Minutes = 20;
    RTC_TimeStruct.RTC_Seconds = 0;
    while (RTC_SetTime(RTC_Format_BIN, &RTC_TimeStruct) != SUCCESS);

    RTC_DateStructInit(&RTC_DateStruct);
    //Set date：XX20-05-20 wednesday
    RTC_DateStruct.RTC_Year = 20;
    RTC_DateStruct.RTC_Month = 5;
    RTC_DateStruct.RTC_Date = 20;
    RTC_DateStruct.RTC_WeekDay = 3;
    while (RTC_SetDate(RTC_Format_BIN, &RTC_DateStruct) != SUCCESS);

    /* If the Set_Sysclk_48M function was called at the top and the user's own clock configuration needs 
    to be restored, the user's own clock configuration function is called at the bottom. Here is the 
    default configuration function for the system clock. */
    User_Sysclk_Recover();
}

/**
  * @brief  RTC Alarm wake stop process
  * @param  None
  * @retval None
  */
void  Bsp_RTC_ALARM_Process(void)
{
    RTC_TimeTypeDef RTC_TimeStruct;
    RTC_DateTypeDef RTC_DateStruct;

    while (1)
    {
        SETLED4(LED_OFF);
        RTC_GetDate(RTC_Format_BIN, &RTC_DateStruct);
        RTC_GetTime(RTC_Format_BIN, &RTC_TimeStruct);

        //Stop mode
        PWR_EnterSTOPMode(PWR_Regulator_LowPower, PWR_STOPEntry_WFI);

        SystemInit();
        
        SETLED4(LED_ON);

        DelaymS(500);

    }
}


/**
  * @brief  Set_Sysclk_48M
  * @param  None
  * @retval None
  */
static void Set_Sysclk_48M(void)
{
    __IO uint32_t StartUpCounter = 0, HSIStatus = 0;

    /* Save some information */
    sys_ccfgr1 = RCC->CFGR;
    sys_ccfgr2 = RCC->CFGR4;
    sys_ccfgr3 = RCC->CFGR2;
    save_Latency = FLASH->ACR & 0x0000000F;
    
    /* Set HSION bit */
    RCC->CR |= (uint32_t)0x00000001;

    /* Reset SW[1:0], HPRE[3:0], PPRE[2:0], ADCPRE and MCOSEL[2:0] bits */
    RCC->CFGR &= (uint32_t)0xF8FFB80C;
    
    /* Reset SW[2] */
    RCC->CFGR4 = 0;
    
    /* Reset HSEON, CSSON and PLLON bits */
    RCC->CR &= (uint32_t)0xFEF6FFFF;

    /* Reset HSEBYP bit */
    RCC->CR &= (uint32_t)0xFFFBFFFF;

    /* Reset PLLSRC, PLLXTPRE and PLLMUL[3:0] bits */
    RCC->CFGR &= (uint32_t)0xFFC0FFFF;

    /* Reset PREDIV1[3:0] bits */
    RCC->CFGR2 &= (uint32_t)0xFFFFFFF0;

    /* Reset USARTSW[1:0], I2CSW, CECSW and ADCSW bits */
    RCC->CFGR3 &= (uint32_t)0xFFFFFEAC;

    /* Reset HSI14 bit */
    RCC->CR2 &= (uint32_t)0xFFFFFFFE;

    /* Disable all interrupts */
    RCC->CIR = 0x00000000;
    
    /* SYSCLK, HCLK, PCLK configuration ----------------------------------------*/
    /* Wait till HSI is ready and if Time out is reached exit */
    do
    {
        HSIStatus = RCC->CR & RCC_CR_HSIRDY;
        StartUpCounter++;  
    } while((HSIStatus == 0) && (StartUpCounter != HSI_STARTUP_TIMEOUT));

    if ((RCC->CR & RCC_CR_HSIRDY) != RESET)
    {
        HSIStatus = (uint32_t)0x01;
    }
    else
    {
        HSIStatus = (uint32_t)0x00;
    }  

    if (HSIStatus == (uint32_t)0x01)
    {
        /* Enable Prefetch Buffer and set Flash Latency */
        FLASH->ACR = FLASH_ACR_PRFTBE | ((uint32_t)0x00000001);

        /* HCLK = SYSCLK */
        RCC->CFGR |= (uint32_t)RCC_CFGR_HPRE_DIV1;
          
        /* PCLK = HCLK */
        RCC->CFGR |= (uint32_t)RCC_CFGR_PPRE_DIV1;

        /* PLL configuration */
        RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_PLLSRC | RCC_CFGR_PLLXTPRE | RCC_CFGR_PLLMULL));
        RCC->CFGR |= (uint32_t)(RCC_CFGR_PLLSRC_HSI_PREDIV | RCC_CFGR_PLLXTPRE_PREDIV1 | RCC_CFGR_PLLMULL6);
                
        /* Enable PLL */
        RCC->CR |= RCC_CR_PLLON;

        /* Wait till PLL is ready */
        while((RCC->CR & RCC_CR_PLLRDY) == 0)
        {
        }

        /* Select PLL as system clock source */
        RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_SW));
        RCC->CFGR |= (uint32_t)RCC_CFGR_SW_PLL;    

        /* Wait till PLL is used as system clock source */
        while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != (uint32_t)RCC_CFGR_SWS_PLL)
        {
        }
    }
    else
    { /* If HSI fails to start-up, the application will have wrong clock 
         configuration. User can add here some code to deal with this error */	  
    }
}

/**
  * @brief  User_Sysclk_Recover
  * @param  None
  * @retval None
  */
static void User_Sysclk_Recover(void)
{
    __IO uint32_t StartUpCounter = 0, HSIStatus = 0;
    __IO uint32_t HSEStatus = 0;
    
    /* Set HSION bit */
    RCC->CR |= (uint32_t)0x00000001;

    /* Reset SW[1:0], HPRE[3:0], PPRE[2:0], ADCPRE and MCOSEL[2:0] bits */
    RCC->CFGR &= (uint32_t)0xF8FFB80C;
    
    /* Reset SW[2] */
    RCC->CFGR4 = 0;
    
    /* Reset HSEON, CSSON and PLLON bits */
    RCC->CR &= (uint32_t)0xFEF6FFFF;

    /* Reset HSEBYP bit */
    RCC->CR &= (uint32_t)0xFFFBFFFF;

    /* Reset PLLSRC, PLLXTPRE and PLLMUL[3:0] bits */
    RCC->CFGR &= (uint32_t)0xFFC0FFFF;

    /* Reset PREDIV1[3:0] bits */
    RCC->CFGR2 &= (uint32_t)0xFFFFFFF0;

    /* Reset USARTSW[1:0], I2CSW, CECSW and ADCSW bits */
    RCC->CFGR3 &= (uint32_t)0xFFFFFEAC;

    /* Reset HSI14 bit */
    RCC->CR2 &= (uint32_t)0xFFFFFFFE;

    /* Disable all interrupts */
    RCC->CIR = 0x00000000;
    
    /* Start HCLK configuration */
    RCC_HCLKConfig(sys_ccfgr1 & 0x000000F0);
    /* End HCLK configuration */
    
    /* Start PCLK configuration */
    RCC_PCLKConfig(sys_ccfgr1 & 0x00000700);
    /* End PCLK configuration */
    
    /* Sysclk config */
    /* SWS[2] */
    if (sys_ccfgr2 & 0x00000002)
    {
        /* Enable Prefetch Buffer and set Flash Latency */
        FLASH->ACR = FLASH_ACR_PRFTBE | ((uint32_t)0x00000000);
        
        /* Enable HSI14 */
        RCC_HSI14Cmd(ENABLE);
        
        /* Wait till HSI14 is ready and if Time out is reached exit */
        do
        {
            StartUpCounter++;  
        } while((RCC_GetFlagStatus(RCC_FLAG_HSI14RDY) == RESET) && (StartUpCounter != HSE_STARTUP_TIMEOUT));

        if (RCC_GetFlagStatus(RCC_FLAG_HSI14RDY) != RESET)
        {
            /* HSI14 as sysclk */
            RCC->CFGR4 = 0x00000001;
            
            /* wait */
            while((RCC->CFGR4 & 0x00000002)!= 0x00000002)
            {
                ;
            }
        }   
    }
    else
    {
        /* SWS[1:0] */
        switch(sys_ccfgr1 & 0x0000000C)
        {
            //HSI
            case 0x00000000:
                /* SYSCLK configuration ----------------------------------------*/
                /* Enable Prefetch Buffer and set Flash Latency */
                FLASH->ACR = FLASH_ACR_PRFTBE | ((uint32_t)0x00000000);
                /* Enable HSI */             
                RCC_HSICmd(ENABLE);
            
                /* Wait till HSE is ready and if Time out is reached exit */
                do
                {
                    HSIStatus = RCC->CR & RCC_CR_HSIRDY;
                    StartUpCounter++;  
                } while((HSIStatus == 0) && (StartUpCounter != HSI_STARTUP_TIMEOUT));

                if ((RCC->CR & RCC_CR_HSIRDY) != RESET)
                {
                    RCC_SYSCLKConfig(RCC_SYSCLKSource_HSI);
                    
                    while(RCC_GetSYSCLKSource() != 0x00)
                    {
                        ;
                    }
                }
            break;
            //HSE
            case 0x00000004:
                /* SYSCLK configuration ----------------------------------------*/
                /* Enable Prefetch Buffer and set Flash Latency */
                FLASH->ACR = FLASH_ACR_PRFTBE | ((uint32_t)save_Latency);
            
                /* Enable HSE */    
                RCC_HSEConfig(RCC_HSE_ON);

                /* Wait till HSE is ready and if Time out is reached exit */
                do
                {
                    StartUpCounter++;  
                } while((RCC_GetFlagStatus(RCC_FLAG_HSERDY) == RESET) && (StartUpCounter != HSE_STARTUP_TIMEOUT));

                if (RCC_GetFlagStatus(RCC_FLAG_HSERDY) != RESET)
                {
                    RCC_SYSCLKConfig(RCC_SYSCLKSource_HSE);
                    
                    while(RCC_GetSYSCLKSource() != 0x04)
                    {
                        ;
                    }
                }                
            break;
            //PLL
            case 0x00000008:
                /* PLL SOURCE */
                switch(sys_ccfgr1 & 0x00018000)    
                {
                    /* HSI/2 */
                    case 0x00000000:
                        
                    /* HSI/PREDIV */
                    case 0x00008000:
                        /* Enable HSI */             
                        RCC->CR |= ((uint32_t)RCC_CR_HSION);

                        /* Wait till HSE is ready and if Time out is reached exit */
                        do
                        {
                            HSIStatus = RCC->CR & RCC_CR_HSIRDY;
                            StartUpCounter++;  
                        } while((HSIStatus == 0) && (StartUpCounter != HSI_STARTUP_TIMEOUT));

                        if ((RCC->CR & RCC_CR_HSIRDY) != RESET)
                        {
                            HSIStatus = (uint32_t)0x01;
                        }
                        else
                        {
                            HSIStatus = (uint32_t)0x00;
                        } 
                    break;
                    /* HSE/PREDIV */
                    case 0x00010000:
                        /* Enable HSE */    
                        RCC_HSEConfig(RCC_HSE_ON);

                        /* Wait till HSE is ready and if Time out is reached exit */
                        do
                        {
                            StartUpCounter++;  
                        } while((RCC_GetFlagStatus(RCC_FLAG_HSERDY) == RESET) && (StartUpCounter != HSE_STARTUP_TIMEOUT));

                        if (RCC_GetFlagStatus(RCC_FLAG_HSERDY) != RESET)
                        {
                            HSIStatus = (uint32_t)0x01;
                        }
                        else
                        {
                            HSIStatus = (uint32_t)0x00;
                        }       
                    break;
                    /* HSI48/PREDIV */
                    case 0x00018000:
                        /* Enable HSI48 */    
                        RCC_HSI48Cmd(ENABLE);

                        /* Wait till HSI48 is ready and if Time out is reached exit */
                        do
                        {
                            StartUpCounter++;  
                        } while((RCC_GetFlagStatus(RCC_FLAG_HSI48RDY) == RESET) && (StartUpCounter != HSE_STARTUP_TIMEOUT));

                        if (RCC_GetFlagStatus(RCC_FLAG_HSI48RDY) != RESET)
                        {
                            HSIStatus = (uint32_t)0x01;
                        }
                        else
                        {
                            HSIStatus = (uint32_t)0x00;
                        }    
                    break;
                }
                
                if(HSIStatus)
                {
                    /* Enable Prefetch Buffer and set Flash Latency */
                    FLASH->ACR = FLASH_ACR_PRFTBE | ((uint32_t)save_Latency);
                    
                    /* PLL configuration */
                    /* Clear PLL Source [16] and Multiplier [21:18] bits */
                    RCC->CFGR &= ~(RCC_CFGR_PLLMULL | RCC_CFGR_PLLSRC);
                    /* Set the PLL Source and Multiplier */
                    RCC->CFGR |= (uint32_t)(sys_ccfgr1 & 0x003F8000);
                    
                    RCC->CFGR2 = sys_ccfgr3;
                    
                    RCC_PLLCmd(ENABLE);
                    
                    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
                    
                    while(RCC_GetSYSCLKSource() != 0x08)
                    {
                        ;
                    }
                }
                else
                {
                
                }
            break;
            //HSI48
            case 0x0000000C:
                /* SYSCLK configuration ----------------------------------------*/
                /* Enable Prefetch Buffer and set Flash Latency */
                FLASH->ACR = FLASH_ACR_PRFTBE | ((uint32_t)0x00000001);
                
                /* Enable HSI48 */    
                RCC_HSI48Cmd(ENABLE);

                /* Wait till HSI48 is ready and if Time out is reached exit */
                do
                {
                    StartUpCounter++;  
                } while((RCC_GetFlagStatus(RCC_FLAG_HSI48RDY) == RESET) && (StartUpCounter != HSE_STARTUP_TIMEOUT));

                if (RCC_GetFlagStatus(RCC_FLAG_HSI48RDY) != RESET)
                {
                    RCC_SYSCLKConfig(RCC_SYSCLKSource_HSI48);
                    
                    while(RCC_GetSYSCLKSource() != 0x0C)
                    {
                        ;
                    }
                }     
            break;            
        }
    }
    /* End Sysclk config */
}






















