/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : FT25C16A.h
* @author    : FMD AE
* @brief     : Header for FT25C16A.c module
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __BSP_SPI_FT25C16A_H
#define __BSP_SPI_FT25C16A_H

/* Includes ------------------------------------------------------------------*/
#include "ft32f0xx.h"



/* Exported macro ------------------------------------------------------------*/

/* Exported functions ------------------------------------------------------- */

#define FLASH_SPIx SPI1

#define FLASH_SPI_APBxClock_FUN RCC_APB2PeriphClockCmd
#define FLASH_SPI_CLK RCC_APB2Periph_SPI1

//CS(NSS) PA4
#define FLASH_SPI_CS_AHBClock_FUN RCC_AHBPeriphClockCmd
#define FLASH_SPI_CS_CLK RCC_AHBPeriph_GPIOA
#define FLASH_SPI_CS_PORT GPIOA
#define FLASH_SPI_CS_PIN GPIO_Pin_4
#define SPI_CS_AF       GPIO_AF_0

//SCK PA5
#define FLASH_SPI_SCK_AHBClock_FUN RCC_AHBPeriphClockCmd
#define FLASH_SPI_SCK_CLK RCC_AHBPeriph_GPIOA
#define FLASH_SPI_SCK_PORT GPIOA
#define FLASH_SPI_SCK_PIN GPIO_Pin_5
#define SPI_SCK_AF       GPIO_AF_0
//MISO PA6
#define FLASH_SPI_MISO_AHBClock_FUN RCC_AHBPeriphClockCmd
#define FLASH_SPI_MISO_CLK RCC_AHBPeriph_GPIOA
#define FLASH_SPI_MISO_PORT GPIOA
#define FLASH_SPI_MISO_PIN GPIO_Pin_6
#define SPI_MISO_AF       GPIO_AF_0
//MOSI PA7
#define FLASH_SPI_MOSI_AHBClock_FUN RCC_AHBPeriphClockCmd
#define FLASH_SPI_MOSI_CLK RCC_AHBPeriph_GPIOA
#define FLASH_SPI_MOSI_PORT GPIOA
#define FLASH_SPI_MOSI_PIN GPIO_Pin_7
#define SPI_MOSI_AF       GPIO_AF_0

#define FLASH_SPI_CS_LOW() GPIO_ResetBits( FLASH_SPI_CS_PORT, FLASH_SPI_CS_PIN )
#define FLASH_SPI_CS_HIGH() GPIO_SetBits( FLASH_SPI_CS_PORT, FLASH_SPI_CS_PIN )


//COMMAND
#define W25X_WriteEnable 0x06
#define W25X_WriteDisable 0x04
#define W25X_ReadStatusReg 0x05
#define W25X_WriteStatusReg 0x01
#define W25X_ReadData 0x03
#define W25X_PageProgram 0x02

#define Dummy_Byte 0xFF

#define WIP_Flag 0x01

#define SPIT_FLAG_TIMEOUT   ((uint32_t)0x1000)

#define SPI_FLASH_PerWritePageSize  32

uint8_t SPI_FLASH_ReadByte(void);
uint8_t SPI_FLASH_SendByte(uint8_t byte);

void SPI_Config(void);
void SPI_FLASH_WriteEnable(void);
uint8_t SPI_FLASH_WaitForWriteEnd(void);

void SPI_FLASH_PageWrite(uint8_t *pBuffer, uint32_t WriteAddr, uint16_t NumByteToWrite);
void SPI_FLASH_BufferWrite(uint8_t *pBuffer, uint32_t WriteAddr, uint16_t NumByteToWrite);
void SPI_FLASH_BufferRead(uint8_t *pBuffer, uint32_t ReadAddr, uint16_t NumByteToRead);




#endif /* __BSP_SPI_FT25C16A_H */

/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
