/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : FT25C16A.c
* @author    : FMD AE
* @brief     : FT25C16A program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "FT25C16A.h"


/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/




uint32_t    SPITimeout;

/**
  * @brief  Configure SPI .
  * @param  None
  *
  *
  *
  *
  *
  * @retval None
  */


void SPI_Config(void)
{
    SPI_InitTypeDef SPI_InitStructure;
    GPIO_InitTypeDef GPIO_InitStructure;


    FLASH_SPI_APBxClock_FUN(FLASH_SPI_CLK, ENABLE);


    FLASH_SPI_CS_AHBClock_FUN(FLASH_SPI_CS_CLK | FLASH_SPI_SCK_CLK |
                              FLASH_SPI_MISO_CLK | FLASH_SPI_MOSI_CLK, ENABLE);

    /* SPI pin mappings */
    GPIO_PinAFConfig(FLASH_SPI_SCK_PORT, FLASH_SPI_SCK_PIN, SPI_SCK_AF);
    GPIO_PinAFConfig(FLASH_SPI_MISO_PORT, FLASH_SPI_MISO_PIN, SPI_MISO_AF);
    GPIO_PinAFConfig(FLASH_SPI_MOSI_PORT, FLASH_SPI_MOSI_PIN, SPI_MOSI_AF);



    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_3;

    GPIO_InitStructure.GPIO_Pin = FLASH_SPI_CS_PIN;
    GPIO_Init(FLASH_SPI_CS_PORT, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;

    GPIO_InitStructure.GPIO_Pin = FLASH_SPI_SCK_PIN;
    GPIO_Init(FLASH_SPI_SCK_PORT, &GPIO_InitStructure);


    GPIO_InitStructure.GPIO_Pin = FLASH_SPI_MISO_PIN;
    GPIO_Init(FLASH_SPI_MISO_PORT, &GPIO_InitStructure);


    GPIO_InitStructure.GPIO_Pin = FLASH_SPI_MOSI_PIN;
    GPIO_Init(FLASH_SPI_MOSI_PORT, &GPIO_InitStructure);



    FLASH_SPI_CS_HIGH();


    SPI_I2S_DeInit(SPI1);
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;


    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;//SPI_NSS_Soft;//SPI_NSS_Hard;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
//  SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_Init(SPI1, &SPI_InitStructure);


    /* Initialize the FIFO threshold */
    SPI_RxFIFOThresholdConfig(SPI1, SPI_RxFIFOThreshold_QF);

    /* enable SPI */
    SPI_Cmd(SPI1, ENABLE);
}

/**
  * @brief  SPI FLASH SENDBYTE .
  * @param  byte    it is the data sent
  *
  *
  *
  *
  *
  * @retval None
  */
uint8_t SPI_FLASH_SendByte(uint8_t byte)
{
    SPITimeout = SPIT_FLAG_TIMEOUT;

    //wait for SPI idle
    while (SPI_I2S_GetFlagStatus(FLASH_SPIx, SPI_I2S_FLAG_TXE) == RESET)
    {
        if ((SPITimeout--) == 0) return ERROR;
    }

    //send one byte
    SPI_SendData8(FLASH_SPIx, byte);
    SPITimeout = SPIT_FLAG_TIMEOUT;

    //wait for SPI idle
    while (SPI_I2S_GetFlagStatus(FLASH_SPIx, SPI_I2S_FLAG_RXNE) == RESET)
    {
        if ((SPITimeout--) == 0) return ERROR;
    }

    //read one byte
    return SPI_ReceiveData8(FLASH_SPIx);
}

/**
  * @brief  SPI FLASH ReadByte
  * @param  None
  *
  *
  *
  *
  *
  * @retval None
  */
uint8_t SPI_FLASH_ReadByte(void)
{
    return (SPI_FLASH_SendByte(Dummy_Byte));
}



/**
  * @brief  SPI FLASH WriteEnable
  * @param  None
  *
  *
  *
  *
  *
  * @retval None
  */

void SPI_FLASH_WriteEnable(void)
{
    //SPI CS Enable
    FLASH_SPI_CS_LOW();

    //Set Write Enable
    SPI_FLASH_SendByte(W25X_WriteEnable);

    //SPI CS Disable
    FLASH_SPI_CS_HIGH();
}

/**
  * @brief  SPI FLASH WaitForWriteEnd
  * @param  None
  *
  *
  *
  *
  *
  * @retval None
  */
uint8_t SPI_FLASH_WaitForWriteEnd(void)
{
    uint8_t FLASH_Status = 0;

    //SPI CS Enable
    FLASH_SPI_CS_LOW();
    //read status flag command
    SPI_FLASH_SendByte(W25X_ReadStatusReg);

    do
    {
        FLASH_Status = SPI_FLASH_SendByte(Dummy_Byte);
    }
    while ((FLASH_Status & WIP_Flag) == SET);

    //SPI CS Disable
    FLASH_SPI_CS_HIGH();

    return   FLASH_Status;
}

/**
  * @brief  SPI_FLASH PageWrite
  * @param  pBuffer  write bufffer
  * @param  WriteAddr  write address
  * @param  NumByteToRead  write data number
  *
  *
  *
  * @retval None
  */
void SPI_FLASH_PageWrite(uint8_t *pBuffer, uint32_t WriteAddr, uint16_t NumByteToWrite)
{
    //
    SPI_FLASH_WriteEnable();

    //SPI CS Enable
    FLASH_SPI_CS_LOW();
    //write data to memory array command
    SPI_FLASH_SendByte(W25X_PageProgram);
    //send write data address
    SPI_FLASH_SendByte((WriteAddr & 0xFF00) >> 8);
    SPI_FLASH_SendByte(WriteAddr & 0xFF);


    if (NumByteToWrite > SPI_FLASH_PerWritePageSize)
    {
        NumByteToWrite = SPI_FLASH_PerWritePageSize;
    }

    //write data
    while (NumByteToWrite--)
    {

        SPI_FLASH_SendByte(*pBuffer);
        pBuffer++;
    }

    //SPI CS Disable
    FLASH_SPI_CS_HIGH();
    SPI_FLASH_WaitForWriteEnd();
}

/**
  * @brief  SPI_FLASH_BufferWrite
  * @param  pBuffer  write bufffer
  * @param  WriteAddr  write address
  * @param  NumByteToRead  write data number
  *
  *
  *
  * @retval None
  */
void SPI_FLASH_BufferWrite(uint8_t *pBuffer, uint32_t WriteAddr, uint16_t NumByteToWrite)
{
    uint8_t NumOfPage = 0, NumOfSingle = 0, Addr = 0, count = 0, temp = 0;

    //
    Addr = WriteAddr % SPI_FLASH_PerWritePageSize;


    count = SPI_FLASH_PerWritePageSize - Addr;

    NumOfPage = NumByteToWrite / SPI_FLASH_PerWritePageSize;

    NumOfSingle = NumByteToWrite % SPI_FLASH_PerWritePageSize;


    if (Addr == 0)
    {
        /* NumByteToWrite < SPI_FLASH_PerWritePageSize */
        if (NumOfPage == 0)
        {
            SPI_FLASH_PageWrite(pBuffer, WriteAddr, NumByteToWrite);
        }
        else /* NumByteToWrite > SPI_FLASH_PageSize */
        {

            while (NumOfPage--)
            {
                SPI_FLASH_PageWrite(pBuffer, WriteAddr, SPI_FLASH_PerWritePageSize);
                WriteAddr += SPI_FLASH_PerWritePageSize;
                pBuffer += SPI_FLASH_PerWritePageSize;
            }
            if (NumOfSingle != 0)
            {

                SPI_FLASH_PageWrite(pBuffer, WriteAddr, NumOfSingle);
            }

        }
    }
    else
    {
        /* NumByteToWrite < SPI_FLASH_PerWritePageSize */
        if (NumOfPage == 0)
        {

            if (NumOfSingle > count)
            {
                temp = NumOfSingle - count;

                SPI_FLASH_PageWrite(pBuffer, WriteAddr, count);

                WriteAddr += count;
                pBuffer += count;

                SPI_FLASH_PageWrite(pBuffer, WriteAddr, temp);
            }
            else
            {
                SPI_FLASH_PageWrite(pBuffer, WriteAddr, NumByteToWrite);
            }
        }
        else /* NumByteToWrite > SPI_FLASH_PerWritePageSize */
        {

            NumByteToWrite -= count;
            NumOfPage = NumByteToWrite / SPI_FLASH_PerWritePageSize;
            NumOfSingle = NumByteToWrite % SPI_FLASH_PerWritePageSize;


            SPI_FLASH_PageWrite(pBuffer, WriteAddr, count);


            WriteAddr += count;
            pBuffer += count;

            while (NumOfPage--)
            {
                SPI_FLASH_PageWrite(pBuffer, WriteAddr, SPI_FLASH_PerWritePageSize);
                WriteAddr += SPI_FLASH_PerWritePageSize;
                pBuffer += SPI_FLASH_PerWritePageSize;
            }

            if (NumOfSingle != 0)
            {
                SPI_FLASH_PageWrite(pBuffer, WriteAddr, NumOfSingle);
            }
        }
    }
}

/**
  * @brief  SPI_FLASH_BufferRead
  * @param  pBuffer  read bufffer
  * @param  ReadAddr  read address
  * @param  NumByteToRead  read data number
  *
  *
  *
  * @retval None
  */
void SPI_FLASH_BufferRead(uint8_t *pBuffer, uint32_t ReadAddr, uint16_t NumByteToRead)
{
    //SPI CS Enable
    FLASH_SPI_CS_LOW();

    //read data command from memory array
    SPI_FLASH_SendByte(W25X_ReadData);

    //send the adress about reading data
    SPI_FLASH_SendByte((ReadAddr & 0xFF00) >> 8);
    SPI_FLASH_SendByte(ReadAddr & 0xFF);


    while (NumByteToRead--)
    {
        //read one byte data
        *pBuffer = SPI_FLASH_SendByte(Dummy_Byte);
        pBuffer++;
    }

    //SPI CS Disable
    FLASH_SPI_CS_HIGH();
}
