/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_spi.c
* @author    : FMD AE
* @brief     : bsp_spi program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "bsp_spi.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
const uint16_t spi1_tx_buff[RX_SIZE] =
{
    0x5588, 0x331e, 0x222d, 0x113c, 0xff4b,
    0xee5a, 0xdd69, 0xcc78, 0xbb87, 0xaa76,
    0x9985, 0x8894, 0x77a3, 0x66b2, 0x55c1,
    0x44d0, 0x33e1, 0x22ff, 0x11aa
};

const uint16_t spi2_tx_buff[RX_SIZE] =
{
    0x6598, 0x331e, 0x222d, 0x113c, 0xff4b,
    0xee5a, 0xdd69, 0xcc78, 0xbb87, 0xaa76,
    0x9985, 0x8894, 0x77a3, 0x66b2, 0x55c1,
    0x44d0, 0x33e1, 0x22ff, 0x11ab
};

/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
uint8_t data_addr = 0;
uint8_t rece_cnt2 = 0;
uint16_t spi1_rx_buff[RX_SIZE];
uint16_t spi2_rx_buff[RX_SIZE];

/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/
/**
  * @brief  Bsp_SPI1_GPIO_Config program.
  * @param  None
  * @retval None
  */
void Bsp_SPI1_GPIO_Config(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    /* Enable the clock of GPIOA */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    /* GPIO Init */
    GPIO_InitStructure.GPIO_Pin = SPI1_NSS | SPI1_SCK | SPI1_MISO | SPI1_MOSI;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(SPI1_PORT, &GPIO_InitStructure);

    /* Pin AF config */
    GPIO_PinAFConfig(SPI1_PORT, SPI1_NSS_PIN_SOURCE, SPI1_NSS_AF);
    GPIO_PinAFConfig(SPI1_PORT, SPI1_SCK_PIN_SOURCE, SPI1_SCK_AF);
    GPIO_PinAFConfig(SPI1_PORT, SPI1_MISO_PIN_SOURCE, SPI1_MISO_AF);
    GPIO_PinAFConfig(SPI1_PORT, SPI1_MOSI_PIN_SOURCE, SPI1_MOSI_AF);
}

/**
  * @brief  Bsp_SPI2_GPIO_Config program.
  * @param  None
  * @retval None
  */
void Bsp_SPI2_GPIO_Config(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    /* Enable the clock of GPIOB */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
    
    /* GPIO Init */
    GPIO_InitStructure.GPIO_Pin = SPI2_NSS | SPI2_SCK | SPI2_MISO | SPI2_MOSI;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(SPI2_PORT, &GPIO_InitStructure);

    /* Pin AF config */
    GPIO_PinAFConfig(SPI2_PORT, SPI2_NSS_PIN_SOURCE, SPI2_NSS_AF);
    GPIO_PinAFConfig(SPI2_PORT, SPI2_SCK_PIN_SOURCE, SPI2_SCK_AF);
    GPIO_PinAFConfig(SPI2_PORT, SPI2_MISO_PIN_SOURCE, SPI2_MISO_AF);
    GPIO_PinAFConfig(SPI2_PORT, SPI2_MOSI_PIN_SOURCE, SPI2_MOSI_AF);

}

/**
  * @brief  Bsp_SPI1_IT_Config program.
  * @param  None
  * @retval None
  */
void Bsp_SPI1_IT_Config(void)
{
    SPI_InitTypeDef SPI_InitStructure;

    /* Enable the clock of SPI1 */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);

    /* GPIO Init */
    Bsp_SPI1_GPIO_Config();

    /* SPI Init */
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Hard;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_Init(SPI1, &SPI_InitStructure);

    /* Set RxFIFO */
    SPI_RxFIFOThresholdConfig(SPI1, SPI_RxFIFOThreshold_QF);

    /* Output enable */
    SPI_SSOutputCmd(SPI1,ENABLE);
    
    /* SPI1 enable*/
    SPI_Cmd(SPI1, ENABLE);

    for (;;)
    {
        for (uint8_t i = 0; i < RX_SIZE; i++)
        {
            /* Waiting to send is empty */
            while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) != SET);
            SPI_SendData8(SPI1, spi1_tx_buff[i]);
            
            /* Wait for receive to be non-empty */
            while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) != SET);
            spi1_rx_buff[i] = SPI_ReceiveData8(SPI1);
        }

        /* Waiting to send FIFO is empty */
        while (SPI_GetTransmissionFIFOStatus(SPI1) != SPI_TransmissionFIFOStatus_Empty);

        /* Wait for receive FIFO is empty */
        while (SPI_GetReceptionFIFOStatus(SPI1) != SPI_ReceptionFIFOStatus_Empty);

        Delay(1000);

    }
}

/**
  * @brief  Bsp_SPI2_IT_Config program.
  * @param  None
  * @retval None
  */
void Bsp_SPI2_IT_Config(void)
{
    SPI_InitTypeDef SPI_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

    /* Enable the clock of SPI2 */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);

    /*SPI2 GPIO Init*/
    Bsp_SPI2_GPIO_Config();

    /*SPI2 Init*/
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Slave;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Hard;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7; 
    SPI_Init(SPI2, &SPI_InitStructure);
    
    /* Set RxFIFO */
    SPI_RxFIFOThresholdConfig(SPI2, SPI_RxFIFOThreshold_QF);

    /* Enable TX RX interrupt */
    SPI_I2S_ITConfig(SPI2, SPI_I2S_IT_TXE, ENABLE);
    SPI_I2S_ITConfig(SPI2, SPI_I2S_IT_RXNE, ENABLE);

    NVIC_InitStructure.NVIC_IRQChannel = SPI2_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    /* SPI1 enable*/
    SPI_Cmd(SPI2, ENABLE);
}

/**
  * @brief  Delay program.
  * @param  time:0~0xFFFFFFFF
  * @retval None
  */
void Delay(uint32_t time)
{
    uint32_t i = 0;

    for (i = 0; i < time; i++)
    {
        __NOP();
    }
}
/* Private function ------ -------------------------------------------------------*/
/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
