/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : main program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "main.h"

/* Private Constant --------------------------------------------------------------*/

/* Private typedef ---------------------------------------------------------------*/

/* Private define ----------------------------------------------------------------*/

/* Private variables -------------------------------------------------------------*/

/* Public variables --------------------------------------------------------------*/


/* Private function prototypes ---------------------------------------------------*/


/**********************************************************************************
  * @brief  main program.
  * @param  None
  * @note
  * @retval None
  *********************************************************************************
*/
int main(void)
{
    Led_Init();
    SysTick_Configuration();
    TIM_6Steps_Config();

    while (1)
    {
        LED_Toggle(LED_PORT, LED3_PIN);
				Delay_ms(500);
    }
}

/* Public function ------ --------------------------------------------------------*/

/******************************************************************************
  * @brief  SysTick_Configuration program.
  * @param  None
  * @retval None
  *****************************************************************************
*/
void SysTick_Configuration(void)
{
    if (SysTick_Config(SystemCoreClock / TICK_RATE_HZ))
    {
        while (1);
    }
    SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);
    NVIC_SetPriority(SysTick_IRQn, 0x00);
}
/******************************************************************************
  * @brief  Led_Init program.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void Led_Init(void)
{
    GPIO_InitTypeDef    GPIO_InitStruct;
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    GPIO_InitStruct.GPIO_Pin = LED3_PIN | LED4_PIN;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(LED_PORT, &GPIO_InitStruct);

    LED3_OFF;
    LED4_OFF;
}
/******************************************************************************
  * @brief  LED_Toggle program.
  * @param  GPIOx AND GPIO_Pin
  * @note
  * @retval None
  *****************************************************************************
*/
void LED_Toggle(GPIO_TypeDef *GPIOx, uint16_t GPIO_Pin)
{
    GPIO_WriteBit(GPIOx, GPIO_Pin, (BitAction)((1 - GPIO_ReadOutputDataBit(GPIOx, GPIO_Pin))));
}

/******************************************************************************
  * @brief  Delay_ms program.
  * @param  cnt
  * @note
  * @retval None
  *****************************************************************************
*/
void Delay_ms(uint16_t cnt)
{
	uint16_t i,k;
	for(i = 0; i < cnt; i++)
	{
		for(k = 0; k < 1000; k++)
		__NOP;
		
	}
}
/* Private function ------ -------------------------------------------------------*/

/************************ (C) COPYRIGHT FMD *****END OF FILE****/
