/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : ft32f0xx_it.c
* @author    : FMD AE
* @brief     : Main Interrupt Service Routines.
*              This file provides template for all exceptions handler and
*              peripherals interrupt service routine.
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "ft32f0xx_it.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/******************************************************************************/
/*            Cortex-M0 Processor Exceptions Handlers                         */
/******************************************************************************/

extern uint16_t IC3ReadValue1, IC3ReadValue2;
extern uint16_t TIM_CaptureNumber;
extern uint32_t Capture;
extern uint32_t TIM1Freq;

/**
  * @brief  This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
    /* Go to infinite loop when Hard Fault exception occurs */
    while (1)
    {
    }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles PendSVC exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
}


/**
  * @brief  This function handles PPP interrupt request.
  * @param  None
  * @retval None
  */
void TIM1_CC_IRQHandler(void)
{
    if (TIM_GetITStatus(TIM1, TIM_IT_CC2) == SET)
    {
        /* Clear TIM1 Capture compare interrupt pending bit */
        TIM_ClearITPendingBit(TIM1, TIM_IT_CC2);
        if (TIM_CaptureNumber == 0)
        {
            /* Get the Input Capture value */
            IC3ReadValue1 = TIM_GetCapture2(TIM1);
            TIM_CaptureNumber = 1;
        }
        else if (TIM_CaptureNumber == 1)
        {
            /* Get the Input Capture value */
            IC3ReadValue2 = TIM_GetCapture2(TIM1);

            /* Capture computation */
            if (IC3ReadValue2 > IC3ReadValue1)
            {
                Capture = (IC3ReadValue2 - IC3ReadValue1);
            }
            else if (IC3ReadValue2 < IC3ReadValue1)
            {
                Capture = ((0xFFFF - IC3ReadValue1) + IC3ReadValue2);
            }
            else
            {
                Capture = 0;
            }
            /* Frequency computation */
            TIM1Freq = (uint32_t) SystemCoreClock / Capture;
            TIM_CaptureNumber = 0;
        }
    }
    else if (TIM_GetITStatus(TIM1, TIM_IT_CC1) == SET)
    {
        /* Clear TIM1 Capture compare interrupt pending bit */
        TIM_ClearITPendingBit(TIM1, TIM_IT_CC1);
        if (TIM_CaptureNumber == 0)
        {
            /* Get the Input Capture value */
            IC3ReadValue1 = TIM_GetCapture1(TIM1);
            TIM_CaptureNumber = 1;
        }
        else if (TIM_CaptureNumber == 1)
        {
            /* Get the Input Capture value */
            IC3ReadValue2 = TIM_GetCapture1(TIM1);

            /* Capture computation */
            if (IC3ReadValue2 > IC3ReadValue1)
            {
                Capture = (IC3ReadValue2 - IC3ReadValue1);
            }
            else if (IC3ReadValue2 < IC3ReadValue1)
            {
                Capture = ((0xFFFF - IC3ReadValue1) + IC3ReadValue2);
            }
            else
            {
                Capture = 0;
            }
            /* Frequency computation */
            TIM1Freq = (uint32_t) SystemCoreClock / Capture;
            TIM_CaptureNumber = 0;
        }
    }
    else if (TIM_GetITStatus(TIM1, TIM_IT_CC3) == SET)
    {
        /* Clear TIM1 Capture compare interrupt pending bit */
        TIM_ClearITPendingBit(TIM1, TIM_IT_CC3);
        if (TIM_CaptureNumber == 0)
        {
            /* Get the Input Capture value */
            IC3ReadValue1 = TIM_GetCapture3(TIM1);
            TIM_CaptureNumber = 1;
        }
        else if (TIM_CaptureNumber == 1)
        {
            /* Get the Input Capture value */
            IC3ReadValue2 = TIM_GetCapture3(TIM1);

            /* Capture computation */
            if (IC3ReadValue2 > IC3ReadValue1)
            {
                Capture = (IC3ReadValue2 - IC3ReadValue1);
            }
            else if (IC3ReadValue2 < IC3ReadValue1)
            {
                Capture = ((0xFFFF - IC3ReadValue1) + IC3ReadValue2);
            }
            else
            {
                Capture = 0;
            }
            /* Frequency computation */
            TIM1Freq = (uint32_t) SystemCoreClock / Capture;
            TIM_CaptureNumber = 0;
        }
    }
    else if (TIM_GetITStatus(TIM1, TIM_IT_CC4) == SET)
    {
        /* Clear TIM1 Capture compare interrupt pending bit */
        TIM_ClearITPendingBit(TIM1, TIM_IT_CC4);
        if (TIM_CaptureNumber == 0)
        {
            /* Get the Input Capture value */
            IC3ReadValue1 = TIM_GetCapture4(TIM1);
            TIM_CaptureNumber = 1;
        }
        else if (TIM_CaptureNumber == 1)
        {
            /* Get the Input Capture value */
            IC3ReadValue2 = TIM_GetCapture4(TIM1);

            /* Capture computation */
            if (IC3ReadValue2 > IC3ReadValue1)
            {
                Capture = (IC3ReadValue2 - IC3ReadValue1);
            }
            else if (IC3ReadValue2 < IC3ReadValue1)
            {
                Capture = ((0xFFFF - IC3ReadValue1) + IC3ReadValue2);
            }
            else
            {
                Capture = 0;
            }
            /* Frequency computation */
            TIM1Freq = (uint32_t) SystemCoreClock / Capture;
            TIM_CaptureNumber = 0;
        }
    }
}

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
