/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : tim_dma_pulse_cnt.c
* @author    : FMD AE
* @brief     : tim_dma_pulse_cnt program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "tim_dma_pulse_cnt.h"
/* Private Constant --------------------------------------------------------------*/
#define TIM1_DMAR_ADDRESS ((uint32_t)0x40012C4C) /* TIM DMAR address */

/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/

/* Private function prototypes ---------------------------------------------------*/

/* Public function ------ --------------------------------------------------------*/



/**********************************************************************************
  * @brief  TIM1_CH1_Config program
  * @param  None
  * @note   TIM1_CH1(PA8) output a waveform with variable duty cycle.
  * @retval None
  *********************************************************************************
*/

void TIM1_CH1_Config(void)
{
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
    GPIO_InitTypeDef GPIO_InitStructure;
    TIM_OCInitTypeDef        TIM_OCInitStructure;
    /* TIM1 clock enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);

    /* GPIOA clock enable */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    /* DMA1 clock enable */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
    TIM_DeInit(TIM1);
    /* GPIOA Configuration: PA8(TIM1 CH1) as alternate function push-pull ------*/
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* Connect TIM pins to AF2 */
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_2);

    /* Time base configuration */
    /* -----------------------------------------------------------------------
      TIM1 Configuration: generate 1 PWM signal using the DMA burst mode:

      TIM1 input clock (TIM1CLK) is set to APB2 clock (PCLK2)
        TIM1CLK = PCLK2
        PCLK2 = HCLK
        => TIM1CLK = HCLK = SystemCoreClock

      To get TIM1 counter clock at 24 MHz, the prescaler is computed as follows:
        Prescaler = (TIM1CLK / TIM1 counter clock) - 1
        Prescaler = (SystemCoreClock /24 MHz) - 1

     TIM1 Frequency = TIM1 counter clock/(ARR + 1)
                                                 = 24 MHz / 4096 = 5.85 KHz
      TIM1 Channel1 duty cycle = (TIM1_CCR1/ TIM1_ARR)* 100 = 33.33%

      Note:
       SystemCoreClock variable holds HCLK frequency and is defined in system_FT32f0xx.c file.
       Each time the core clock (HCLK) changes, user had to call SystemCoreClockUpdate()
       function to update SystemCoreClock variable value. Otherwise, any configuration
       based on this variable will be incorrect.
    ----------------------------------------------------------------------- */
    TIM_TimeBaseStructure.TIM_Period = 240;//
    TIM_TimeBaseStructure.TIM_Prescaler = (uint16_t)(SystemCoreClock / 24000000) - 1;
    TIM_TimeBaseStructure.TIM_ClockDivision = 0x0;
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseStructure.TIM_RepetitionCounter   = 0;
    TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

    /* TIM Configuration in PWM Mode */
    TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Disable;
    TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
    TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCPolarity_High;
    TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Reset;
    TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCNIdleState_Reset;

    TIM_OCInitStructure.TIM_OCMode =  TIM_OCMode_PWM1;
    TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
    TIM_OCInitStructure.TIM_Pulse = 0x0000;
    TIM_OC1Init(TIM1, &TIM_OCInitStructure);
}

/**********************************************************************************
  * @brief  configure DMA1 CH2
  * @param  *buf      DMA buffer area
  * @param  len       the size of the DMA buffer.
  * @note   TIM1_CH1(PA8) output a waveform with variable duty cycle.
  * @retval None
  *********************************************************************************
*/

void DMA_Config(uint8_t *buf, uint16_t len)
{
    DMA_InitTypeDef          DMA_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;
    /* DeInitialize the DMA1 Stream5 */
    DMA_DeInit(DMA1_Channel2);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)TIM1_DMAR_ADDRESS;
    DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)buf;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
    DMA_InitStructure.DMA_BufferSize = len;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
    DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
    DMA_Init(DMA1_Channel2, &DMA_InitStructure);

    TIM_DMAConfig(TIM1, TIM_DMABase_CCR1, TIM_DMABurstLength_1Transfer);
    TIM_DMACmd(TIM1, TIM_DMA_CC1, ENABLE);
    TIM_OC1PreloadConfig(TIM1, TIM_OCPreload_Enable);

    NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel2_3_IRQn ;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    DMA_ITConfig(DMA1_Channel2, DMA_IT_TC, ENABLE);

    /* Enable DMA1 Stream2  */
    DMA_Cmd(DMA1_Channel2, ENABLE);
    TIM_CtrlPWMOutputs(TIM1, ENABLE);
    TIM_Cmd(TIM1, ENABLE);
}



/************************* (C) COPYRIGHT FMD *****END OF FILE*********************/
