/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_led.c
* @author    : FMD AE
* @brief     : bsp_led program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "bsp_led.h"

const PIN_CONFIG_t  PIN_CONFIG[LED_NUM] =
{
    { LED1_CLK, LED1_GPIOX, LED1_GPIO_Pin, Bit_RESET, 0U },
    { LED2_CLK, LED2_GPIOX, LED2_GPIO_Pin, Bit_RESET, 0U },
    { LED3_CLK, LED3_GPIOX, LED3_GPIO_Pin, Bit_SET, 0U },
    { LED4_CLK, LED4_GPIOX, LED4_GPIO_Pin, Bit_SET, 0U },
};

/**********************************************************************************
  * @brief  LEDGPIO_CONFIG program.
  * @param  None
  * @note
  * @retval None
  *********************************************************************************
*/

void LEDGPIO_CONFIG(void)
{

    /* Enable the GPIO_LED Clock */
    RCC_AHBPeriphClockCmd(RCC_AHBENR_GPIOBEN, ENABLE);

    GPIO_InitTypeDef    GPIO_InitStructure;

    /* Configure the GPIO_PC10 pin */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_3;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_WriteBit(GPIOB, GPIO_Pin_2, Bit_RESET);//
}

/**********************************************************************************
  * @brief  SETLED1 program.
  * @param  vale  may set LED_ON or LED_OFF
  * @note
  * @retval None
  *********************************************************************************
*/
void SETLED1(uint8_t vale)
{
    if (vale)
        GPIO_WriteBit(GPIOB, GPIO_Pin_0, Bit_SET);
    else
        GPIO_WriteBit(GPIOB, GPIO_Pin_0, Bit_RESET);
}
/**********************************************************************************
  * @brief  SETLED2 program.
  * @param  vale  may set LED_ON or LED_OFF
  * @note
  * @retval None
  *********************************************************************************
*/
void SETLED2(uint8_t vale)
{
    if (vale)
        GPIO_WriteBit(GPIOB, GPIO_Pin_1, Bit_SET);
    else
        GPIO_WriteBit(GPIOB, GPIO_Pin_1, Bit_RESET);
}
/**********************************************************************************
  * @brief  SETLED3 program.
  * @param  vale  may set LED_ON or LED_OFF
  * @note
  * @retval None
  *********************************************************************************
*/
void SETLED3(uint8_t vale)
{
    if (vale)
        GPIO_WriteBit(GPIOB, GPIO_Pin_2, Bit_RESET);
    else
        GPIO_WriteBit(GPIOB, GPIO_Pin_2, Bit_SET);
}
/**********************************************************************************
  * @brief  SETLED4 program.
  * @param  vale  may set LED_ON or LED_OFF
  * @note
  * @retval None
  *********************************************************************************
*/
void SETLED4(uint8_t vale)
{
    if (vale)
        GPIO_WriteBit(GPIOB, GPIO_Pin_3, Bit_RESET);
    else
        GPIO_WriteBit(GPIOB, GPIO_Pin_3, Bit_SET);
}

