/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : ft32f0xx_it.c
* @author    : FMD AE
* @brief     : ft32f0xx_it program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/
	
/* Includes ------------------------------------------------------------------*/
#include "ft32f0xx_it.h"
#include "bsp_led.h"
#include "bsp_tim_IR.h"


extern uint16_t LedCnt;
extern uint16_t IRcnt;

STRUCT_IR IR_Rec;

uint16_t TIM_CaptureNumber = 0;
uint32_t Capture = 0;
uint32_t TIM1Freq = 0;


extern void GetTouchValue(void);
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/******************************************************************************/
/*            Cortex-M0 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief  This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles PendSVC exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles TIM16 Handler.
  * @param  None
  * @retval None
  */
void TIM16_IRQHandler(void)
{

    if (TIM_GetITStatus(TIM16, TIM_IT_Update) != RESET)
    {
        TIM_ClearITPendingBit(TIM16, TIM_IT_Update);

        if (IR.busy)
        {

            TIM16->ARR  =  IR.data[IR.adr].period;
            TIM16->CCR1 =  IR.data[IR.adr].pulse;
            IR.adr++;
            if (IR.adr > IR.dataCnt)
            {
                TIM16->CCR1 = 0;
                IR.busy = 0;
            }

        }
    }

}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
	LedCnt++;

	IRcnt++;
}

/**
  * @brief  This function handles EXTI2_3 interrupt request.
  * @param  None
  * @retval None
  */
void EXTI2_3_IRQHandler(void)
{
    if (EXTI_GetITStatus(EXTI_Line2) != RESET)
    {
        /* Clear the EXTI line 2 pending bit */
        EXTI_ClearITPendingBit(EXTI_Line2);
				if(IR_Rec.status == 0)
				{
					IR_Rec.status++;
					IR_Rec.cnt = 0;
					IR_Rec.adr = 0;
				}
				else if(IR_Rec.status == 1)
				{
					IR_Rec.data[IR_Rec.adr].pulse = IR_Rec.cnt;
					IR_Rec.status++;
				}
				else if(IR_Rec.status == 2)
				{
					IR_Rec.data[IR_Rec.adr].period = IR_Rec.cnt;
					IR_Rec.adr++;
					IR_Rec.cnt = 0;
					IR_Rec.status = 1;
				}
				
				IR_Rec.recFlg = 1;
				IRcnt = 0;
       
    }
}

/**
  * @brief  This function handles TIM15.
  * @param  None
  * @retval None
  */
void TIM15_IRQHandler(void)   									
{
	if (TIM_GetITStatus(TIM15, TIM_IT_Update) != RESET)  		//update interrupt
	{
		TIM_ClearITPendingBit(TIM15, TIM_IT_Update );  			//clear interrupt update interrupt flag 
		
		IR_Rec.cnt++;
	}
}
/******************************************************************************/
/*                 FT32F0xx Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_ft32f0xx.s).                                               */
/******************************************************************************/

/**
  * @brief  This function handles PPP interrupt request.
  * @param  None
  * @retval None
  */
/*void PPP_IRQHandler(void)
{
}*/

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
