/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : main program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "string.h"


uint16_t LedCnt;
uint16_t IRcnt;

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */

int main(void)
{
    /*!< At this stage the microcontroller clock setting is already configured,
       this is done through SystemInit() function which is called from startup
       file (startup_ft32f0xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_ft32f0xx.c file
     */

    /* Initialize LED-related GPIO */

    SystickInit();

    LEDGPIO_CONFIG();

		PWM_Init();
		EXTI2_2_Config();
    TIM15_Time_Init();
		SendIRdata();
		while(1)
		{
			if(AnalyseIrProtocol())				
				break;
			SETLED1(LED_ON);
      SETLED2(LED_ON);
		}
	
		
    while (1)
    {
			if(LedCnt < 300)
			{
				SETLED1(LED_ON);
				SETLED2(LED_ON);
			}
			else if(LedCnt < 600)
			{
				SETLED1(LED_OFF);
				SETLED2(LED_OFF);
			}
			else
				LedCnt =0;

    }

}

/**
  * @brief  DelayNus program.
  * @param  time: This value can range from 0 to 0xffFFFFFF.
  * @retval None
  */
void DelayNus(uint32_t time)
{
    uint32_t i, j;
    /*when sysclk equal 48M ,the delay is about 1us*/
    for (i = 0; i < time; i++)
    {
        for (j = 0; j < 4; j++)
        {
            __NOP();
        }
    }
}

/**
  * @brief  DelayNms program.
  * @param  time: This value can range from 0 to 0xffFFFFFF.
  * @retval None
  */
void DelayNms(uint32_t time)
{
    for (uint32_t i = 0; i < time; i++)
    {
        DelayNus(1000);
    }
}

/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
