/**
	******************************************************************************
	* @file 		bsp_usart.c
	* @author 	FMD AE
	* @brief 		bsp_usart program body
	* @version 	V1.0.0
	* @data 		2021-09-14
	******************************************************************************
	* @attention
	* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
	* This software is provided by the copyright holders and contributors,and the
	*	software is believed to be accurate and reliable. However, Fremont Micro
	*	Devices (SZ) Corporation assumes no responsibility for the consequences of
	*	use of such software or for any infringement of patents of other rights
	*	of third parties, which may result from its use. No license is granted by
	*	implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
	*	Corporation.
	******************************************************************************
	*/
	
/* Includes ------------------------------------------------------------------*/
#include "bsp_usart.h"

/**
  * @brief  Configures CLOCK.
  * @param  None
  * @retval None
  */
static void RCC_Configuration(void)
{   
  /* Enable GPIO clock */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
  
  /* Enable 2xUSARTs Clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE); 
		
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
}


/**
  * @brief  Configures GPIO ports.
  * @param  None
  * @retval None
  */
static void GPIO_Configuration(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  
  /* USART1 Pins configuration ************************************************/
  /* Connect pin to Periph */
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_1); 
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_1);    
  
  /* Configure pins as AF pushpull */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure);  
	
	 /* USART2 Pins configuration ************************************************/
  /* Connect pin to Periph */
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_1); 
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource3, GPIO_AF_1);    
  
  /* Configure pins as AF pushpull */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure); 
}
   
	
	
/**
  * @brief  Configures the nested vectored interrupt controller.
  * @param  None
  * @retval None
  */
static void NVIC_Configuration(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;
  
  /* USART1 IRQ Channel configuration */
  NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPriority = 0x00;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
	
	 /* USART2 IRQ Channel configuration */
  NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPriority = 0x00;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}


/**
  * @brief  USART1 Send data.
  * @param  bufer: data.  
						cnt  : the size of data
  * @retval None
  */
void SendSerialDataByUsart1(uint8_t *bufer, uint16_t cnt)
{
	uint16_t i;
	for(i = 0; i < cnt; i++)
	{
			USART_SendData(USART1, bufer[i]);
        /* Loop until USART1 DR register is empty */ 
      while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET)
        {}
	}
}




/**
  * @brief  USART2 Send data.
  * @param  bufer: data.  
						cnt  : the size of data
  * @retval None
  */
void SendSerialDataByUsart2(uint8_t *bufer, uint16_t cnt)
{
	uint16_t i;
	for(i = 0; i < cnt; i++)
	{
			USART_SendData(USART2, bufer[i]);
        /* Loop until USART1 DR register is empty */ 
      while(USART_GetFlagStatus(USART2, USART_FLAG_TXE) == RESET)
        {}
	}
}

/**
  * @brief  Initialize Usart.
  * @param  baudrate: .  
						
  * @retval None
  */
void IniUart(uint32_t baudrate)
{
	USART_InitTypeDef USART_InitStructure;
	
	RCC_Configuration();
  /* Configure GPIO Pin Tx/Rx for Usart communication */
  GPIO_Configuration();
  /* Configure NVIC */
  NVIC_Configuration();
  
  USART_InitStructure.USART_BaudRate = baudrate;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  
  /* Prepare all uart to receive a data packet */
    USART_Init(USART1, &USART_InitStructure);
    /* Enable 2xUSARTs Receive interrupts */
    USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
    /* Enable the 2xUSARTs */
    USART_Cmd(USART1, ENABLE);
		
		/* Prepare all uart to receive a data packet */
    USART_Init(USART2, &USART_InitStructure);
    /* Enable 2xUSARTs Receive interrupts */
    USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);
    /* Enable the 2xUSARTs */
    USART_Cmd(USART2, ENABLE);
}
