/**
  *********************************************************************************
  * @file           bsp_systick.c
  * @author         FMD AE
  * @brief          This file contains  functions  for the bsp_systick firmware
  *                 library.
  * @version        V1.0.0
  * @data           2022-01-13
  *********************************************************************************
  * @attention
  * COPYRIGHT (C) 2021 Fremont Micro Devices Corporation All rights reserved.
  *    This software is provided by the copyright holders and contributors,and the
  *software is believed to be accurate and reliable. However, Fremont Micro Devices
  *Corporation assumes no responsibility for the consequences of use of such
  *software or for any infringement of patents of other rights of third parties,
  *which may result from its use. No license is granted by implication or otherwise
  *under any patent rights of Fremont Micro Devices Corporation.
  *  ******************************************************************************
  */

/* Includes ----------------------------------------------------------------------*/
#include "bsp_systick.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
__IO uint32_t TimingDelay;
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/

/* Public function ---------------------------------------------------------------*/
/******************************************************************************
  * @brief  Systick_Init    program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void Systick_Init(void)
{
    if(SysTick_Config(48000))
    {
        while(1);
    }
    SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);

}
/******************************************************************************
  * @brief  TimingDelay_Decrement   program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void TimingDelay_Decrement(void)
{
    if(TimingDelay != 0x00)
    {
        TimingDelay--;
    }
}
/******************************************************************************
  * @brief  SysTick_Delay_Ms    program
  * @param  nTime
  * @note
  * @retval None
  *****************************************************************************
*/
void SysTick_Delay_Ms(__IO uint32_t nTime)
{
    TimingDelay = nTime;            //时钟滴答数
    while(TimingDelay != 0)
    {
        ;
    }
}

/*48M时 大概1us*/
/******************************************************************************
  * @brief  Delay_Nus   program
  * @param  nTime
  * @note
  * @retval None
  *****************************************************************************
*/
void Delay_Nus(__IO uint32_t nTime)
{
    uint32_t i,j;

    for (i=0;i<32;i++)
    {
        for (j=0;j<nTime;j++)
        {
            __NOP();
        }
    }
}
/******************************************************************************
  * @brief  Delay_Nms   program
  * @param  nTime
  * @note
  * @retval None
  *****************************************************************************
*/
void Delay_Nms(__IO uint32_t nTime)
{
    uint32_t i;

    for (i=0;i<nTime;i++)
    {
        Delay_Nus(1000);
    }
}
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/
