/**
  ******************************************************************************
  * @file           usb_ep.h
  * @author         FMD AE
  * @brief          This file contains all the functions prototypes for the usb_ep
  * @version        V1.0.0
  * @data           2022-01-13
  ******************************************************************************
  * @attention
  * COPYRIGHT (C) 2021 Fremont Micro Devices Corporation All rights reserved.
  *    This software is provided by the copyright holders and contributors,and the
  *software is believed to be accurate and reliable. However, Fremont Micro Devices
  * Corporation assumes no responsibility for the consequences of use of such
  *software or for any infringement of patents of other rights of third parties,
  *which may result from its use. No license is granted by implication or otherwise
  *under any patent rights of Fremont Micro Devices Corporation.
  *  ******************************************************************************
  */
#ifndef _ENDPOINT_H_
#define _ENDPOINT_H_
/* Includes ---------------------------------------------------------------------*/
#include "usb_core.h"
#include "usb_desc.h"

/* Public Constant prototypes----------------------------------------------------*/

/* Public typedef ---------------------------------------------------------------*/

/* Bulk In Endpoint status structure */
typedef struct M_EPBIN_STATUS {
  void *pData;          /* Pointer to data to transmit */
  int   nBytesLeft;     /* Number of bytes left to send */
  BYTE  byEP;           /* Endpoint number */
} M_EPBIN_STATUS;
typedef M_EPBIN_STATUS * PM_EPBIN_STATUS;

/* Bulk Out Endpoint status structure */
typedef struct M_EPBOUT_STATUS {
  void *pData;          /* Pointer to data buffer */
  int   nBuffLen;       /* Length of data buffer */
  int   nBytesRecv;     /* Number of bytes received */
  BYTE  byEP;           /* Endpoint number */
  } M_EPBOUT_STATUS;
typedef M_EPBOUT_STATUS * PM_EPBOUT_STATUS;
  
  
  
enum EndpointN
{
    EP0 = 0,
    EP1,
    EP2,
    EP3,
    EP4,
    EP5,
    EP6,
    EP7
};
/* Public define ----------------------------------------------------------------*/
#define M_EP_NODATA     -1

/* Define maximum packet size for bulk endpoints */
#define M_EP_MAXP       128
/* Public variables prototypes --------------------------------------------------*/
/* Public function prototypes----------------------------------------------------*/
/* Function prototypes */
void EndpointBulkIn(PM_EPBIN_STATUS, int nCallState);
void EndpointBulkOut(PM_EPBOUT_STATUS, int);
void USB_Ep_Direc(uint8_t ep,uint8_t Direction);
void Endpoint_Tx_data(PM_EPBIN_STATUS pbistate);
void USB_Rx_NAK(PM_EPBOUT_STATUS pbostate, FunctionalState NewState);
#endif /* _ENDPOINT_H_ */
/************************* (C) COPYRIGHT FMD *****END OF FILE*********************/
