/**
  *********************************************************************************
  * @file           usb_desc.c
  * @author         FMD AE
  * @brief          This file contains  functions  for the usb_desc firmware
  *                 library.
  * @version        V1.0.0
  * @data           2022-01-13
  *********************************************************************************
  * @attention
  * COPYRIGHT (C) 2021 Fremont Micro Devices Corporation All rights reserved.
  *    This software is provided by the copyright holders and contributors,and the
  *software is believed to be accurate and reliable. However, Fremont Micro Devices
  *Corporation assumes no responsibility for the consequences of use of such
  *software or for any infringement of patents of other rights of third parties,
  *which may result from its use. No license is granted by implication or otherwise
  *under any patent rights of Fremont Micro Devices Corporation.
  *  ******************************************************************************
  */

/* Includes ----------------------------------------------------------------------*/
#include "usb_desc.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/




uint8_t Printer_DEVICE_ID[DEVICE_ID_LEN] =
{
    0x00, 0x67,
    'M', 'A', 'N', 'U', 'F', 'A', 'C', 'T', 'U', 'R', 'E', 'R', ':',
    'F', 'T', '3', '2', 'D', 'E', 'V', 'I', 'C', 'E', '-', ';',
    'C', 'O', 'M', 'M', 'A', 'N', 'D', ' ', 'S', 'E', 'T', ':',
    'P', 'C', 'L', ',', 'M', 'P', 'L', ';',
    'M', 'O', 'D', 'E', 'L', ':',
    'L', 'a', 's', 'e', 'r', 'B', 'e', 'a', 'm', '?', ';',
    'C', 'O', 'M', 'M', 'E', 'N', 'T', ':',
    'G', 'o', 'o', 'd', ' ', '!', ';',
    'A', 'C', 'T', 'I', 'V', 'E', ' ', 'C', 'O', 'M', 'M', 'A', 'N', 'D', ' ', 'S', 'E', 'T', ':',
    'P', 'C', 'L', ';'
};
const uint8_t USB_FSDeviceDescriptor[] =
{
    /*---------------------- Device Descriptor ----------------------*/
    0x12,       // bLength
    0x01,       // bDescriptorType
    0x00,       // bcdUSB  L
    0x02,       // bcdUSB  H
    0x00,       // bDeviceClass
    0x00,       // bDeviceSubClass
    0x00,       // bDeviceProtocol
    0x08,       // bMaxPacketSize0
    0xD3,       // idVendor  L
    0x34,       // idVendor     H
    0x00,       // idProduct L
    0x11,       // idProduct H
    0x00,       // bcdDevice L
    0x01,       // bcdDevice H
    0x01,       // iManufacturer
    0x02,       // iProduct
    0x03,       // iSerialNumber
    0x01        // bNumConfigurations
};
const uint16_t USB_FSDeviceDescriptorSize = sizeof(USB_FSDeviceDescriptor);

const uint8_t USB_FSConfigDescriptor[] =
{
    /* Configuration 1 */
    0x09,       /* bLength */
    0x02,       /* bDescriptorType */
    0x20,       /* wTotalLength   bytes*/
    0x00,
    0x01,       /* bNumInterfaces */
    0x01,       /* bConfigurationValue */
    0x00,       /* iConfiguration */
    0xC0,       /* bmAttributes  BUS Powred*/
    0x32,       /* bMaxPower = 100 mA*/
    /* 09 byte*/

    /*Interface 0 Descriptors*/
    0x09,       /* bLength*/
    0x04,       /* bDescriptorType */
    0x00,       /* bInterfaceNumber*/
    0x00,       /* bAlternateSetting*/
    0x02,       /* bNumEndpoints*/
    0x07,       /* bInterfaceClass*/
    0x01,       /* bInterfaceSubClass*/
    0x02,       /* bInterfaceProtocol*/
    0x00,       /* iInterface*/
    /*  09 byte*/

    /*Endpoint 6 Descriptors Out */
    0x07,       /* bLength*/
    0x05,       /* bDescriptorType */
    0x06,       /* bEndpointAddress*/
    0x02,       /* bmAttributes  bulk out endpoint*/
    0x80,       /* wMaxPacketSize */
    0x00,
    0x00,       /* bInterval*/
    /*  07 byte*/

    /*Endpoint 7 Descriptors In */
    0x07,       /* bLength*/
    0x05,       /* bDescriptorType */
    0x87,       /* bEndpointAddress*/
    0x02,       /* bmAttributes  bulk out endpoint*/
    0x80,       /* wMaxPacketSize */
    0x00,
    0x00,       /* bInterval*/
    /*  07 byte*/
};
const uint16_t USB_FSConfigDescriptorSize = sizeof(USB_FSConfigDescriptor);

const uint8_t LanguageId[] =
{
    0x04,       // 本描述符的长度
    0x03,       // 字符串描述符
    0x09,       // 0x0409为美式英语ID
    0x04
};
const uint16_t LanguageIdSize = sizeof(LanguageId);

const uint8_t ManufacturerStringDescriptor[] =
{
    (3*2 + 2),  // length
    0x03,       // 字符串描述符的类型编码0x03
    'F',0,
    'M',0,
    'D',0,
};
const uint16_t ManufacturerStringDescriptorSize = sizeof(ManufacturerStringDescriptor);

const uint8_t ProductStringDescriptor[] =
{
    (11*2 + 2), // length
    0x03,       // 字符串描述符的类型编码0x03
    'F',0,
    'M',0,
    'D',0,
    ' ',0,
    'P',0,
    'r',0,
    'i',0,
    'n',0,
    't',0,
    'e',0,
    'r',0,
};
const uint16_t ProductStringDescriptorSize = sizeof(ProductStringDescriptor);

const uint8_t SerialNumberSringDescriptor[] =
{
    (10*2 + 2), // length
    0x03,       // 字符串描述符的类型编码0x03
    '2',0,
    '0',0,
    '2',0,
    '0',0,
    '-',0,
    '0',0,
    '7',0,
    '-',0,
    '0',0,
    '7',0,
};
const uint16_t SerialNumberSringDescriptorSize = sizeof(SerialNumberSringDescriptor);

const uint8_t USB_HID_FSReportDescriptor_1 [] =
{
    0x00,
};
const uint16_t USB_HID_FSReportDescriptor_1Size = sizeof(USB_HID_FSReportDescriptor_1);

const uint8_t USB_HID_FSReportDescriptor_2 [] =
{
    0x00,0x00
};
const uint16_t USB_HID_FSReportDescriptor_2Size = sizeof(USB_HID_FSReportDescriptor_2);
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/
