#include "usb_scsi.h"

uint32_t ByteAddr;  //字节地址

const uint8_t Some_Data[] =
{
    0x03,0x00,0x00,0x00
};
const uint8_t Inquiry_Data[36] =
{
    0x00,/*外设类型*/
    0x80,/*可移除设备*/
    0x00,
    0x01,/*响应数据格式*/
    0x1f,/*附加数据长度*/
    /*5-7保留*/
    0x00,
    0x00,
    0x00,
    /*厂商信息*/
    'F',
    0x00,
    'M',
    0x00,
    'D',
    0x00,
    0x00,
    0x00,
    /*产品信息*/
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    0x00,
    /*产品版本信息*/
    0x00,
    0x00,
    0x00,
    0x00
};

const uint8_t Read_Format_Capacities_Data[] =
{
 0x00, 0x00, 0x00, //保留
 0x08,  //容量列表长度
 0x01, 0x00, 0x00, 0x00,  //块数(最大支持8GB)
 0x03, //描述符代码为3，表示最大支持的格式化容量
 0x00, 0x02, 0x00 //每块大小为512字节
};

const uint8_t Read_Capacity_Data[] =
{
 0x00,0x03,0xFF,0xFF, //能够访问的最大逻辑块地址
 0x00,0x00,0x02,0x00  //块的长度
 //所以该磁盘的容量为
 //(0x3FFFF+1)*0x200 = 0x8000000 = 128*1024*1024 = 128MB.
};

const uint8_t REQUEST_Sense_Data[18]=
{
 0x70, //错误代码，固定为0x70
 0x00, //保留
 0x05, //Sense Key为0x05，表示无效请求（ILLEGAL REQUEST）
 0x00, 0x00, 0x00, 0x00, //Information为0
 0x0A, //附加数据长度为10字节
 0x00, 0x00, 0x00, 0x00, //保留
 0x20, //Additional Sense Code(ASC)为0x20，表示无效命令操作码（INVALID COMMAND OPERATION CODE）
 0x00, //Additional Sense Code Qualifier(ASCQ)为0
 0x00, 0x00, 0x00, 0x00 //保留
};

/**
    *@USB_CBW_Data_TransferLength
    */
uint32_t USB_CBW_Data_TransferLength(pUSBC_CBW_TYPE cbw_temp)
{
 uint32_t Len;

 //CBW[8]~CBW[11]为传输长度（小端结构）
 Len = cbw_temp->dCBWDataTransferLength[3]<<24 | cbw_temp->dCBWDataTransferLength[2]<<16 |\
             cbw_temp->dCBWDataTransferLength[1]<<8  | cbw_temp->dCBWDataTransferLength[0];

 return Len;

}

/********************************************************************
函数功能：从CBW中获取逻辑块地址LBA的字节数。
入口参数：无。
返    回：逻辑块地址LBA。
备    注：无。
********************************************************************/
uint32_t GetLba(void)
{
 uint32_t Lba;
 pUSBC_CBW_TYPE cbw = (void *)epn_receive_buf;
 //读和写命令时，CBW[17]~CBW[20]为逻辑块地址（大端结构）

 Lba = cbw->CBWCB[2] <<24 | cbw->CBWCB[3] <<16 | cbw->CBWCB[4] <<8 | cbw->CBWCB[5];

 return Lba;
}

/********************************************************************
函数功能：获取磁盘数据函数。
入口参数：PM_EPBIN_STATUS pbistate。
返    回：无。
备    注：无。
********************************************************************/
void Get_Disk_Data(PM_EPBIN_STATUS pbistate)
{
    //判断该返回什么数据
    if(ByteAddr==0) pbistate->pData = (void *)Dbr; //返回DBR
    if(ByteAddr==512) pbistate->pData = (void *)Fat; //返回FAT
    if((ByteAddr>=576)&&(ByteAddr<16896)) pbistate->pData = (void *)Zeros;
    if(ByteAddr==16896) pbistate->pData = (void *)Fat; //返回FAT（备份FAT）
    if((ByteAddr>=16960)&&(ByteAddr<33280)) pbistate->pData = (void *)Zeros;
    if(ByteAddr==33280) pbistate->pData = (void *)RootDir; //返回根目录
    if((ByteAddr>=33344)&&(ByteAddr<49664)) pbistate->pData = (void *)Zeros;
    if(ByteAddr==49664) pbistate->pData = (void *)TestFileData; //返回文件数据
    if(ByteAddr>50175) pbistate->pData = (void *)Zeros;

    ByteAddr += M_EP_MAXP; //调整字节地址，每次发送最大包长度的数据
}

/********************************************************************
函数功能：填充CSW。
入口参数：Residue：剩余字节数；Status：命令执行的状态。
返    回：无。
备    注：无。
********************************************************************/
void USB_CSW_Set(int Residue, uint8_t Status)
{
    __IO pUSBC_CBW_TYPE cbw = &usb_cbw;

    //设置CSW的签名，其实可以不用每次都设置的，
    //开始初始化设置一次就行了，这里每次都设置
    usb_csw.dCSWSignature[0] = 0x55;
    usb_csw.dCSWSignature[1] = 0x53;
    usb_csw.dCSWSignature[2] = 0x42;
    usb_csw.dCSWSignature[3] = 0x53;

    //复制dCBWTag到CSW的dCSWTag中去
    usb_csw.dCSWTag[0] = cbw->dCBWTag[0];
    usb_csw.dCSWTag[1] = cbw->dCBWTag[1];
    usb_csw.dCSWTag[2] = cbw->dCBWTag[2];
    usb_csw.dCSWTag[3] = cbw->dCBWTag[3];

    //剩余字节数
    usb_csw.dCSWDataResidue[0] = Residue&0xFF;
    usb_csw.dCSWDataResidue[1] = (Residue>>8)&0xFF;
    usb_csw.dCSWDataResidue[2] = (Residue>>16)&0xFF;
    usb_csw.dCSWDataResidue[3] = (Residue>>24)&0xFF;

    //命令执行的状态，0表示成功，1表示失败。
    usb_csw.bCSWStatus = Status;
}

/**
    *@USB_SCSI_CMD
    */
void USB_SCSI_CMD(PM_EPBIN_STATUS pbistate,pUSBC_CBW_TYPE cbw_temp)
{
    uint16_t data_len = 0;

    /*进入数据阶段*/
    USB_CBW_STATUS = CBW_DATA_STAGE;

    switch(cbw_temp->CBWCB[0])
    {
        /*查询命令INQUIRY*/
        case    INQUIRY_CMD:            //0x12:
            data_len = USB_CBW_Data_TransferLength(cbw_temp);
            if(data_len > sizeof(Inquiry_Data))
            {
                data_len = sizeof(Inquiry_Data);
            }
            pbistate->pData = (void *)Inquiry_Data;
            pbistate->nBytesLeft = data_len;

            Endpoint_Tx_data(pbistate);
            break;
        /*读格式化命令READ FORMAT CAPACITIES*/
        case    READ_FT_CAPAC_CMD:                  //0x23:
            pbistate->pData = (void *)Read_Format_Capacities_Data;
            data_len = sizeof(Read_Format_Capacities_Data);
            if(data_len > sizeof(Read_Format_Capacities_Data))
            {
                data_len = sizeof(Read_Format_Capacities_Data);
            }
            pbistate->nBytesLeft = data_len;

            Endpoint_Tx_data(pbistate);
            break;
        /*读容量命令READ CAPACITY*/
        case    READ_CAPAC_CMD:         //0x25:
            pbistate->pData = (void *)Read_Capacity_Data;
            data_len = USB_CBW_Data_TransferLength(cbw_temp);
            if(data_len > sizeof(Read_Capacity_Data))
            {
                data_len = sizeof(Read_Capacity_Data);
            }
            pbistate->nBytesLeft = data_len;

            Endpoint_Tx_data(pbistate);
            break;
            /*read(10)命令*/
        case READ10_CMD:        //0x28:
            #if defined USB_DEBUG
                printf("read10 data\r\n");
            #endif
            /*获取需要读取的地址*/
            ByteAddr = GetLba() * 512;
            /*需要读取的字节数*/
            data_len = (cbw_temp->dCBWDataTransferLength[3]<<24 | cbw_temp->dCBWDataTransferLength[2]<<16 |\
                                    cbw_temp->dCBWDataTransferLength[1]<<8  |   cbw_temp->dCBWDataTransferLength[0]);
            pbistate->nBytesLeft = data_len;
            Get_Disk_Data(pbistate);
            Endpoint_Tx_data(pbistate);
            break;
        /*write(10)命令*/
        case WRITE10_CMD:           //0x2A:
            data_len = (cbw_temp->dCBWDataTransferLength[3]<<24 | cbw_temp->dCBWDataTransferLength[2]<<16 |\
                                    cbw_temp->dCBWDataTransferLength[1]<<8  |   cbw_temp->dCBWDataTransferLength[0]);
            recieved_len = data_len;

            break;
        /*REQUEST SENSE命令*/
        case REQUEST_SENSE_CMD:     //0x03:
            pbistate->pData = (void *)REQUEST_Sense_Data;

            data_len = USB_CBW_Data_TransferLength(cbw_temp);
            if(data_len > sizeof(REQUEST_Sense_Data))
            {
                data_len = sizeof(REQUEST_Sense_Data);
            }
            pbistate->nBytesLeft = data_len;
            Endpoint_Tx_data(pbistate);
            break;
        /*TEST UNIT READY*/
        case TEST_UNIT_READY_CMD:       //0x00:
            USB_CBW_STATUS = CBW_CMD_STAGE;
            USB_SCSI_CMD_STAGE(pbistate,0,0);
            Endpoint_Tx_data(pbistate);
            break;
        case    0x1a:
            pbistate->pData = (void *)Some_Data;

            data_len = USB_CBW_Data_TransferLength(cbw_temp);
            if(data_len > sizeof(Some_Data))
            {
                data_len = sizeof(Some_Data);
            }
            pbistate->nBytesLeft = data_len;
            Endpoint_Tx_data(pbistate);
            break;
        default:
//          if(cbw_temp->bmCBWFlags & 0x80)
//          {
//              //输入请求
//
//          }
            USB_CBW_STATUS = CBW_CMD_STAGE;
            USB_SCSI_CMD_STAGE(pbistate,0,1);
            Endpoint_Tx_data(pbistate);
            break;
    }
}
/**
    *@USB_SCSI_CMD_STAGE
    */
void USB_SCSI_CMD_STAGE(PM_EPBIN_STATUS pbistate,int Residue, uint8_t Status)
{
    if(Residue == M_EP_NODATA)
    {
        Residue = 0;
    }
    USB_CSW_Set(Residue,Status);

    pbistate->byEP = USE_EP_N(2);
    pbistate->pData = (void*)&usb_csw;
    pbistate->nBytesLeft = sizeof(usb_csw);
}
