/**
  ******************************************************************************
  * @file    board.c
  * @author  FMD-AE
  * @version V1.0.0
  * @date    2021-8-2
  * @brief   board program body
  ******************************************************************************
*/

#include "board.h"

__IO uint32_t TimingDelay;

/******************************************************************************
  * @brief  Led_Init program.
  * @param  None
  * @retval None
  *****************************************************************************
*/
void Led_Init(void)
{
    GPIO_InitTypeDef    GPIO_InitStruct;
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB,ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOF,ENABLE);

    GPIO_InitStruct.GPIO_Pin = LED1_PIN | LED2_PIN | LED3_PIN | LED4_PIN;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(LED_PORT,&GPIO_InitStruct);

    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_5;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOF,&GPIO_InitStruct);

    LED1_OFF;LED2_OFF;LED3_OFF;LED4_OFF;
}
/******************************************************************************
  * @brief  LED_Toggle program.
  * @param  GPIOx AND GPIO_Pin
  * @retval None
  *****************************************************************************
*/
void LED_Toggle(GPIO_TypeDef* GPIOx,uint16_t GPIO_Pin)
{
    GPIO_WriteBit(GPIOx, GPIO_Pin, (BitAction)((1-GPIO_ReadOutputDataBit(GPIOx, GPIO_Pin))));
}

/******************************************************************************
  * @brief  Key_Init program.
  * @param  None
  * @retval None
  *****************************************************************************
*/
void Key_Init(void)
{
    GPIO_InitTypeDef    GPIO_InitStruct;
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC,ENABLE);

    GPIO_InitStruct.GPIO_Pin = KEY1_PIN;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_DOWN;
    GPIO_Init(KEY_PORT,&GPIO_InitStruct);

    GPIO_InitStruct.GPIO_Pin = KEY2_PIN;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(KEY_PORT,&GPIO_InitStruct);
}
/******************************************************************************
  * @brief  Key_Scan    program
  * @param  FlashRate
  * @retval None
  *****************************************************************************
*/
KEY_DATA Key_Scan(void)
{
    static uint8_t key_up=1;
    static uint8_t key_time=0;

    if(key_up&&(KEY1_PRES==1 || KEY2_PRES==0 ))
    {
        key_time++;
        if(key_time>=2)
        {
            key_up=0;
            if(KEY1_PRES==1) return KEY1_Data;
            if(KEY2_PRES==0) return KEY2_Data;
        }
    }
    else if(KEY1_PRES==0 && KEY2_PRES==1)
    {
        key_up=1;
        key_time=0;
    }
    return KEY_None;
}
/******************************************************************************
  * @brief  SysTick_Configuration program.
  * @param  None
  * @retval None
  *****************************************************************************
*/
void SysTick_Configuration(void)
{
    if(SysTick_Config(SystemCoreClock/TICK_RATE_HZ))
    {
        while(1);
    }
  SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);
  NVIC_SetPriority(SysTick_IRQn, 0x04);
}

/******************************************************************************
  * @brief  TimingDelay_Decrement program.
  * @param  None
  * @retval None
  *****************************************************************************
*/
void TimingDelay_Decrement(void)
{
    if(TimingDelay != 0x00)
    {
        TimingDelay--;
    }
}

/******************************************************************************
  * @brief  SysTick_Delay_Ms program.
  * @param  nTime
  * @retval None
  *****************************************************************************
*/
void SysTick_Delay_Ms(__IO uint32_t nTime)
{
    TimingDelay = nTime/10;
    while(TimingDelay != 0);
}


/**
  * @}
  */

/************************ (C) COPYRIGHT FMD *****END OF FILE****/
