/**
  *********************************************************************************
  * @file           main.c
  * @author         FMD AE
  * @brief          Main Service Routines.
  * @version        V1.0.0
  * @data           2022-01-13
  *********************************************************************************
  * @attention
  * COPYRIGHT (C) 2021 Fremont Micro Devices Corporation All rights reserved.
  *    This software is provided by the copyright holders and contributors,and the
  *software is believed to be accurate and reliable. However, Fremont Micro Devices
  *Corporation assumes no responsibility for the consequences of use of such
  *software or for any infringement of patents of other rights of third parties,
  *which may result from its use. No license is granted by implication or otherwise
  *under any patent rights of Fremont Micro Devices Corporation.
  *  ******************************************************************************
  */

/* Includes ----------------------------------------------------------------------*/
#include "main.h"
#include "board.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
uint8_t TxBuff[128] = {1,2,3,4,5,6,7,8,9,10};
uint8_t send_flag;
/* Public variables --------------------------------------------------------------*/
extern uint8_t Time10ms_flag;
extern M_EPBOUT_STATUS gepbout1;
uint32_t Timecount;
/* Private function prototypes ---------------------------------------------------*/
ErrorStatus USB_EP_Tx(uint8_t Ep, uint8_t *ptr, uint8_t data_len);
void LED_Flash(uint8_t FlashRate);
void Key_Pro(void);
/* Public function ---------------------------------------------------------------*/
/******************************************************************************
  * @brief  main program
  * @param
  * @note
  * @retval
  *****************************************************************************
*/
int main(void)
{
    uint8_t n = 0;

    Bsp_USART_Config();
    printf("SYSRESET!!!!\r\n");
    SysTick_Configuration();
    USB_Init();
    Led_Init();
    Key_Init();
    while (1)
    {
        if (Time10ms_flag == 1)
        {
            Time10ms_flag = 0;
            Timecount++;
            Key_Pro();
            LED_Flash(200);
        }
        if(gepbout1.nBytesRecv != 0)
        {
            if(epn_receive_buf[0] == 0x10 && epn_receive_buf[1] == 0x04 && epn_receive_buf[2] == 0x01)
                send_flag = 1;
            while(gepbout1.nBytesRecv != n)
            {
                while (USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET)
                {

                }
                USART_SendData(USART1,epn_receive_buf[n++]);                                              
            }               
            gepbout1.pData = epn_receive_buf;
            gepbout1.nBytesRecv = 0;
            n = 0;      
            USB_Rx_NAK(&gepbout1, DISABLE);
        }
        if( send_flag )
        {
            USB_EP_Tx(7,TxBuff,10);
            send_flag = 0;
        }
        
    }
}




/****************************************************************************
  USB EP Tx
  This routine is called when data is to be sent to the host.
****************************************/
ErrorStatus USB_EP_Tx(uint8_t Ep, uint8_t *ptr, uint8_t data_len)
{
    uint32_t timeout = 0;
    M_EPBIN_STATUS  gepbin1;
    if (gnDevState > DEVSTATE_ADDRESS)
    {
        gepbin1.byEP = Ep;
        gepbin1.nBytesLeft = data_len;
        gepbin1.pData = ptr;

        EndpointBulkIn(&gepbin1, M_EP_NORMAL);
        while (MREAD_BYTE(M_REG_INCSR1) & 0x01)
        {
            timeout++;
            if (timeout >= 0xffffffff)
            {
                return ERROR;
            }
        }
        return SUCCESS;
    }

    return ERROR;
}

/******************************************************************************
  * @brief  LED_Flash   program
  * @param  FlashRate
  * @retval None
  *****************************************************************************
*/
void LED_Flash(uint8_t FlashRate)
{
    if (Timecount % (FlashRate / 10) == 0)
    {
        LED_Toggle(LED_PORT, LED3_PIN);
    }
}
/******************************************************************************
  * @brief  Key_Pro program
  * @param  FlashRate
  * @retval None
  *****************************************************************************
*/
void Key_Pro(void)
{
    uint8_t Key_Value;
    Key_Value = Key_Scan();
    if (Key_Value)
    {
        switch (Key_Value)
        {
        case KEY1_Data:

            break;
        case KEY2_Data:

            break;
        default:
            break;
        }
    }
}

/******************************************************************************
  * @brief  Bsp_USART_GPIO_Init program
  * @param  num
  * @note
  * @retval None
  *****************************************************************************
*/
void Bsp_USART_GPIO_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStruct;

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    /*GPIO INIT*/
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_Init(GPIOA, &GPIO_InitStruct);

    /*复用功能定义*/
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_1);
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_1);
}
/******************************************************************************
  * @brief  Bsp_USART_Config program
  * @param  num
  * @note
  * @retval None
  *****************************************************************************
*/
void Bsp_USART_Config(void)
{

    USART_InitTypeDef USART_InitStruct;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

    /*初始化USART1 GPIO*/
    Bsp_USART_GPIO_Init();

    /*初始化USART1*/
    USART_InitStruct.USART_BaudRate = 115200;
    USART_InitStruct.USART_WordLength = USART_WordLength_8b;
    USART_InitStruct.USART_StopBits = USART_StopBits_1;
    USART_InitStruct.USART_Parity = USART_Parity_No;
    USART_InitStruct.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;
    USART_InitStruct.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_Init(USART1, &USART_InitStruct);
    USART_Cmd(USART1, ENABLE);

}

/******************************************************************************
  * @brief  fputc program
  * @param  num
  * @note
  * @retval None
  *****************************************************************************
*/
int fputc(int ch, FILE *f)
{
    while (USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET)
    {

    }
    USART_SendData(USART1, (uint8_t) ch);
    return ch;
}
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/
