/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_desc.c
* @author    : FMD AE
* @brief     : usb_desc program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "usb_desc.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/

const uint8_t USB_FSDeviceDescriptor[] =
{
    /*---------------------- Device Descriptor ----------------------*/
    0x12,       /* bLength */
    0x01,       /* bDescriptorType */
    0x00,       /* bcdUSB L */
    0x02,       /* bcdUSB H */
    0x00,       /* bDeviceClass */
    0x00,       /* bDeviceSubClass */
    0x00,       /* bDeviceProtocol */
    0x08,       /* bMaxPacketSize0 */
    0xD3,       /* idVendor L */
    0x34,       /* idVendor H */
    0x04,       /* idProduct L */
    0x00,       /* idProduct H */
    0x00,       /* bcdDevice L */
    0x01,       /* bcdDevice H */
    0x01,       /* iManufacturer */
    0x02,       /* iProduct */
    0x03,       /* iSerialNumber */
    0x01        /* bNumConfigurations */
};
const uint16_t USB_FSDeviceDescriptorSize = sizeof(USB_FSDeviceDescriptor);

const uint8_t USB_FSConfigDescriptor[] =
{
    //Configuration:wTotalLength = 0x00be,NumInterfaces = 0x03,
    0x09,       /* bLength */
    0x02,       /* bDescriptorType */
    0xbe,       /* wTotalLength*/
    0x00,
    0x03,       /* bNumInterfaces */
    0x01,       /* bConfigurationValue */
    0x00,       /* iConfiguration */
    0x80,       /* bmAttributes  BUS Powred*/
    0xfa,       /* bMaxPower = 100 mA*/

    //standard interface AC descriptor(Interface 0, Alternate Setting 0):
    //bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x01(audio ctl),
    0x09,       /* Length */
    0x04,       /* DescriptorType:Inerface */
    0x00,       /* InterfaceNum:0 */
    0x00,       /* AlternateSetting:0 */
    0x00,       /* NumEndpoint:0 */
    0x01,       /* InterfaceClass:audio */
    0x01,       /* InterfaceSubClass:audio ctl */
    0x00,       /* InterfaceProtocol */
    0x00,       /* Interface String */

    //class-specific AC interface descriptor,audio interface(0x24),audio control header(0x01),
    //Total Length 0x0048,Number of streaming interface 2,interfaceNr 2,1
    0x0a,       /* Length */
    0x24,       /* DescriptorType:audio interface descriptor */
    0x01,       /* DescriptorSubType:audio control header */
    0x00,
    0x01,       /* bcdADC:audio Device Class v1.00 */
    0x48,
    0x00,       /* TotalLength:0x0048 */
    0x02,       /* InCollection:2 AudioStreaming interface */
    0x02,       /* InterfaceNr(2) - AS #1 id AudioStreaming interface 2 belongs to this AudioControl interface */
    0x01,       /* InterfaceNr(1) - AS #2 id AudioStreaming interface 1 belongs to this AudioControl interface */

    //USB Microphone IT:audio interface descriptor,audio control input terminal(0x02),terminal id 0x01,
    //Microphone(0x0201),Input Terminal(0x02),2 channel:Left Front,Right Front
    0x0c,       /* Length */
    0x24,       /* DescriptorType:audio interface descriptor */
    0x02,       /* DescriptorSubType:Input Terminal */
    0x01,       /* TerminalID:0x01 */
    0x01,
    0x02,       /* TerminalType:USB Microphone */
    0x00,       /* AssocTerminal */
    0x02,       /* NrChannels:2 channel */
    0x03,
    0x00,       /* ChannelConfig:Left Front,Right Front */
    0x00,       /* ChannelName String */
    0x00,       /* Terminal String */

    //Audio Feature Unit Descriptor:audio interface descriptor,feature_unit(0x06),terminal id 0x02,
    //SourceId 0x01,ControlSize 0x01,Mute,Volume,
    0x0a,//Length
    0x24,       /* DescriptorType:audio interface descriptor */
    0x06,       /* DescriptorSubType:Audio Feature Unit */
    0x02,       /* UnitID:0x02 */
    0x01,       /* SourceID:1 #Microphone IT */
    0x01,       /* ControlSize:1 byte */
    0x01,       /* Controls:Mute */
    0x00,       /* Controls(0):Volume */
    0x00,       /* Controls(1):Volume */
    0x00,       /* Feature String */

    //USB Streaming OT:audio interface descriptor,audio control output terminal(0x03),terminal id 0x03,
    //USB Streaming(0x0101),Output Terminal(0x03),SourceId 0x02,
    0x09,       /* Length */
    0x24,       /* DescriptorType:audio interface descriptor */
    0x03,       /* DescriptorSubTYpe:Output Terminal */
    0x03,       /* TerminalID:0x03 */
    0x01,
    0x01,       /* TerminalType:USB Streaming */
    0x00,       /* AssocTerminal:ID 0 */
    0x02,       /* SourceID:2 #Feature UNIT */
    0x00,       /* Terminal String */

    //USB USB Streaming IT:audio interface descriptor,audio control input terminal(0x02),terminal id 0x04,
    //USB Streaming(0x0101),Input Terminal(0x02),2 channel:Left Front,Right Front
    0x0c,       /* Length */
    0x24,       /* DescriptorType:audio interface descriptor */
    0x02,       /* DescriptorSubType:Input Terminal */
    0x04,       /* TerminalID:0x04 */
    0x01,
    0x01,       /* TerminalType:USB Streaming */
    0x00,       /* AssocTerminal */
    0x02,       /* NrChannels:2 channel */
    0x03,
    0x00,       /* ChannelConfig:Left Front,Right Front */
    0x00,       /* ChannelName String */
    0x00,       /* Terminal String */

    //Audio Feature Unit Descriptor:audio interface descriptor,feature_unit(0x06),terminal id 0x05,
    //SourceId 0x04,ControlSize 0x01,Mute,Volume,
    0x0a,       /* Length */
    0x24,       /* DescriptorType:audio interface descriptor */
    0x06,       /* DescriptorSubType:Audio Feature Unit */
    0x05,       /* UnitID:0x05 */
    0x04,       /* SourceID:4 #USB Streaming IT */
    0x01,       /* ControlSize:1 byte */
    0x01,       /* Controls:Mute */
    0x00,       /* Controls(0):Volume */
    0x00,       /* Controls(1):Volume */
    0x00,       /* Feature String */

    //USB Speaker OT:audio interface descriptor,audio control output terminal(0x03),terminal id 0x06,
    //USB Speaker(0x0301),Output Terminal(0x03),SourceId 0x05,
    0x09,       /* Length */
    0x24,       /* DescriptorType:audio interface descriptor */
    0x03,       /* DescriptorSubTYpe:Output Terminal */
    0x06,       /* TerminalID:0x06 */
    0x01,
    0x03,       /* TerminalType:Speaker */
    0x00,       /* AssocTerminal */
    0x05,       /* SourceID:5 #Feature UNIT */
    0x00,       /* Terminal String */

    //-------------------Microphone interface---------------------//
    //standard interface AS descriptor(Interface 1, Alternate Setting 0):
    //bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02(audio stream),
    0x09,       /* Length */
    0x04,       /* DescriptorType:Interface */
    0x01,       /* InterfaceNum:1 */
    0x00,       /* AlternateSetting:0 */
    0x00,       /* NumEndpoint:0 */
    0x01,       /* InterfaceClass:audio */
    0x02,       /* InterfaceSubClass:audio streaming */
    0x00,       /* InterfaceProtocol */
    0x00,       /* Interface String */

    //standard interface AS descriptor(Interface 1, Alternate Setting 1):
    //bNumEndpoints = 0x01,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02(audio stream),
    0x09,       /* Length */
    0x04,       /* DescriptorType:Interface */
    0x01,       /* InterfaceNum:1 */
    0x01,       /* AlternateSetting:1 */
    0x01,       /* NumEndpoint:1 */
    0x01,       /* InterfaceClass:audio */
    0x02,       /* InterfaceSubClass:audio streaming */
    0x00,       /* InterfaceProtocol */
    0x00,       /* Interface String */

    //Audio Streaming Interface Descriptor:AS_GENERAL(0x01),
    //TerminalLink 0x03,PCM(0x0001)
    0x07,       /* Length */
    0x24,       /* DescriptorType:audio interface descriptor */
    0x01,       /* DescriptorSubType:AS_GENERAL */
    0x03,       /* TerminalLink:#3USB USB Streaming OT */
    0x01,       /* Delay:1 */
    0x01,
    0x00,       /* FormatTag:PCM */

    //Type 1 Format type descriptor:FORMAT_TYPE(0x02),FORMAT_TYPE_I(0x01),
    //physical channels 0x02,two byte per audio subframe(0x02),16bit,
    //32K(0x007d00)
    0x0b,       /* Length */
    0x24,       /* DescriptorType:audio interface descriptor */
    0x02,       /* DescriptorSubType:Format_type */
    0x01,       /* FormatType:Format type 1 */
    0x02,       /* NumberOfChanne:2 */
    0x02,       /* SubframeSize:2byte */
    0x10,       /* BitsResolution:16bit */
    0x01,       /* SampleFreqType:One sampling frequency */
    0x00,
    0x7d,
    0x00,       /* 32K(0x007d00) */

    //Endpoint1 - Standard Descriptor:Input Endpoint1
    //Isochronous,Synchronization Type(Asynchronous),MaxPacketSize 0x0084,
    0x07,       /* Length */
    0x05,       /* DescriptorType:endpoint descriptor */
    0x86,       /* EndpointAddress:Input endpoint 6 */
    0x05,       /* Attributes:0x05,Isochronous,Synchronization Type(Asynchronous) */
    0x84,
    0x00,       /* MaxPacketSize:0x0084= */
    0x01,       /* Interval */

    //Endpoint - Audio Streaming Descriptor:
    //Audio Endpoint descriptor,General,
    0x07,       /* Length */
    0x25,       /* DescriptorType:audio endpoint descriptor */
    0x01,       /* DescriptorSubType:audio endpiont general */
    0x00,       /* Attributes:0x00 */
    0x00,       /* LockDelayUnits */
    0x00,
    0x00,       /* LockDelay */

    //-------------------Speaker interface---------------------//
    //standard interface AS descriptor(Interface 2, Alternate Setting 0):
    //bNumEndpoints = 0x00,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02(audio stream),
    0x09,       /* Length */
    0x04,       /* DescriptorType:Interface */
    0x02,       /* InterfaceNum:2 */
    0x00,       /* AlternateSetting:0 */
    0x00,       /* NumEndpoint:0 */
    0x01,       /* InterfaceClass:audio */
    0x02,       /* InterfaceSubClass:audio streaming */
    0x00,       /* InterfaceProtocol */
    0x00,       /* Interface String */

    //standard interface AS descriptor(Interface 2, Alternate Setting 1):
    //bNumEndpoints = 0x01,bInterFaceClass = 0x01(audio),bInterfaceSubClass = 0x02(audio stream),
    0x09,       /* Length */
    0x04,       /* DescriptorType:Interface */
    0x02,       /* InterfaceNum:2 */
    0x01,       /* AlternateSetting:1 */
    0x01,       /* NumEndpoint:1 */
    0x01,       /* InterfaceClass:audio */
    0x02,       /* InterfaceSubClass:audio streaming */
    0x00,       /* InterfaceProtocol */
    0x00,       /* Interface String */

    //Audio Streaming Interface Descriptor:AS_GENERAL(0x01),
    //TerminalLink 0x04,PCM(0x0001)
    0x07,       /* Length */
    0x24,       /* DescriptorType:audio interface descriptor */
    0x01,       /* DescriptorSubType:AS_GENERAL */
    0x04,       /* TerminalLink:#4 USB Streaming IT */
    0x01,       /* Delay:1 */
    0x01,
    0x00,       /* FormatTag:PCM */

    //Type 1 Format type descriptor:FORMAT_TYPE(0x02),FORMAT_TYPE_I(0x01),
    //physical channels 0x02,two byte per audio subframe(0x02),16bit,
    //32K(0x007d00)
    0x0b,       /* Length */
    0x24,       /* DescriptorType:audio interface descriptor */
    0x02,       /* DescriptorSubType:Format_type */
    0x01,       /* FormatType:Format type 1 */
    0x02,       /* NumberOfChanne:2 */
    0x02,       /* SubframeSize:2byte */
    0x10,       /* BitsResolution:16bit */
    0x01,       /* SampleFreqType:One sampling frequency */
    0x00,
    0x7d,
    0x00,       /* 32K(0x007d00) */

    //Endpoint2 - Standard Descriptor:Output Endpoint2
    //Isochronous,Synchronization Type(Asynchronous),MaxPacketSize 0x0084,
    0x07,       /* Length */
    0x05,       /* DescriptorType:endpoint descriptor */
    0x07,       /* EndpointAddress:Output endpoint 7 */
    0x05,       /* Attributes:0x05,Isochronous,Synchronization Type(Asynchronous) */
    0x84,
    0x00,       /* MaxPacketSize:0x0084= */
    0x01,       /* Interval */

    //Endpoint - Audio Streaming Descriptor:
    //Audio Endpoint descriptor,General,
    0x07,       /* Length */
    0x25,       /* DescriptorType:audio endpoint descriptor */
    0x01,       /* DescriptorSubType:audio endpiont general */
    0x00,       /* Attributes:0x00 */
    0x00,       /* LockDelayUnits */
    0x00,
    0x00,       /* LockDelay */
};
const uint16_t USB_FSConfigDescriptorSize = sizeof(USB_FSConfigDescriptor);

const uint8_t LanguageId[] =
{
    0x04,       /* ĳ */
    0x03,       /* ַ */
    0x09,       /* 0x0409ΪʽӢID */
    0x04
};
const uint16_t LanguageIdSize = sizeof(LanguageId);

const uint8_t ManufacturerStringDescriptor[] =
{
    (3 * 2 + 2), /* length */
    0x03,       /* ַͱ0x03 */
    'F', 0,
    'M', 0,
    'D', 0,
};
const uint16_t ManufacturerStringDescriptorSize = sizeof(ManufacturerStringDescriptor);

const uint8_t ProductStringDescriptor[] =
{
    (9 * 2 + 2), /* length */
    0x03,       /* ַͱ0x03 */
    'F', 0,
    'M', 0,
    'D', 0,
    ' ', 0,
    'A', 0,
    'U', 0,
    'D', 0,
    'I', 0,
    'O', 0
};
const uint16_t ProductStringDescriptorSize = sizeof(ProductStringDescriptor);

const uint8_t SerialNumberSringDescriptor[] =
{
    (10 * 2 + 2), /* length */
    0x03,       /* ַͱ0x03 */
    '2', 0,
    '0', 0,
    '2', 0,
    '0', 0,
    '-', 0,
    '0', 0,
    '7', 0,
    '-', 0,
    '0', 0,
    '7', 0,
};
const uint16_t SerialNumberSringDescriptorSize = sizeof(SerialNumberSringDescriptor);

const uint8_t USB_HID_FSReportDescriptor_1 [] =
{
    0x00,
};
const uint16_t USB_HID_FSReportDescriptor_1Size = sizeof(USB_HID_FSReportDescriptor_1);

const uint8_t USB_HID_FSReportDescriptor_2 [] =
{
    0x05, 0x0C,           /* Usage Page (Consumer) */
    0x09, 0x01,           /* Usage (Consumer Control) */
    0xA1, 0x01,           /* Collection (Application) */
    0x85, 0x01,           /* Report ID (0x01) */
    0x09, 0xE0,           /* Usage (Volume) */
    0x15, 0xE8,           /* Logical Minimum (-24) */
    0x25, 0x18,           /* Logical Maximum (24) */
    0x75, 0x07,           /* Report Size (7) */
    0x95, 0x01,           /* Report Count (1) */
    0x81, 0x06,           /* Input (Var, Rel) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x25, 0x01,           /* Logical Maximum (1) */
    0x75, 0x01,           /* Report Size (1) */
    0x09, 0xE2,           /* Usage (Mute) */
    0x81, 0x06,           /* Input (Var, Rel) */
    0xC0,                 /* End Collection */
    0x06, 0x01, 0x00,     /* Usage Page (Generic Desktop Controls) */
    0x09, 0x80,           /* Usage (System Control) */
    0xA1, 0x01,           /* Collection (Application) */
    0x85, 0x02,           /* Report ID (0x02) */
    0x25, 0x01,           /* Logical Maximum (1) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x75, 0x01,           /* Report Size (1) */
    0x0A, 0x81, 0x00,     /* Usage (System Power Down) */
    0x0A, 0x82, 0x00,     /* Usage (System Sleep) */
    0x0A, 0x83, 0x00,     /* Usage (System Wake Up) */
    0x95, 0x03,           /* Report Count (3) */
    0x81, 0x06,           /* Input (Var, Rel) */
    0x95, 0x05,           /* Report Count (5) */
    0x81, 0x01,           /* Input (Const) */
    0xC0,                 /* End Collection */
    0x06, 0x0C, 0x00,     /* Usage Page (Consumer) */
    0x09, 0x01,           /* Usage (Consumer Control) */
    0xA1, 0x01,           /* Collection (Application) */
    0x85, 0x03,           /* Report ID (0x03) */
    0x25, 0x01,           /* Logical Maximum (1) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x75, 0x01,           /* Report Size (1) */
    0x0A, 0xB5, 0x00,     /* Usage (Scan Next Track) */
    0x0A, 0xB6, 0x00,     /* Usage (Scan Previous Track) */
    0x0A, 0xB7, 0x00,     /* Usage (Stop) */
    0x0A, 0xB8, 0x00,     /* Usage (Eject) */
    0x0A, 0xCD, 0x00,     /* Usage (Play/Pause) */
    0x0A, 0xE2, 0x00,     /* Usage (Mute) */
    0x0A, 0xE9, 0x00,     /* Usage (Volume Increment) */
    0x0A, 0xEA, 0x00,     /* Usage (Volume Decrement) */
    0x95, 0x08,           /* Report Count (8) */
    0x81, 0x02,           /* Input (Var) */
    0x0A, 0x83, 0x01,     /* Usage (AL Consumer Control Configuration) */
    0x0A, 0x8A, 0x01,     /* Usage (AL Email Reader) */
    0x0A, 0x92, 0x01,     /* Usage (AL Calculator) */
    0x0A, 0x94, 0x01,     /* Usage (AL Local Machine Browser) */
    0x0A, 0x21, 0x02,     /* Usage (AC Search) */
    0x0A, 0x23, 0x02,     /* Usage (AC Home) */
    0x0A, 0x24, 0x02,     /* Usage (AC Back) */
    0x0A, 0x25, 0x02,     /* Usage (AC Forward) */
    0x95, 0x08,           /* Report Count (8) */
    0x81, 0x02,           /* Input (Var) */
    0x0A, 0x26, 0x02,     /* Usage (AC Stop) */
    0x0A, 0x27, 0x02,     /* Usage (AC Refresh) */
    0x0A, 0x2A, 0x02,     /* Usage (0x22A) */
    0x0A, 0xB3, 0x00,     /* Usage (Fast Forward) */
    0x0A, 0xB4, 0x00,     /* Usage (Rewind) */
    0x95, 0x05,           /* Report Count (5) */
    0x81, 0x02,           /* Input (Var) */
    0x95, 0x03,           /* Report Count (3) */
    0x81, 0x01,           /* Input (Const) */
    0xC0,                 /* End Collection */
    0x06, 0x00, 0xFF,     /* Usage Page (unk) */
    0x09, 0x01,           /* Usage (0x01) */
    0xA1, 0x01,           /* Collection (Application) */
    0x85, 0x05,           /* Report ID (0x05) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x26, 0xFF, 0x00,     /* Logical Maximum (255) */
    0x19, 0x01,           /* Usage Minimum (1) */
    0x29, 0x02,           /* Usage Maximum (2) */
    0x75, 0x08,           /* Report Size (8) */
    0x95, 0x05,           /* Report Count (5) */
    0xB1, 0x02,           /* Feature (Var) */
    0xC0                  /* End Collection */
};
const uint16_t USB_HID_FSReportDescriptor_2Size = sizeof(USB_HID_FSReportDescriptor_2);
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/
