/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : main program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "main.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Interrupt vector mapping replication address, the start ADDRESS of RAM needs to be set in APP project */
__IO uint32_t VectorTable[48] __attribute__((at(0X20000000)));
uint8_t key_scan[2] = {0};
uint8_t old_scan[2] = {0};
uint8_t new_scan[2] = {0};
uint8_t key_status = 0;

/* Public variables --------------------------------------------------------------*/
uint32_t test_usb = 0;
uint8_t USB_Receive_Buffer[64] = {0};
uint8_t debounce_time[2] = {0};

/* Private function prototypes ---------------------------------------------------*/
void usb_send_data(void);
static void NVIC_SetVectorTable(void);
static void TIM14Config(void);
static void KB_LedPoint(void);
static void KEY_Disposed(void);

/* Public function ---------------------------------------------------------------*/
/******************************************************************************
  * @brief  main program
  * @param
  * @note
  * @retval
  *****************************************************************************
*/
int main(void)
{
    NVIC_SetVectorTable();
    Systick_Init();
    USB_Init();
    Test_IO_Config();
    TIM14Config();

    while (1)
    {
        KEY_Disposed();

        KB_LedPoint();
//      #if 0
//          /*˯߻Ѳ*/
//      //1.remote wakeup:/*PA3Ϊ͵ƽʱʾа£ִлѲ*/
//      //2.other device wakeup:ͨ豸ꡢ̻USBģ
//          USB_Suspend();
//      #else
//          /*ԣȫ104*/
//          f_send_kb = 1;
//          usb_send_data();
//      #endif
    }
}
/******************************************************************************
  * @brief  KEY_Disposed program.
  * @param  None
  * @note   Setting address Offset
  * @retval None
  *****************************************************************************
*/
static void KEY_Disposed(void)
{
    uint8_t i = 0;
    static uint8_t key1_cnt = 0;
    static uint8_t key2_cnt = 0;

    /* scan key */
    key_scan[0] = GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_13);
    key_scan[1] = GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_11);

    key_scan[1] ^= 0x01;

    /* key disposed */
    for (i = 0; i < 2; i++)
    {
        if (old_scan[i] != key_scan[i])
        {
            old_scan[i] = key_scan[i];
            debounce_time[i] = 0;
        }
        else
        {
            if (debounce_time[i] >= 3)
            {
                if (old_scan[i] ^ new_scan[i])
                {
                    new_scan[i] = old_scan[i];

                    if (new_scan[i])
                    {
                        key_status = 0xff;
                    }
                    else
                    {
                        key_status = 0;
                    }

                    if (key_status)
                    {
                        if (i == 0)
                        {
                            if (kb_code_tab[key1_cnt][0] == 0x00)
                            {
                                //Common buttons
                                ep1_send_buf[0] = 0x00;
                                ep1_send_buf[1] = 0x00;
                                ep1_send_buf[2] = kb_code_tab[key1_cnt][1];
                                ep1_send_buf[3] = 0x00;
                            }
                            else if (kb_code_tab[key1_cnt][0] == 0x01)
                            {
                                //hotkey
                                ep1_send_buf[0] = kb_code_tab[key1_cnt][1];
                                ep1_send_buf[2] = 0x00;
                                ep1_send_buf[1] = 0x00;
                                ep1_send_buf[3] = 0x00;
                            }
                            key1_cnt++;
                            if (key1_cnt >= kb_size)
                            {
                                key1_cnt = 0;
                            }
                        }
                        else
                        {
                            if (kb_code_tab[key2_cnt][0] == 0x00)
                            {
                                //Common buttons
                                ep1_send_buf[0] = 0x00;
                                ep1_send_buf[1] = 0x00;
                                ep1_send_buf[2] = kb_code_tab[key2_cnt][1];
                                ep1_send_buf[3] = 0x00;
                            }
                            else if (kb_code_tab[key2_cnt][0] == 0x01)
                            {
                                //hotkey
                                ep1_send_buf[0] = kb_code_tab[key2_cnt][1];
                                ep1_send_buf[2] = 0x00;
                                ep1_send_buf[1] = 0x00;
                                ep1_send_buf[3] = 0x00;
                            }
                            key2_cnt++;
                            if (key2_cnt >= kb_size)
                            {
                                key2_cnt = 0;
                            }
                        }

                        USB_EP_Tx(3, ep1_send_buf, 8);
                    }
                    else
                    {
                        //free
                        ep1_send_buf[0] = 0x00;
                        ep1_send_buf[2] = 0x00;
                        ep1_send_buf[1] = 0x00;
                        ep1_send_buf[3] = 0x00;
                        USB_EP_Tx(3, ep1_send_buf, 8);
                    }
                }
            }
        }
    }
}
/******************************************************************************
  * @brief  KB_LedPoint program.
  * @param  None
  * @note   Setting address Offset
  * @retval None
  *****************************************************************************
*/
static void KB_LedPoint(void)
{
    uint8_t LedState = c_hid_data.wHID_SetRptByte[0];

    if (LedState & 0x01) //num lock
    {
        GPIO_ResetBits(GPIOB, GPIO_Pin_0);
    }
    else
    {
        GPIO_SetBits(GPIOB, GPIO_Pin_0);
    }

    if (LedState & 0x02) //cap lock
    {
        GPIO_ResetBits(GPIOB, GPIO_Pin_1);
    }
    else
    {
        GPIO_SetBits(GPIOB, GPIO_Pin_1);
    }

    if (LedState & 0x04) //scr lock
    {
        GPIO_SetBits(GPIOB, GPIO_Pin_3);
    }
    else
    {
        GPIO_ResetBits(GPIOB, GPIO_Pin_3);
    }
}
/******************************************************************************
  * @brief  NVIC_SetVectorTable program.
  * @param  None
  * @note   Setting address Offset
  * @retval None
  *****************************************************************************
*/
static void NVIC_SetVectorTable(void)
{
    uint8_t i;
    FLASH_Unlock();

    for (i = 0; i < 48; i++)
    {
        VectorTable[i] = *(__IO uint32_t *)(ApplicationAddress + (i << 2));
    }
    /* Enable the SYSCFG peripheral clock*/
    FLASH_Lock();
    /* Enable the SYSCFG peripheral clock*/
    RCC_APB2PeriphResetCmd(RCC_APB2Periph_SYSCFG, ENABLE);
    RCC_APB2PeriphResetCmd(RCC_APB2Periph_SYSCFG, DISABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

    /* Remap SRAM at 0x00000000 */
    SYSCFG_MemoryRemapConfig(SYSCFG_MemoryRemap_SRAM);
}

/******************************************************************************
  * @brief  TIM14Config program.
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
static void TIM14Config(void)
{
    NVIC_InitTypeDef NVIC_InitStructure;
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM14, ENABLE);
    TIM_DeInit(TIM14);
    TIM_TimeBaseStructure.TIM_Period = 10000;   //ARR value
    TIM_TimeBaseStructure.TIM_Prescaler = 48 - 1; //10ms
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInit(TIM14, &TIM_TimeBaseStructure);
    TIM_ITConfig(TIM14, TIM_IT_Update, ENABLE);
    TIM_Cmd(TIM14, ENABLE);

    NVIC_InitStructure.NVIC_IRQChannel = TIM14_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 3;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
}

/******************************************************************************
  * @brief  usb_send_data program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void usb_send_data(void)
{

    if (f_send_kb)
    {
        for (uint8_t i = 0; i < KEY_SIZE; i++)
        {
            if (kb_code_tab[i][0] == 0x00)
            {
                //
                ep1_send_buf[0] = 0x00;
                ep1_send_buf[1] = 0x00;
                ep1_send_buf[2] = kb_code_tab[i][1];
                ep1_send_buf[3] = 0x00;
            }
            else if (kb_code_tab[i][0] == 0x01)
            {
                ep1_send_buf[0] = kb_code_tab[i][1];
                ep1_send_buf[2] = 0x00;
                ep1_send_buf[1] = 0x00;
                ep1_send_buf[3] = 0x00;
            }

            if (USB_EP_Tx(3, ep1_send_buf, 8) != SUCCESS)
            {
                f_send_kb = 0;
            }

            SysTick_Delay_Ms(100);

            ep1_send_buf[0] = 0x00;
            ep1_send_buf[2] = 0x00;
            ep1_send_buf[1] = 0x00;
            ep1_send_buf[3] = 0x00;
            if (USB_EP_Tx(3, ep1_send_buf, 8) != SUCCESS)
            {
                f_send_kb = 0;
            }
            SysTick_Delay_Ms(100);
        }
    }
}
/******************************************************************************
  * @brief  Test_IO_Config program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void Test_IO_Config(void)
{
    GPIO_InitTypeDef GPIO_InitStruct;

    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);

    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_3;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;

    GPIO_Init(GPIOB, &GPIO_InitStruct);

    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_11;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOC, &GPIO_InitStruct);

    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_13;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_DOWN;
    GPIO_Init(GPIOC, &GPIO_InitStruct);

    GPIO_SetBits(GPIOB, GPIO_Pin_2);
}
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/
