/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_desc.c
* @author    : FMD AE
* @brief     : usb_desc program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "usb_desc.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
const uint8_t USB_FSDeviceDescriptor[] =
{
    /*---------------------- Device Descriptor ----------------------*/
    0x12,       /* bLength */
    0x01,       /* bDescriptorType */
    0x00,       /* bcdUSB L */
    0x02,       /* bcdUSB H */
    0x00,       /* bDeviceClass */
    0x00,       /* bDeviceSubClass */
    0x00,       /* bDeviceProtocol */
    0x08,       /* bMaxPacketSize0 */
    0xD3,       /* idVendor VID:34D3 */
    0x34,
    0x01,       /* idProduct */
    0x00,
    0x01,       /* bcdDevice */
    0x00,
    0x01,       /* iManufacturer */
    0x02,       /* iProduct */
    0x00,       /* iSerialNumber */
    0x01        /* bNumConfigurations */
};
const uint16_t USB_FSDeviceDescriptorSize = sizeof(USB_FSDeviceDescriptor);

const uint8_t USB_FSConfigDescriptor[] =
{
    /*------------------ Configuration Descriptor -------------------*/
    0x09,       /* bLength */
    0x02,       /* bDescriptorType */
    0x3B,       /* wTotalLength */
    0x00,
    0x02,       /* bNumInterfaces */
    0x01,       /* bConfigurationValue */
    0x03,       /* iConfiguration */
    0xA0,       /* bmAttributes */
    0x32,       /* MaxPower */
    /*---------------- Interface 0 Descriptor -------------------------*/
    0x09,       /* bLength */
    0x04,       /* bDescriptorType */
    0x00,       /* bInterfaceNumber */
    0x00,       /* bAlternateSetting */
    0x01,       /* bNumEndpoints */
    0x03,       /* bInterfaceClass */
    0x01,       /* bInterfaceSubClass */
    0x01,       /* bInterfaceProtocol */
    0x02,       /* iInterface */
    /*---------------- HID Descriptor -------------------------------*/
    0x09,       /* bLength */
    0x21,       /* bDescriptorType */
    0x10,       /* bcdHID */
    0x01,       /* bcdHID */
    0x00,       /* bCountryCode */
    0x01,       /* bNumDescriptors */
    0x22,       /* bDescriptorType */
    0x41,       /* wDescriptorLength */
    0x00,
    /*------------------- Endpoint Descriptor------------------------*/
    0x07,       /* bLength */
    0x05,       /* bDescriptorType */
    0x81,       /* bEndpointAddress */
    0x03,       /* bmAttributes */
    0x08,       /* wMaxPacketSize */
    0x00,       /* wMaxPacketSize */
    0x01,       /* bInterval  0x0a */
    /*------------------- Interface 1 Descriptor ---------------------*/
    0x09,       /* bLength */
    0x04,       /* bDescriptorType */
    0x01,       /* bInterfaceNumber */
    0x00,       /* bAlternateSetting */
    0x01,       /* bNumEndpoints */
    0x03,       /* bInterfaceClass */
    0x00,       /* bInterfaceSubClass */
    0x00,       /* bInterfaceProtocol */
    0x02,       /* iInterface */
    /*------------------------ HID Descriptor ---------------------*/
    0x09,       /* bLength */
    0x21,       /* bDescriptorType */
    0x10,       /* bcdHID */
    0x01,
    0x00,       /* bCountryCode */
    0x01,       /* bNumDescriptors */
    0x22,       /* bDescriptorType */
    0xB8,       /* wDescriptorLength */
    0x00,
    /*------------------------- Endpoint Descriptor ---------------*/
    0x07,       /* bLength */
    0x05,       /* bDescriptorType */
    0x82,       /* bEndpointAddress */
    0x03,       /* bmAttributes */
    0x04,       /* wMaxPacketSize */
    0x00,
    0xFF        /* bInterval */
};
const uint16_t USB_FSConfigDescriptorSize = sizeof(USB_FSConfigDescriptor);

const uint8_t LanguageId[] =
{
    0x04,       /* ĳ */
    0x03,       /* ַ */
    0x09,       /* 0x0409ΪʽӢID */
    0x04
};
const uint16_t LanguageIdSize = sizeof(LanguageId);

const uint8_t ManufacturerStringDescriptor[] =
{
    (3 * 2 + 2), /* length */
    0x03,       /* ַͱ0x03 */
    'F', 0,
    'M', 0,
    'D', 0,
};
const uint16_t ManufacturerStringDescriptorSize = sizeof(ManufacturerStringDescriptor);

const uint8_t ProductStringDescriptor[] =
{
    (12 * 2 + 2), /* length */
    0x03,       /* ַͱ0x03 */
    'F', 0,
    'M', 0,
    'D', 0,
    ' ', 0,

    'G', 0,
    'a', 0,
    'm', 0,
    'e', 0,
    ' ', 0,
    'P', 0,
    'a', 0,
    'd', 0
};
const uint16_t ProductStringDescriptorSize = sizeof(ProductStringDescriptor);

const uint8_t SerialNumberSringDescriptor[] =
{
    (10 * 2 + 2), /* length */
    0x03,       /* ַͱ0x03 */
    '2', 0,
    '0', 0,
    '2', 0,
    '0', 0,
    '-', 0,
    '0', 0,
    '6', 0,
    '-', 0,
    '2', 0,
    '8', 0,
};
const uint16_t SerialNumberSringDescriptorSize = sizeof(SerialNumberSringDescriptor);

const uint8_t USB_HID_FSReportDescriptor_1 [] =
{
    0x05, 0x01,           /* Usage Page (Generic Desktop Controls) */
    0x09, 0x06,           /* Usage (Keyboard) */
    0xA1, 0x01,           /* Collection (Application) */
    0x05, 0x07,           /* Usage Page (Keyboard) */
    0x19, 0xE0,           /* Usage Minimum (-32) */
    0x29, 0xE7,           /* Usage Maximum (-25) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x25, 0x01,           /* Logical Maximum (1) */
    0x75, 0x01,           /* Report Size (1) */
    0x95, 0x08,           /* Report Count (8) */
    0x81, 0x02,           /* Input (Var) */
    0x95, 0x01,           /* Report Count (1) */
    0x75, 0x08,           /* Report Size (8) */
    0x81, 0x01,           /* Input (Const) */
    0x95, 0x05,           /* Report Count (5) */
    0x75, 0x01,           /* Report Size (1) */
    0x05, 0x08,           /* Usage Page (LEDs) */
    0x19, 0x01,           /* Usage Minimum (1) */
    0x29, 0x05,           /* Usage Maximum (5) */
    0x91, 0x02,           /* Output (Var) */
    0x95, 0x01,           /* Report Count (1) */
    0x75, 0x03,           /* Report Size (3) */
    0x91, 0x01,           /* Output (Const) */
    0x95, 0x06,           /* Report Count (6) */
    0x75, 0x08,           /* Report Size (8) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x26, 0xFF, 0x00,     /* Logical Maximum (255) */
    0x05, 0x07,           /* Usage Page (Keyboard) */
    0x19, 0x00,           /* Usage Minimum (0) */
    0x2A, 0xFF, 0x00,     /* Usage Maximum (255) */
    0x81, 0x00,           /* Input () */
    0xC0                  /* End Collection */
};
const uint16_t USB_HID_FSReportDescriptor_1Size = sizeof(USB_HID_FSReportDescriptor_1);

const uint8_t USB_HID_FSReportDescriptor_2 [] =
{
    0x05, 0x0C,           /* Usage Page (Consumer) */
    0x09, 0x01,           /* Usage (Consumer Control) */
    0xA1, 0x01,           /* Collection (Application) */
    0x85, 0x01,           /* Report ID (0x01) */
    0x09, 0xE0,           /* Usage (Volume) */
    0x15, 0xE8,           /* Logical Minimum (-24) */
    0x25, 0x18,           /* Logical Maximum (24) */
    0x75, 0x07,           /* Report Size (7) */
    0x95, 0x01,           /* Report Count (1) */
    0x81, 0x06,           /* Input (Var, Rel) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x25, 0x01,           /* Logical Maximum (1) */
    0x75, 0x01,           /* Report Size (1) */
    0x09, 0xE2,           /* Usage (Mute) */
    0x81, 0x06,           /* Input (Var, Rel) */
    0xC0,                 /* End Collection */
    0x06, 0x01, 0x00,     /* Usage Page (Generic Desktop Controls) */
    0x09, 0x80,           /* Usage (System Control) */
    0xA1, 0x01,           /* Collection (Application) */
    0x85, 0x02,           /* Report ID (0x02) */
    0x25, 0x01,           /* Logical Maximum (1) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x75, 0x01,           /* Report Size (1) */
    0x0A, 0x81, 0x00,     /* Usage (System Power Down) */
    0x0A, 0x82, 0x00,     /* Usage (System Sleep) */
    0x0A, 0x83, 0x00,     /* Usage (System Wake Up) */
    0x95, 0x03,           /* Report Count (3) */
    0x81, 0x06,           /* Input (Var, Rel) */
    0x95, 0x05,           /* Report Count (5) */
    0x81, 0x01,           /* Input (Const) */
    0xC0,                 /* End Collection */
    0x06, 0x0C, 0x00,     /* Usage Page (Consumer) */
    0x09, 0x01,           /* Usage (Consumer Control) */
    0xA1, 0x01,           /* Collection (Application) */
    0x85, 0x03,           /* Report ID (0x03) */
    0x25, 0x01,           /* Logical Maximum (1) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x75, 0x01,           /* Report Size (1) */
    0x0A, 0xB5, 0x00,     /* Usage (Scan Next Track) */
    0x0A, 0xB6, 0x00,     /* Usage (Scan Previous Track) */
    0x0A, 0xB7, 0x00,     /* Usage (Stop) */
    0x0A, 0xB8, 0x00,     /* Usage (Eject) */
    0x0A, 0xCD, 0x00,     /* Usage (Play/Pause) */
    0x0A, 0xE2, 0x00,     /* Usage (Mute) */
    0x0A, 0xE9, 0x00,     /* Usage (Volume Increment) */
    0x0A, 0xEA, 0x00,     /* Usage (Volume Decrement) */
    0x95, 0x08,           /* Report Count (8) */
    0x81, 0x02,           /* Input (Var) */
    0x0A, 0x83, 0x01,     /* Usage (AL Consumer Control Configuration) */
    0x0A, 0x8A, 0x01,     /* Usage (AL Email Reader) */
    0x0A, 0x92, 0x01,     /* Usage (AL Calculator) */
    0x0A, 0x94, 0x01,     /* Usage (AL Local Machine Browser) */
    0x0A, 0x21, 0x02,     /* Usage (AC Search) */
    0x0A, 0x23, 0x02,     /* Usage (AC Home) */
    0x0A, 0x24, 0x02,     /* Usage (AC Back) */
    0x0A, 0x25, 0x02,     /* Usage (AC Forward) */
    0x95, 0x08,           /* Report Count (8) */
    0x81, 0x02,           /* Input (Var) */
    0x0A, 0x26, 0x02,     /* Usage (AC Stop) */
    0x0A, 0x27, 0x02,     /* Usage (AC Refresh) */
    0x0A, 0x2A, 0x02,     /* Usage (0x22A) */
    0x0A, 0xB3, 0x00,     /* Usage (Fast Forward) */
    0x0A, 0xB4, 0x00,     /* Usage (Rewind) */
    0x95, 0x05,           /* Report Count (5) */
    0x81, 0x02,           /* Input (Var) */
    0x95, 0x03,           /* Report Count (3) */
    0x81, 0x01,           /* Input (Const) */
    0xC0,                 /* End Collection */
    0x06, 0x00, 0xFF,     /* Usage Page (unk) */
    0x09, 0x01,           /* Usage (0x01) */
    0xA1, 0x01,           /* Collection (Application) */
    0x85, 0x05,           /* Report ID (0x05) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x26, 0xFF, 0x00,     /* Logical Maximum (255) */
    0x19, 0x01,           /* Usage Minimum (1) */
    0x29, 0x02,           /* Usage Maximum (2) */
    0x75, 0x08,           /* Report Size (8) */
    0x95, 0x05,           /* Report Count (5) */
    0xB1, 0x02,           /* Feature (Var) */
    0xC0                  /* End Collection */
};
const uint16_t USB_HID_FSReportDescriptor_2Size = sizeof(USB_HID_FSReportDescriptor_2);

/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/

