/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_usb.c
* @author    : FMD AE
* @brief     : bsp_usb program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "bsp_usb.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
uint8_t usb_state = 0;
/* Status for 2 Bulk In endpoints */
//M_EPBIN_STATUS  gepbin1, gepbin2, gepbin3;
/* Status for 2 Bulk Out endpoints */
M_EPBOUT_STATUS gepbout1, gepbout2, gepbout4;

/* Public variables --------------------------------------------------------------*/
VarBit  flag1;
uint8_t ep1_send_buf[64] = {0};
uint8_t ep2_send_buf[64] = {0};
/* Private function prototypes ---------------------------------------------------*/

/* Public function ---------------------------------------------------------------*/
/******************************************************************************
  * @brief  USB_Init program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_Init(void)
{
    NVIC_InitTypeDef NVIC_InitStructure;

    if (RCC_GetFlagStatus(RCC_FLAG_HSI48RDY) != SET)
    {
        RCC_HSI48Cmd(ENABLE);
        while (RCC_GetFlagStatus(RCC_FLAG_HSI48RDY) != SET);
    }
    RCC_USBCLKConfig(RCC_USBCLK_HSI48);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USB, ENABLE);


    USB_ITConfig(USB_IT_USB_RESUME_FLAG, ENABLE);
    USB_ITConfig(USB_IT_USB_SUSPEND_FLAG, ENABLE);
    USB_ITConfig(USB_IT_USB_SOF_FLAG, ENABLE);
    USB_ITConfig(USB_IT_USB_RESET_FLAG, ENABLE);

    USB_PDCTRLConfig(USB_PDCTRL_PU_ENABLE);

    NVIC_InitStructure.NVIC_IRQChannel = USB_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_CRS, ENABLE);
    CRS_FrequencyErrorCounterCmd(ENABLE);
    CRS_AutomaticCalibrationCmd(ENABLE);
}

/******************************************************************************
  * @brief  USB_FS_IRQHandler program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_FS_IRQHandler(void)
{
    uint8_t usb_intrin = 0, usb_intrusb = 0;

    usb_intrin = USB->INTRIN;                                   //端点中断标志
    usb_intrusb = USB->INTRUSB;                                 //USB中断标志RESET、RESUME、SUSPEND、SOF

    /*Check for resume from suspend mode, Add call to resume routine here */
    if ((usb_intrusb & 0X02) != RESET)                          //RESUME
    {
        usb_state = USB_RESUME_FLAG;
    }

    if (usb_intrusb & USB_IT_USB_SOF_FLAG)                      //SOF
    {
        USB->INTRUSB &= ~0x08;                                  //sof
        f_sof_flag = 1;
    }
    /* Check for system interrupts */
    if ((usb_intrusb & USB_IT_USB_RESET_FLAG) != RESET)
    {
        USB_Reset();
    }

    /* Check for endpoint 0 interrupt */
    if ((usb_intrin & USB_IT_IN_EP0_FLAG) != RESET)
    {
        USB_Endpoint0(M_EP_NORMAL);
    }

    /* Check for suspend mode */
    /* Add call to suspend routine here*/
    if ((usb_intrusb & 0x01) != RESET)
    {
        if (gnDevState > DEVSTATE_ADDRESS)
        {
            usb_state = USB_SUSPEND_FLAG;                       //usb suspend
        }
    }
}
/******************************************************************************
  * @brief  USB_Suspend program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_Suspend(void)
{
    if (usb_state == USB_SUSPEND_FLAG)
    {
        /*关闭一些时钟省电*/

        /*结束*/
        while (USB->POWER & 0x02)                               //suspend flag
        {
            GPIOB->ODR ^= GPIO_Pin_2;
            SysTick_Delay_Ms(500);
            if (GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_11) == RESET) //按键唤醒
            {
                /*remote wakeup*/
                USB_Remote_Wakeup();

                f_send_kb = 1;
                USB->POWER &= ~0X01;                            //不允许挂起模块
                usb_state = USB_NORMAL;
            }
        }
        /*重新配置时钟*/
    }
    else if (usb_state == USB_RESUME_FLAG)                      //自动退出suspend
    {
        usb_state = USB_NORMAL;
        GPIO_SetBits(GPIOB, GPIO_Pin_2);
        /*重新配置时钟*/
    }

    if (GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_11) == RESET)     //按键唤醒
    {
        SysTick_Delay_Ms(10);
        if (GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_11) == RESET) //按键唤醒
            f_send_kb = 1;
    }
    if (f_send_kb)
    {
        GPIO_SetBits(GPIOB, GPIO_Pin_2);
        SysTick_Delay_Ms(500);
        ep1_send_buf[2] = 0x29;
        if (USB_EP_Tx(1, ep1_send_buf, 8))
        {
            f_send_kb = 0;
        }
        ep1_send_buf[2] = 0x00;
        if (USB_EP_Tx(1, ep1_send_buf, 8))
        {
            USB->POWER |= 0X01;//允许挂起模块
            f_send_kb = 0;
        }
    }
}
/******************************************************************************
  * @brief  USB_Reset program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_Reset(void)
{
    M_EPBIN_STATUS  tEpbin;
    /* Set device into default state */
    gnDevState = DEVSTATE_DEFAULT;
//    pInformation->Current_Feature = USB_FSConfigDescriptor[7];
    /* Reset any state machines for each endpoint */
    /* This example assumes 2 bulk IN endpoints with endpoint numbers 1,2, */
    /* and 2 bulk out endpoints with endpoint numbers 1,2 */
    USB_Endpoint0(M_EP_RESET);

    /* Initialise endpoint status structures with endpoint numbers */

    tEpbin.byEP = 1;
    tEpbin.nBytesLeft = M_EP_NODATA;
//  tEpbin.pData = ep1_send_buf;
    EndpointBulkIn(tEpbin, M_EP_RESET);

    tEpbin.byEP = 2;
    tEpbin.nBytesLeft = M_EP_NODATA;
//  tEpbin.pData = ep2_send_buf;
    EndpointBulkIn(tEpbin, M_EP_RESET);

    USB->POWER |= 0X01;                                             //允许挂起模块
}
/******************************************************************************
  * @brief  USB_Remote_Wakeup program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_Remote_Wakeup(void)
{
    USB->POWER |= 0x04;
    SysTick_Delay_Ms(12);
    USB->POWER &= ~0X04;
}
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/

