/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_ep0.h
* @author    : FMD AE
* @brief     : Header for usb_ep0.c module
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

#ifndef _ENDPOINT0_H_
#define _ENDPOINT0_H_

/* Includes ---------------------------------------------------------------------*/
#include "ft32f0xx.h"
#include "usb_desc.h"
#include "usb_core.h"

/* Public Constant prototypes----------------------------------------------------*/

/* Public typedef ---------------------------------------------------------------*/
typedef struct
{
    /* When send data out of the device,
    CopyData() is used to get data buffer 'Length' bytes data
    if Length is 0,
    CopyData() returns the total length of the data
    if the request is not supported, returns 0
    (NEW Feature )
    if CopyData() returns -1, the calling routine should not proceed
    further and will resume the SETUP process by the class device
    if Length is not 0,
    CopyData() returns a pointer to indicate the data location
    Usb_wLength is the data remain to be sent,
    Usb_wOffset is the Offset of original data
    When receive data from the host,
    CopyData() is used to get user data buffer which is capable
    of Length bytes data to copy data from the endpoint buffer.
    if Length is 0,
    CopyData() returns the available data length,
    if Length is not 0,
    CopyData() returns user buffer address
    Usb_rLength is the data remain to be received,
    Usb_rPointer is the Offset of data buffer
    */
    uint16_t  Usb_wLength;
    uint16_t  Usb_wOffset;
    uint16_t  PacketSize;
    uint8_t   *(*CopyData)(uint16_t Length);
} ENDPOINT_INFO;

typedef struct
{
    uint8_t ControlState;               /* of type CONTROL_STATE */
    uint8_t Current_Feature;
    uint8_t Current_Configuration;      /* Selected configuration */
    uint8_t Current_Interface;          /* Selected interface of current configuration */
    uint8_t Current_AlternateSetting;   /* Selected Alternate Setting of current                                    interface*/
    ENDPOINT_INFO Ctrl_Info;
} DEVICE_INFO;
/* Public define ----------------------------------------------------------------*/
/* Endpoint 0 states */
#define M_EP0_IDLE                  0   //״̬
#define M_EP0_RX                    1
#define M_EP0_TX                    2

/* Define maximum packet size for endpoint 0 */
#define M_EP0_MAXP                  8

/* Define maximum number of interfaces in a configuration */
#define M_MAXIFS                    8

/* HID Request Types */
#define HID_REQUEST_SET             0x21
#define HID_REQUEST_GET             0xA1

/* HID Request Codes */
#define HID_REQUEST_GET_REPORT      0x01
#define HID_REQUEST_GET_IDLE        0x02
#define HID_REQUEST_GET_PROTOCOL    0x03
#define HID_REQUEST_SET_REPORT      0x09
#define HID_REQUEST_SET_IDLE        0x0A
#define HID_REQUEST_SET_PROTOCOL    0x0B

/* HID Report Types */
#define HID_REPORT_INPUT            0x01
#define HID_REPORT_OUTPUT           0x02
#define HID_REPORT_FEATURE          0x03

/* Define device states */
#define DEVSTATE_DEFAULT            0       //λ״̬
#define DEVSTATE_ADDRESS            1       //ַ״̬
#define DEVSTATE_CONFIG             2       //״̬
/* Define endpoint call status values */
#define M_EP_NORMAL                 0
#define M_EP_RESET                  1
/* Public variables prototypes --------------------------------------------------*/
extern DEVICE_INFO *pInformation;
extern uint32_t     gnDevState;
extern PSTD_IF_DSCR gpCurIf[M_MAXIFS];

/* Public function prototypes----------------------------------------------------*/
void USB_Endpoint0(int);
void USB_Remote_Wakeup(void);
ErrorStatus USB_EP_Tx(uint8_t Ep, uint8_t *ptr, uint8_t data_len);

#endif /* _ENDPOINT0_H_ */

/************************* (C) COPYRIGHT FMD *****END OF FILE*********************/

