/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_desc.c
* @author    : FMD AE
* @brief     : usb_desc program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "usb_desc.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
const uint8_t USB_FSDeviceDescriptor[] =
{
    /*---------------------- Device Descriptor ----------------------*/
    0x12,       /* bLength */
    0x01,       /* bDescriptorType */
    0x00,       /* bcdUSB L */
    0x02,       /* bcdUSB H */
    0x00,       /* bDeviceClass */
    0x00,       /* bDeviceSubClass */
    0x00,       /* bDeviceProtocol */
    0x08,       /* bMaxPacketSize0 */
    0xD3,       /* idVendor VID:34D3 */
    0x34,
    0x01,       /* idProduct */
    0x00,
    0x01,       /* bcdDevice */
    0x00,
    0x01,       /* iManufacturer */
    0x02,       /* iProduct */
    0x00,       /* iSerialNumber */
    0x01        /* bNumConfigurations */
};
const uint16_t USB_FSDeviceDescriptorSize = sizeof(USB_FSDeviceDescriptor);

const uint8_t USB_FSConfigDescriptor[] =
{
    /*------------------ Configuration Descriptor -------------------*/
    0x09,       /* bLength */
    0x02,       /* bDescriptorType */
    0x3b,       /* wTotalLength */
    0x00,
    0x02,       /* bNumInterfaces */
    0x01,       /* bConfigurationValue */
    0x03,       /* iConfiguration */
    0xA0,       /* bmAttributes */
    0x32,       /* MaxPower */
    /*---------------- Interface 0 Descriptor -------------------------*/
    0x09,       /* bLength */
    0x04,       /* bDescriptorType */
    0x00,       /* bInterfaceNumber */
    0x00,       /* bAlternateSetting */
    0x01,       /* bNumEndpoints */
    0x03,       /* bInterfaceClass */
    0x01,       /* bInterfaceSubClass */
    0x01,       /* bInterfaceProtocol */
    0x00,       /* iInterface */
    /*---------------- HID Descriptor -------------------------------*/
    0x09,       /* bLength */
    0x21,       /* bDescriptorType */
    0x10,       /* bcdHID */
    0x01,       /* bcdHID */
    0x00,       /* bCountryCode */
    0x01,       /* bNumDescriptors */
    0x22,       /* bDescriptorType */
    0x41,       /* wDescriptorLength */
    0x00,
    /*------------------- In Endpoint Descriptor------------------------*/
    0x07,       /* bLength */
    0x05,       /* bDescriptorType */
    0x81,       /* bEndpointAddress */
    0x03,       /* bmAttributes */
    0x08,       /* wMaxPacketSize */
    0x00,       /* wMaxPacketSize */
    0x01,       /* bInterval  0x0a */

    /*---------------- Interface 1 Descriptor -------------------------*/
    0x09,       /* bLength */
    0x04,       /* bDescriptorType */
    0x01,       /* bInterfaceNumber */
    0x00,       /* bAlternateSetting */
    0x01,       /* bNumEndpoints */
    0x03,       /* bInterfaceClass */
    0x01,       /* bInterfaceSubClass */
    0x02,       /* bInterfaceProtocol */
    0x00,       /* iInterface */
    /*---------------- HID Descriptor -------------------------------*/
    0x09,       /* bLength */
    0x21,       /* bDescriptorType */
    0x10,       /* bcdHID */
    0x01,
    0x00,       /* bCountryCode */
    0x01,       /* bNumDescriptors */
    0x22,       /* bDescriptorType */
    0x2E,       /* wDescriptorLength */
    0x00,
    /*------------------- Endpoint Descriptor------------------------*/
    0x07,       /* bLength */
    0x05,       /* bDescriptorType */
    0x82,       /* bEndpointAddress */
    0x03,       /* bmAttributes */
    0x04,       /* wMaxPacketSize */
    0x00,
    0x0A,       /* bInterval */

};
const uint16_t USB_FSConfigDescriptorSize = sizeof(USB_FSConfigDescriptor);

const uint8_t LanguageId[] =
{
    0x04,       /* ĳ */
    0x03,       /* ַ */
    0x09,       /* 0x0409ΪʽӢID */
    0x04
};
const uint16_t LanguageIdSize = sizeof(LanguageId);

const uint8_t ManufacturerStringDescriptor[] =
{
    (3 * 2 + 2), /* length */
    0x03,       /* ַͱ0x03 */
    'F', 0,
    'M', 0,
    'D', 0,
};
const uint16_t ManufacturerStringDescriptorSize = sizeof(ManufacturerStringDescriptor);

const uint8_t ProductStringDescriptor[] =
{
    (18 * 2 + 2), /* length */
    0x03,       /* ַͱ0x03 */
    'F', 0,
    'M', 0,
    'D', 0,
    ' ', 0,
    'K', 0,
    'e', 0,
    'y', 0,
    'b', 0,
    'o', 0,
    'a', 0,
    'r', 0,
    'd', 0,
    '&', 0,
    'M', 0,
    'o', 0,
    'u', 0,
    's', 0,
    'e', 0
};
const uint16_t ProductStringDescriptorSize = sizeof(ProductStringDescriptor);

const uint8_t SerialNumberSringDescriptor[] =
{
    (10 * 2 + 2), /* length */
    0x03,       /* ַͱ0x03 */
    '2', 0,
    '0', 0,
    '2', 0,
    '0', 0,
    '-', 0,
    '0', 0,
    '6', 0,
    '-', 0,
    '2', 0,
    '8', 0,
};
const uint16_t SerialNumberSringDescriptorSize = sizeof(SerialNumberSringDescriptor);

const uint8_t USB_HID_KeyboardReportDescriptor [] =
{
    0x05, 0x01,           /* Usage Page (Generic Desktop Controls) */
    0x09, 0x06,           /* Usage (Keyboard) */
    0xA1, 0x01,           /* Collection (Application) */
    0x05, 0x07,           /* Usage Page (Keyboard) */
    0x19, 0xE0,           /* Usage Minimum (-32) */
    0x29, 0xE7,           /* Usage Maximum (-25) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x25, 0x01,           /* Logical Maximum (1) */
    0x75, 0x01,           /* Report Size (1) */
    0x95, 0x08,           /* Report Count (8) */
    0x81, 0x02,           /* Input (Var) */
    0x95, 0x01,           /* Report Count (1) */
    0x75, 0x08,           /* Report Size (8) */
    0x81, 0x01,           /* Input (Const) */
    0x95, 0x05,           /* Report Count (5) */
    0x75, 0x01,           /* Report Size (1) */
    0x05, 0x08,           /* Usage Page (LEDs) */
    0x19, 0x01,           /* Usage Minimum (1) */
    0x29, 0x05,           /* Usage Maximum (5) */
    0x91, 0x02,           /* Output (Var) */
    0x95, 0x01,           /* Report Count (1) */
    0x75, 0x03,           /* Report Size (3) */
    0x91, 0x01,           /* Output (Const) */
    0x95, 0x06,           /* Report Count (6) */
    0x75, 0x08,           /* Report Size (8) */
    0x15, 0x00,           /* Logical Minimum (0) */
    0x26, 0xFF, 0x00,     /* Logical Maximum (255) */
    0x05, 0x07,           /* Usage Page (Keyboard) */
    0x19, 0x00,           /* Usage Minimum (0) */
    0x2A, 0xFF, 0x00,     /* Usage Maximum (255) */
    0x81, 0x00,           /* Input () */
    0xC0                  /* End Collection */
};
const uint16_t USB_HID_KeyboardReportDescriptorSize = sizeof(USB_HID_KeyboardReportDescriptor);

const uint8_t USB_HID_MouseReportDescriptor [] =
{
    0x05, 0x01,         //Usage Page (Generic Desktop)
    0x09, 0x02,         //Usage (Mouse)
    0xA1, 0x01,         //Collection (Application)
    0x09, 0x01,         //Usage (Pointer)
    0xA1, 0x00,         //Collection (Physical)
    0x05, 0x09,         //Usage Page (Button)
    0x19, 0x01,         //Usage Minimum (Button 1)
    0x29, 0x03,         //Usage Maximum (Button 3)
    0x15, 0x00,         //Logical Minimum (0)
    0x25, 0x01,         //Logical Maximum (1)
    0x95, 0x08,         //Report Count (8)
    0x75, 0x01,         //Report Size (1)
    0x81, 0x02,         //Input (Data,Var,Abs,NWrp,Lin,Pref,NNul,Bit)
    0x05, 0x01,         //Usage Page (Generic Desktop)
    0x09, 0x30,         //Usage (X)
    0x09, 0x31,         //Usage (Y)
    0x09, 0x38,         //Usage (Wheel)
    0x15, 0x81,         //Logical Minimum (-127)
    0x25, 0x7F,         //Logical Maximum (127)
    0x75, 0x08,         //Report Size (8)
    0x95, 0x03,         //Report Count (3)
    0x81, 0x06,         //Input (Data,Var,Rel,NWrp,Lin,Pref,NNul,Bit)
    0xC0,       //End Collection
    0xC0        //End Collection
};
const uint16_t USB_HID_MouseReportDescriptorSize = sizeof(USB_HID_MouseReportDescriptor);

/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/

