/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_keycode.c
* @author    : FMD AE
* @brief     : bsp_keycode program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "bsp_keycode.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/


const uint8_t kb_code_tab[][2] =
{
    //bit2
    0x00, 0x29,         // ESC
    0x00, 0x3A,         // F1
    0x00, 0x3B,         // F2
    0x00, 0x3C,         // F3
    0x00, 0x3D,         // F4
    0x00, 0x3E,         // F5
    0x00, 0x3f,         // F6
    0x00, 0x40,         // F7
    0x00, 0x41,         // F8
    0x00, 0x42,         // F9
    0x00, 0x43,         // F10
    0x00, 0x44,         // F11
    0x00, 0x45,         // F12
    0x00, 0x46,         // PRINT SCREEN
    0x00, 0x47,         // SCROLL LOCK
    0x00, 0x48,         // PAUSE
    0x00, 0x35,         // ~
    0x00, 0x1e,         // 1
    0x00, 0x1f,         // 2
    0x00, 0x20,         // 3
    0x00, 0x21,         // 4
    0x00, 0x22,         // 5
    0x00, 0x23,         // 6
    0x00, 0x24,         // 7
    0x00, 0x25,         // 8
    0x00, 0x26,         // 9
    0x00, 0x27,         // 0
    0x00, 0x2d,         // -
    0x00, 0x2e,         // =
    0x00, 0x2a,         // BACKSPACE
    0x00, 0x49,         // INSERT
    0x00, 0x4a,         // HOME
    0x00, 0x4b,         // PAGE UP
    0x00, 0x53,         // NUM LOCK
    0x00, 0x54,         // /
    0x00, 0x55,         // *
    0x00, 0x56,         // -
    0x00, 0x2b,         // TAB
    0x00, 0x14,         // Q
    0x00, 0x1a,         // W
    0x00, 0x08,         // E
    0x00, 0x15,         // R
    0x00, 0x17,         // T
    0x00, 0x1c,         // Y
    0x00, 0x18,         // U
    0x00, 0x0c,         // I
    0x00, 0x12,         // O
    0x00, 0x13,         // P
    0x00, 0x2f,         // [
    0x00, 0x30,         // ]
    0x00, 0x31,         /* \*/
    0x00, 0x4c,         // DELETE
    0x00, 0x4d,         // END
    0x00, 0x4e,         // PAGE DOWN
    0x00, 0x5f,         // NUM 7
    0x00, 0x60,         // NUM 8
    0x00, 0x61,         // NUM 9
    0x00, 0x57,         // NUM +
    0x00, 0X39,         // CAP
    0x00, 0x04,         // A
    0x00, 0x16,         // S
    0x00, 0x07,         // D
    0x00, 0X09,         // F
    0x00, 0X0A,         // G
    0x00, 0X0B,         // H
    0x00, 0X0D,         // J
    0x00, 0X0E,         // K
    0x00, 0X0F,         // L
    0x00, 0X33,         // ;
    0x00, 0X34,         // '
    0x00, 0X28,         // ENTER
    0x00, 0X5C,         // NUM 4
    0x00, 0X5D,         // NUM5
    0x00, 0X5E,         // NUM 6
    0x01, 0x02,         // SHIFT L  bit0
    0x00, 0x1d,         // Z
    0x00, 0x1b,         // X
    0x00, 0x06,         // C
    0x00, 0x19,         // V
    0x00, 0x05,         // B
    0x00, 0x11,         // N
    0x00, 0x10,         // M
    0x00, 0x36,         // ,
    0x00, 0x37,         // .
    0x00, 0x38,         // /
    0x01, 0x20,         // SHIFT R bit0
    0x00, 0x52,         // UP
    0x00, 0x59,         // NUM 1
    0x00, 0x5a,         // NUM 2
    0x00, 0x5b,         // NUM 3
    0x00, 0x58,         // NUM ENTER
    0x01, 0x01,         // CTRL L bit0
    0x01, 0x08,         // WIN L bit0
    0x01, 0x04,         // ALT_L bit0
    0x00, 0x2c,         // SPACE bit0
    0x01, 0x40,         // ALT R bit0
    0x01, 0x80,         // WIN R bit0
    0x00, 0x65,         // APP
    0x01, 0x10,         // CTRL R bit0
    0x00, 0x50,         // LEFT
    0x00, 0x51,         // DOWN
    0x00, 0x4f,         // RIGHT
    0x00, 0x62,         // NUM 0
    0x00, 0x63,         // NUM .
};
const uint8_t kb_size = sizeof(kb_code_tab) / 2;

const uint8_t mouse_code_tab[][2] =
{
    //byte0
    0x01, 0x01,         // 
    0x01, 0x04,         // м
    0x01, 0x02,         // Ҽ

    //X/Y/ֵĹȡֵΧ-127~127
    //byte1
    0x02, 0x01,         // X
    0x02, 0xff,         // X
    //byte2
    0x03, 0x01,         // Y
    0x03, 0xff,         // Y
    //byte3
    0x04, 0x01,         // Ϲ
    0x04, 0xff,         // ¹
};

const uint8_t ms_size = sizeof(mouse_code_tab) / 2;

/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
/* Public function ------ --------------------------------------------------------*/

/************************** (C) COPYRIGHT FMD **********************END OF FILE****/

