/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_usb.c
* @author    : FMD AE
* @brief     : bsp_usb program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/

#include "bsp_usb.h"
#include "main.h"

extern int  gnDevState;

/**********************************************************************************
  * @brief  USB_Init
    * @param  none
    * @note
  * @retval None
  *********************************************************************************
*/
void USB_Init(void)
{
    NVIC_InitTypeDef NVIC_InitStructure;

    RCC_USBCLKConfig(RCC_USBCLK_PLLCLK);

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USB, ENABLE);
    USB_ITConfig(USB_IT_USB_RESUME_FLAG, ENABLE);
    USB_ITConfig(USB_IT_USB_SUSPEND_FLAG, ENABLE);
    USB_ITConfig(USB_IT_USB_SOF_FLAG, ENABLE);
    USB_ITConfig(USB_IT_USB_RESET_FLAG, ENABLE);
    USB_PDCTRLConfig(USB_PDCTRL_PU_ENABLE);

    NVIC_InitStructure.NVIC_IRQChannel = USB_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
}

/**********************************************************************************
  * @brief  Set_Sysclk_48M
    * @param  none
    * @note
  * @retval None
  *********************************************************************************
*/
void Set_Sysclk_48M(void)
{

    //flash unlock
    FLASH ->KEYR = 0x45670123;
    FLASH ->KEYR = 0xcdef89ab;

    FLASH ->OPTKEYR = 0x45670123;
    FLASH ->OPTKEYR = 0xcdef89ab;
    FLASH ->OPTKEYR = 0x6100beef;   //USER OPTION

    *(uint32_t *)(0x40022000 + 0x38c) = (uint32_t)(0x00 << 21); //PLL FACT OPTION

    FLASH->ACR = FLASH_ACR_PRFTBE | FLASH_ACR_LATENCY;

    RCC_HSICmd(ENABLE);


    while (RCC_GetFlagStatus(RCC_FLAG_HSIRDY) != SET);

    RCC_PLLConfig(RCC_PLLSource_HSI, RCC_PLLMul_6);

    RCC_PLLCmd(ENABLE);

    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    /*PLL as sysclk*/
    while (RCC_GetSYSCLKSource() != 0x08);

    RCC_HCLKConfig(RCC_SYSCLK_Div1);

    RCC_PCLKConfig(RCC_HCLK_Div1);

    /*renew SystemCoreClock*/
    SystemCoreClockUpdate();

}


/**********************************************************************************
  * @brief  USB_PDCTRLConfig
    * @param  PDCT:
    * @note
  * @retval None
  *********************************************************************************
*/
void USB_PDCTRLConfig(uint8_t PDCT)
{
    uint8_t temp = 0;

    assert_param(IS_USB_PDCTRL(PDCT));

    temp = USB->PDCTRL;
    temp &= ~USB_PDCTRL_FLAG;
    temp |= PDCT;
    USB->PDCTRL = temp;

}



/**********************************************************************************
  * @brief  USB_ITConfig
    * @param  USB_IT:
    * @param  NewState:
    * @note
  * @retval None
  *********************************************************************************
*/
void USB_ITConfig(uint32_t USB_IT, FunctionalState NewState)
{
    __IO uint8_t *tmp = (__IO uint8_t *)USB_BASE;
    uint32_t IT_FLAG = 0;

    assert_param(IS_USB_CONFIG_IT(USB_IT));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (USB_IT & 0x00000100)
    {
        tmp = &USB->INTRINE;
        IT_FLAG = USB_IT & 0xfffffeff;
    }
    else if (USB_IT & 0x00000200)
    {
        tmp = &USB->INTROUTE;
        IT_FLAG = USB_IT & 0xfffffdff;
    }
    else if (USB_IT & 0x00000400)
    {
        tmp = &USB->INTRUSBE;
        IT_FLAG = USB_IT & 0xfffffbff;
    }

    if (NewState != DISABLE)
    {
        *((__IO uint8_t *)tmp) |= (uint8_t)IT_FLAG;
    }
    else
    {
        *((__IO uint8_t *)tmp) &= ~((uint8_t)IT_FLAG);
    }
}


/**********************************************************************************
  * @brief  send data by usb
    * @param  Ep:   port
    * @param  ptr:
    * @param  data_len:    port
    * @note   Send data no more than 64 bytes
  * @retval None
  *********************************************************************************
*/
ErrorStatus USB_EP_Tx2(uint8_t Ep, uint8_t *ptr, uint8_t data_len)
{
    uint32_t timeout = 0;

    if (gnDevState > DEVSTATE_ADDRESS)
    {
        gepbin2.byEP = Ep;
        gepbin2.nBytesLeft = data_len;
        gepbin2.pData = ptr;

        EndpointBulkIn(&gepbin2, M_EP_NORMAL);
        while (MREAD_BYTE(M_REG_INCSR1) & 0x01)
        {
            timeout++;
            if (timeout >= 0xffffffff)
            {
                return ERROR;
            }
        }
        return SUCCESS;
    }

    return ERROR;
}

/**********************************************************************************
  * @brief  send data by usb
    * @param  buf:
    * @param  len:
    * @param  Ep:    port
    * @note   The transmitted data can be greater than 64 bytes
  * @retval None
  *********************************************************************************
*/
void usb_send_data(uint8_t *buf, uint32_t len, uint8_t Ep)
{
    uint32_t nBytes = 0;

    while (len)
    {
        if (len > 64)
        {
            nBytes = 64;
            len -= 64;
        }
        else
        {
            nBytes = len;
            len = 0;
        }
        if (nBytes)
        {
            USB_EP_Tx2(Ep, buf, nBytes);
            buf += nBytes;
        }
    }
}
