/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_core.h
* @author    : FMD AE
* @brief     : Header for usb_core.c module
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

#ifndef _USB_CORE_H_
#define _USB_CORE_H_

#include "usb_desc.h"
#include "usb_ep0.h"
#include "usb_ep.h"
#include "config.h"
#include "ft32f0xx.h"
#include "main.h"
/* Define MUSBFSFC register addresses */
#define BASE_ADDR 0x40005c00


#define USE_EP_N(X)     X



#define M_REG_FADDR     BASE_ADDR
#define M_REG_POWER     BASE_ADDR+1
#define M_REG_INTRIN1   BASE_ADDR+2
#define M_REG_INTRIN2   BASE_ADDR+3
#define M_REG_INTROUT1  BASE_ADDR+4
#define M_REG_INTROUT2  BASE_ADDR+5
#define M_REG_INTRUSB   BASE_ADDR+6
#define M_REG_INTRIN1E  BASE_ADDR+7
#define M_REG_INTRIN2E  BASE_ADDR+8
#define M_REG_INTROUT1E BASE_ADDR+9
#define M_REG_INTROUT2E BASE_ADDR+10
#define M_REG_INTRUSBE  BASE_ADDR+11
#define M_REG_FRAME1    BASE_ADDR+12
#define M_REG_FRAME2    BASE_ADDR+13
#define M_REG_INDEX     BASE_ADDR+14

#define M_REG_CSR0      BASE_ADDR+17
#define M_REG_INMAXP    BASE_ADDR+16
#define M_REG_INCSR1    BASE_ADDR+17
#define M_REG_INCSR2    BASE_ADDR+18
#define M_REG_OUTMAXP   BASE_ADDR+19
#define M_REG_OUTCSR1   BASE_ADDR+20
#define M_REG_OUTCSR2   BASE_ADDR+21
#define M_REG_OUTCOUNT1 BASE_ADDR+22
#define M_REG_OUTCOUNT2 BASE_ADDR+23

#define M_FIFO_EP0      BASE_ADDR+32


/* Interrupt register bit masks */
#define M_INTR_SUSPEND  0x01
#define M_INTR_RESUME   0x02
#define M_INTR_RESET    0x04
#define M_INTR_EP0      0x0001

/* CSR0 bit masks */
#define M_CSR0_OUTPKTRDY        0x01
#define M_CSR0_INPKTRDY         0x02
#define M_CSR0_SENTSTALL        0x04
#define M_CSR0_DATAEND          0x08
#define M_CSR0_SETUPEND         0x10
#define M_CSR0_SENDSTALL        0x20
#define M_CSR0_SVDOUTPKTRDY     0x40
#define M_CSR0_SVDSETUPEND      0x80

/* Endpoint CSR register bits */
#define M_INCSR2_ISO    0x40
#define M_INCSR2_MODE   0x20
#define M_INCSR_CDT     0x40
#define M_INCSR_FF      0x08
#define M_INCSR_IPR     0x01
#define M_OUTCSR2_ISO   0x40
#define M_OUTCSR_CDT    0x80
#define M_OUTCSR_FF     0x10
#define M_OUTCSR_OPR    0x01

/* Register read/write macros */
#define MREAD_BYTE(addr) *((BYTE *)(addr))
#define MWRITE_BYTE(addr,data) *((BYTE *)(addr)) = data

/* Define endpoint call status values */
#define M_EP_NORMAL     0
#define M_EP_RESET      1

/* Define device states */
#define DEVSTATE_DEFAULT        0
#define DEVSTATE_ADDRESS        1
#define DEVSTATE_CONFIG         2

/* Define standard constants */
#define TRUE  1
#define FALSE 0
#ifndef NULL
    #define NULL 0
#endif

/* Function prototypes */
void FIFORead(int, int, void *);
void FIFOWrite(int, int, void *);
void USB_Reset(void);
void USB_Endpoint0(int);
extern ErrorStatus USB_EP_Tx(uint8_t Ep, uint8_t *ptr, uint8_t data_len);

extern M_EPBIN_STATUS  gepbin1;
extern M_EPBOUT_STATUS gepbout1;

extern M_EPBIN_STATUS  gepbin2;

void USB_FS_IRQHandler(void);
void USB_Suspend(void);
void USB_Resume(void);


#endif

