/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_ep0.h
* @author    : FMD AE
* @brief     : Header for usb_ep0.c module
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

#ifndef _ENDPOINT0_H_
#define _ENDPOINT0_H_


#include "ft32f0xx.h"
#include "main.h"

/* Endpoint 0 states */
#define M_EP0_IDLE      0
#define M_EP0_RX        1
#define M_EP0_TX        2

/* HID Request Types */
#define HID_REQUEST_SET                         0x21
#define HID_REQUEST_GET                         0xA1

/* HID Request Codes */
#define HID_REQUEST_GET_REPORT                  0x01
#define HID_REQUEST_GET_IDLE                    0x02
#define HID_REQUEST_GET_PROTOCOL                0x03
#define HID_REQUEST_SET_REPORT                  0x09
#define HID_REQUEST_SET_IDLE                    0x0A
#define HID_REQUEST_SET_PROTOCOL                0x0B

/* HID Report Types */

#undef HID_REPORT_INPUT
#undef HID_REPORT_OUTPUT
#undef HID_REPORT_FEATURE

#define HID_REPORT_INPUT                        0x01
#define HID_REPORT_OUTPUT                       0x02
#define HID_REPORT_FEATURE                      0x03

/*usb_state FLAG*/
#define USB_NORMAL          0x00
#define USB_SUSPEND_FLAG    0x01
#define USB_RESUME_FLAG     0x02

/* Endpoint 0 status structure */
typedef struct M_EP0_STATUS
{
    int   nState;         /* IDLE/RX/TX */
    int   nBytesLeft;     /* Number of bytes left to send in TX mode */
    void *pData;          /* Pointer to data to transmit/receive */
    int   nBytesRecv;     /* Number of bytes received in RX mode */
    BYTE  byFAddr;        /* New function address */
} M_EP0_STATUS;
typedef M_EP0_STATUS *PM_EP0_STATUS;

extern CLASS_HID_DATA   c_hid_data;;
extern uint8_t  usb_state;



void USB_EP_RRx(uint8_t Ep);

#endif /* _ENDPOINT0_H_ */

