/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_core.c
* @author    : FMD AE
* @brief     : usb_core program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/

#include "usb_core.h"
#include "USB_userHId.h"
#include "uart.h"



extern  uint16_t USBcnt;
/* Device descriptor is global */
STD_DEV_DSCR            stddevdsc;

/* Configuration structures as defined in config.h */
M_CFG_1 cfg1;
M_CFG_2 cfg2;
M_CFG   cfg;

/* Current device state */
int gnDevState = DEVSTATE_DEFAULT;
/* Pointer to current configuration descriptor */
void *gpCurCfg = NULL;
/* Pointer to current interface descriptors */
PSTD_IF_DSCR    gpCurIf[M_MAXIFS];
/* Current alternate interface values */
BYTE    gbyCurIfVal[M_MAXIFS];
/* Status for 2 Bulk In endpoints */
M_EPBIN_STATUS  gepbin1;
/* Status for 2 Bulk Out endpoints */
M_EPBOUT_STATUS gepbout1;


M_EPBIN_STATUS  gepbin2;


extern  M_EPBOUT_BUF gBulkoutData2;

/**********************************************************************************
  * @brief  Top level interrupt handler
  * @param  None
    * @note   USB receives data via interrupt
  * @retval None
  *********************************************************************************
*/
void USB_FS_IRQHandler(void)
{
    uint8_t usb_intrin = 0, usb_intrusb = 0, usb_introut = 0;

    usb_intrin = USB->INTRIN;
    usb_intrusb = USB->INTRUSB;
    usb_introut = USB->INTROUT;
    /*Check for resume from suspend mode, Add call to resume routine here */
    if ((usb_intrusb & 0X02) != RESET)
    {
        usb_state = USB_RESUME_FLAG;
    }

    if (usb_intrusb & USB_IT_USB_SOF_FLAG)
    {
        USB->INTRUSB &= ~0x08;//sof
    }
    /* Check for system interrupts */
    if ((usb_intrusb & USB_IT_USB_RESET_FLAG) != RESET)
    {
        USB_Reset();
    }

    /* Check for endpoint 0 interrupt */
    if ((usb_intrin & USB_IT_IN_EP0_FLAG) != RESET)
    {
        USB_Endpoint0(M_EP_NORMAL);
    }


    if ((usb_introut & USB_IT_OUT_EP1_FLAG) != RESET) //receive data by port1
    {

        Usb_Receive_Data();
    }


    if ((usb_introut & USB_IT_OUT_EP5_FLAG) != RESET) //receive data by port5
    {
        gepbout1.byEP = 5;
        if (gBulkoutData2.recvStatus == 0)
        {
            gepbout1.nBuffLen = MAX_COM_BUFFER_SIZE;
            gepbout1.nBytesRecv = 0;
            gepbout1.pData = gBulkoutData2.recvBuf;
            EndpointBulkOut(&gepbout1, M_EP_NORMAL);
            gBulkoutData2.recvNum = gepbout1.nBytesRecv;
            gBulkoutData2.recvStatus = 1;
            USBcnt = 0;
        }
        else if (gBulkoutData2.recvStatus)
        {
            gepbout1.nBuffLen = MAX_COM_BUFFER_SIZE - gBulkoutData2.recvNum;
            gepbout1.nBytesRecv = 0;
            gepbout1.pData = &gBulkoutData2.recvBuf[gBulkoutData2.recvNum];
            EndpointBulkOut(&gepbout1, M_EP_NORMAL);
            gBulkoutData2.recvNum += gepbout1.nBytesRecv;
            USBcnt = 0;
        }
        else
        {
            gepbout1.nBuffLen = 0;
            gepbout1.nBytesRecv = 0;
            EndpointBulkOut(&gepbout1, M_EP_NORMAL);
        }
    }

    /* Check for suspend mode */
    /* Add call to suspend routine here*/
    if ((usb_intrusb & 0x01) != RESET)
    {
        if (gnDevState > DEVSTATE_ADDRESS)
        {
            usb_state = USB_SUSPEND_FLAG;       //usb suspend
        }
    }
}

/**********************************************************************************
  * @brief  Reset USB
  * @param  None
    * @note   suspend module
  * @retval None
  *********************************************************************************
*/

void USB_Reset(void)
{
    /* Set device into default state */
    gnDevState = DEVSTATE_DEFAULT;

    /* Reset any state machines for each endpoint */
    /* This example assumes 2 bulk IN endpoints with endpoint numbers 1,2, */
    /* and 2 bulk out endpoints with endpoint numbers 1,2 */
    USB_Endpoint0(M_EP_RESET);

//  /* Initialise endpoint status structures with endpoint numbers */

    /* Initialise endpoint status structures with endpoint numbers */
    gepbout1.byEP = USE_EP_N(1);
    gepbout1.nBuffLen = M_EP_MAXP;
    gepbout1.nBytesRecv = 0;

    gepbin1.byEP = USE_EP_N(2);
    gepbin1.nBytesLeft = M_EP_NODATA;

    EndpointBulkOut(&gepbout1, M_EP_RESET);
    EndpointBulkIn(&gepbin1, M_EP_RESET);

    USB->POWER |= 0X01;//suspend module
}



/**********************************************************************************
  * @brief  Read data through port.
  * @param  nEP :  port number
    * @param  nBytes : data numbers
    * @param  pDst   : data buffer
    * @note
  * @retval None
  *********************************************************************************
*/
void FIFORead(int nEP, int nBytes, void *pDst)
{
    int     nCount;
    BYTE   *pby;
    int     nAddr;

    if (nBytes)
    {
        nAddr = M_FIFO_EP0 + (nEP << 2);
        nCount = nBytes;
        pby = (BYTE *)pDst;
        while (nCount)
        {
            *pby++ = *((BYTE *)nAddr);
            nCount--;
        }
    }

    return;
}


/****************************************
  FIFOWrite
****************************************/
/**********************************************************************************
  * @brief  Write data through port.
  * @param  nEP :  port number
    * @param  nBytes : data numbers
    * @param  pSrc   : data buffer
    * @note
  * @retval None
  *********************************************************************************
*/
void FIFOWrite(int nEP, int nBytes, void *pSrc)
{
    int     nCount;
    BYTE   *pby;
    int     nAddr;

    if (nBytes)
    {
        nAddr = M_FIFO_EP0 + (nEP << 2);
        nCount = nBytes;
        pby = (BYTE *)pSrc;
        while (nCount)
        {
            *((BYTE *)nAddr) = *pby++;
            nCount--;
        }
    }

    return;

}



