/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_desc.c
* @author    : FMD AE
* @brief     : usb_desc program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/

#include "usb_desc.h"

const uint8_t USB_FSDeviceDescriptor[] =
{
    /*---------------------- Device Descriptor ----------------------*/
    0x12,           // bLength             the number of bytes the descriptor contains
    0x01,           // bDescriptorType      the type of device descriptor
    0x00,           // bcdUSB  L
    0x02,           // bcdUSB  H            The device complies with the USB protocol version number followed
    0x00,           // bDeviceClass
    0x00,           // bDeviceSubClass
    0x00,           // bDeviceProtocol      3bytes The allocated by usb-if is generally 0
    0x08,           // bMaxPacketSize0      The maximum package length of EP0 must be 64
    0xD3,           // idVendor  L
    0x34,           // idVendor     H
    0x00,           // idProduct L
    0x11,             // idProduct H

    0x00,           // bcdDevice L
    0x02,           // bcdDevice H          The version number of the device itself
    0x01,           // iManufacturer        The index number of the character descriptor used to describe the manufacturer
    0x02,           // iProduct             The index number of the character description used to describe the product
    0x03,           // iSerialNumber        The index number of the character descriptor used to describe the product serial number
    0x01            // bNumConfigurations   Number of configurations supported by the device:1

};
const uint16_t USB_FSDeviceDescriptorSize = sizeof(USB_FSDeviceDescriptor);

const uint8_t USB_FSConfigDescriptor[] =
{
    /*------------------ Configuration Descriptor -------------------*/
    0x09,           // bLength
    0x02,           // bDescriptorType
    0x73, 0x00,     // wTotalLength

    0x03,           // bNumInterfaces       1
    0x01,           // 0x01,   //bConfigurationValue
    0x00,           // iConfiguration
    0xC0,           // bmAttributes
    0x32,           // MaxPower              50X2=100mA
    /*--------------------------IAD functio 1 Descriptor---------------------*/
    0x08,           // bLength：IAD
    0x0B,           // bDescriptorType：IAD
    0x00,           // bFirstInterface：
    0x01,           // bInferfaceCount：
    0x03,           // bFunctionClass：
    0x00,           // bFunctionSubClass：
    0x00,           // bFunctionProtocol：
    0x00,           // iFunction：
    //
    /*---------------- Interface 0 Descriptor -------------------------*/
    0x09,           // bLength
    0x04,           // bDescriptorType
    0x00,           // bInterfaceNumber
    0x00,           // bAlternateSetting
    0x02,           // bNumEndpoints
    0x03,           // bInterfaceClass
    0x00,           // bInterfaceSubClass    1=BOOT,0=no boot
    0x00,           // bInterfaceProtocol   Protocols supported by this interface: 0 custom, 1 keyboard, 2 mouse
    0x00,           // iInterface
    /*---------------- HID Descriptor -------------------------------*/
    0x09,           // bLength
    0x21,           // bDescriptorType   /* bDescriptorType: HID */
    0x10,           // bcdHID
    0x01,           // bcdHID
    0x00,           // bCountryCode
    0x01,           // bNumDescriptors
    0x22,           // bDescriptorType
    0x22,           // 0x21,   //wDescriptorLength  /* wItemLength: Total length of Report descriptor */
    0x00,
    /*------------------- Endpoint Descriptor------------------------*/
    0x07,           // bLength
    0x05,           // bDescriptorType
    0x82,           // bEndpointAddress
    0x03,           // bmAttributes
    0x40,           // wMaxPacketSize
    0x00,           // wMaxPacketSize
    0x02,           // 0x02,   //bInterval   ：2ms
    /*------------------- Endpoint Descriptor------------------------*/
    0x07,           // bLength
    0x05,           // bDescriptorType
    0x01,           // bEndpointAddress
    0x03,           // bmAttributes
    0x40,           // wMaxPacketSize
    0x00,           // wDescriptorLength
    0x02,           // bInterval 0x02,       ：2ms
    /*------------------- IAD2 Descriptor ------------------------*/
    0x08,           /*  bLength  */
    0x0B,           /*  bDescriptorType*/
    0x01,           /*  bFirstInterface*/
    0x02,           /*  bInterfaceCount*/
    0x02,           /*  bFunctionClass --CDC*/
    0x02,           /*  bFunctionSubClass*/
    0x01,           /*  bFunctionProtocoll*/
    0x00,           /*  iFunction */
    /*---------- CDC Interface(Interface 1) Descriptor ------------------*/
    0x09,           // bLength: Interface Descriptor size
    0x04,           // bDescriptorType: Interface
    0x01,           // bInterfaceNumber: Number of Interface
    0x00,           // bAlternateSetting: Alternate setting
    0x01,           // bNumEndpoints: One endpoints used
    0x02,           // bInterfaceClass: Communication Interface Class
    0x02,           // bInterfaceSubClass: Abstract Control Model
    0x01,           // bInterfaceProtocol: Common AT commands
    0x00,           // iInterface:
    /*------------------  Header Functional Descriptor ------------------*/
    0x05,           // bLength: Endpoint Descriptor size
    0x24,           // bDescriptorType: CS_INTERFACE
    0x00,           // bDescriptorSubtype: Header Func Desc
    0x10,           // bcdCDC: spec release number
    0x01,           // 2021
    /*---------------- Call Management Functional Descriptor -------------*/
    0x05,           // bFunctionLength */
    0x24,           // bDescriptorType: CS_INTERFACE */
    0x01,           // bDescriptorSubtype: Call Management Func Desc */
    0x00,           // bmCapabilities: D0+D1 */
    0x00,           // bDataInterface: 1 */
    /*-------------------- ACM Functional Descriptor ---------------------*/
    0x04,           // bFunctionLength */
    0x24,           // bDescriptorType: CS_INTERFACE */
    0x02,           // bDescriptorSubtype: Abstract Control Management desc */
    0x02,           // bmCapabilities */
    /*-------------------- Union Functional Descriptor -------------------*/
    0x05,           // bFunctionLength */
    0x24,           // bDescriptorType: CS_INTERFACE */
    0x06,           // bDescriptorSubtype: Union func desc */
    0x00,           // bMasterInterface: Communication class interface */
    0x01,           // bSlaveInterface0: Data Class Interface */
    /*------------------- Endpoint1 Descriptor------------------------*/
    0x07,           // bLength
    0x05,           // bDescriptorType
    0x83,           // bEndpointAddress
    0x03,           // bmAttributes
    0x08,           // wMaxPacketSize
    0x00,           // wMaxPacketSize
    0x01,           // bInterval  0x0a
    /*------------------- Interface 1 Descriptor ---------------------*/
    0x09,           // bLength
    0x04,           // bDescriptorType
    0x02,           // bInterfaceNumber
    0x00,           // bAlternateSetting
    0x02,           // bNumEndpoints
    0x0A,           // bInterfaceClass
    0x00,           // bInterfaceSubClass
    0x00,           // bInterfaceProtocol
    0x00,           // iInterface
    /*------------------------- Endpoint2 Descriptor ---------------*/
    0x07,           // bLength
    0x05,           // bDescriptorType
    0x84,           // bEndpointAddress
    0x02,           // bmAttributes
    0x40,           // wMaxPacketSize
    0x00,
    0x00,           // bInterval
    /*------------------------- Endpoint3 Descriptor ---------------*/
    0x07,           // bLength
    0x05,           // bDescriptorType
    0x05,           // bEndpointAddress
    0x02,           // bmAttributes
    0x40,           // wMaxPacketSize
    0x00,
    0x00,           // bInterval
};


const uint16_t USB_FSConfigDescriptorSize = sizeof(USB_FSConfigDescriptor);

const uint8_t LanguageId[] =
{
    0x04,
    0x03,
    0x09,
    0x04
};
const uint16_t LanguageIdSize = sizeof(LanguageId);

const uint8_t ManufacturerStringDescriptor[] =
{
    (3 * 2 + 2), //length
    0x03,
    'F', 0,
    'M', 0,
    'D', 0,
};
const uint16_t ManufacturerStringDescriptorSize = sizeof(ManufacturerStringDescriptor);

const uint8_t ProductStringDescriptor[] =
{
    (14 * 2 + 2), //length
    0x03,
    'F', 0,
    'M', 0,
    'D', 0,
    ' ', 0,
    'H', 0,
    'M', 0,
    'I', 0,
    'D', 0,
    ' ', 0,
    'U', 0,
    'S', 0,
    'A', 0,
    'R', 0,
    'T', 0,
};
const uint16_t ProductStringDescriptorSize = sizeof(ProductStringDescriptor);

const uint8_t SerialNumberSringDescriptor[] =
{
    (10 * 2 + 2), //length
    0x03,
    '2', 0,
    '0', 0,
    '2', 0,
    '1', 0,
    '-', 0,
    '1', 0,
    '0', 0,
    '-', 0,
    '2', 0,
    '8', 0,
};
const uint16_t SerialNumberSringDescriptorSize = sizeof(SerialNumberSringDescriptor);

const uint8_t USB_HID_FSReportDescriptor_1 [] =
{
//  0x05, 0x8c, /* USAGE_PAGE (ST Page) */
    0x06, 0x00, 0xFF,
    0x09, 0x01, /* USAGE (Demo Kit) */
    0xa1, 0x01, /* COLLECTION (Application) */

    // The Input report
    0x09, 0x03,             // USAGE ID - Vendor defined
    0x15, 0x00,             // LOGICAL_MINIMUM (0)
    0x26, 0x00, 0xFF,       // LOGICAL_MAXIMUM (255)
    0x75, 0x08,             // REPORT_SIZE (8bit)
    0x95, 0x40,             // REPORT_COUNT (64Byte)
    0x81, 0x02,             // INPUT (Data,Var,Abs)

    // The Output report
    0x09, 0x04,             // USAGE ID - Vendor defined
    0x15, 0x00,             // LOGICAL_MINIMUM (0)
    0x26, 0x00, 0xFF,       // LOGICAL_MAXIMUM (255)
    0x75, 0x08,             // REPORT_SIZE (8bit)
    0x95, 0x40,             // REPORT_COUNT (64Byte)
    0x91, 0x02,             // OUTPUT (Data,Var,Abs)

    0xc0        /* END_COLLECTION */
};
const uint16_t USB_HID_FSReportDescriptor_1Size = sizeof(USB_HID_FSReportDescriptor_1);

const uint8_t USB_HID_FSReportDescriptor_2 [] =
{
    0x05, 0x0C,             //  Usage Page (Consumer)
    0x09, 0x01,             //  Usage (Consumer Control)
    0xA1, 0x01,             //  Collection (Application)
    0x85, 0x01,             //  Report ID (0x01)
    0x09, 0xE0,             //  Usage (Volume)
    0x15, 0xE8,             //  Logical Minimum (-24)
    0x25, 0x18,             //  Logical Maximum (24)
    0x75, 0x07,             //  Report Size (7)
    0x95, 0x01,             //  Report Count (1)
    0x81, 0x06,             //  Input (Var, Rel)
    0x15, 0x00,             //  Logical Minimum (0)
    0x25, 0x01,             //  Logical Maximum (1)
    0x75, 0x01,             //  Report Size (1)
    0x09, 0xE2,             //  Usage (Mute)
    0x81, 0x06,             //  Input (Var, Rel)
    0xC0,                   //End Collection
    0x06, 0x01, 0x00,       //  Usage Page (Generic Desktop Controls)
    0x09, 0x80,             //  Usage (System Control)
    0xA1, 0x01,             //  Collection (Application)
    0x85, 0x02,             //  Report ID (0x02)
    0x25, 0x01,             //  Logical Maximum (1)
    0x15, 0x00,             //  Logical Minimum (0)
    0x75, 0x01,             //  Report Size (1)
    0x0A, 0x81, 0x00,       //  Usage (System Power Down)
    0x0A, 0x82, 0x00,       //  Usage (System Sleep)
    0x0A, 0x83, 0x00,       //  Usage (System Wake Up)
    0x95, 0x03,             //  Report Count (3)
    0x81, 0x06,             //  Input (Var, Rel)
    0x95, 0x05,             //  Report Count (5)
    0x81, 0x01,             //  Input (Const)
    0xC0,                   //  End Collection
    0x06, 0x0C, 0x00,       //  Usage Page (Consumer)
    0x09, 0x01,             //  Usage (Consumer Control)
    0xA1, 0x01,             //  Collection (Application)
    0x85, 0x03,             //  Report ID (0x03)
    0x25, 0x01,             //  Logical Maximum (1)
    0x15, 0x00,             //  Logical Minimum (0)
    0x75, 0x01,             //  Report Size (1)
    0x0A, 0xB5, 0x00,       //  Usage (Scan Next Track)
    0x0A, 0xB6, 0x00,       //  Usage (Scan Previous Track)
    0x0A, 0xB7, 0x00,       //  Usage (Stop)
    0x0A, 0xB8, 0x00,       //  Usage (Eject)
    0x0A, 0xCD, 0x00,       //  Usage (Play/Pause)
    0x0A, 0xE2, 0x00,       //  Usage (Mute)
    0x0A, 0xE9, 0x00,       //  Usage (Volume Increment)
    0x0A, 0xEA, 0x00,       //  Usage (Volume Decrement)
    0x95, 0x08,             //  Report Count (8)
    0x81, 0x02,             //  Input (Var)
    0x0A, 0x83, 0x01,       //  Usage (AL Consumer Control Configuration)
    0x0A, 0x8A, 0x01,       //  Usage (AL Email Reader)
    0x0A, 0x92, 0x01,       //  Usage (AL Calculator)
    0x0A, 0x94, 0x01,       //  Usage (AL Local Machine Browser)
    0x0A, 0x21, 0x02,       //  Usage (AC Search)
    0x0A, 0x23, 0x02,       //  Usage (AC Home)
    0x0A, 0x24, 0x02,       //  Usage (AC Back)
    0x0A, 0x25, 0x02,       //  Usage (AC Forward)
    0x95, 0x08,             //  Report Count (8)
    0x81, 0x02,             //  Input (Var)
    0x0A, 0x26, 0x02,       //  Usage (AC Stop)
    0x0A, 0x27, 0x02,       //  Usage (AC Refresh)
    0x0A, 0x2A, 0x02,       //  Usage (0x22A)
    0x0A, 0xB3, 0x00,       //  Usage (Fast Forward)
    0x0A, 0xB4, 0x00,       //  Usage (Rewind)
    0x95, 0x05,             //  Report Count (5)
    0x81, 0x02,             //  Input (Var)
    0x95, 0x03,             //  Report Count (3)
    0x81, 0x01,             //  Input (Const)
    0xC0,                   //  End Collection
    0x06, 0x00, 0xFF,       //  Usage Page (unk)
    0x09, 0x01,             //  Usage (0x01)
    0xA1, 0x01,             //  Collection (Application)
    0x85, 0x05,             //  Report ID (0x05)
    0x15, 0x00,             //  Logical Minimum (0)
    0x26, 0xFF, 0x00,       //  Logical Maximum (255)
    0x19, 0x01,             //  Usage Minimum (1)
    0x29, 0x02,             //  Usage Maximum (2)
    0x75, 0x08,             //  Report Size (8)
    0x95, 0x05,             //  Report Count (5)
    0xB1, 0x02,             //  Feature (Var)
    0xC0                    //  End Collection
};
const uint16_t USB_HID_FSReportDescriptor_2Size = sizeof(USB_HID_FSReportDescriptor_2);

const uint8_t USB_DEVQUALDescriptor[] =
{
    0x0A,       // bLength
    0x06,       // bDescriptorType
    0x00, 0x02, // bcdUSB
    0x00,       // bDeviceClass
    0x00,       // bDeviceSubClass
    0x00,       // bDeviceProtocol
    0x08,       // bMaxPacketSize0
    0x01,       // bNumConfigurations
    0x00,       // bReserved
};
const uint16_t USB_DEVQUALDescriptor_Size = sizeof(USB_DEVQUALDescriptor);
