/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_ep0.c
* @author    : FMD AE
* @brief     : usb_ep0 program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/

#include "usb_core.h"
#include "usb_ep0.h"
#include "usb_desc.h"

#include "main.h"
#include "uart.h"


M_EPBOUTIN_BUF USB_buffer[4];

uint8_t SetSerialPara=0;
extern uint16_t RecCnt;
  uint8_t SerialPortPara[7] = {0x00,0xC2,0x01,0x00,0x00,0x02,0x08};
extern uint16_t USB_RequestCountI;     // Request  Count In

extern STD_DEV_DSCR	stddevdsc;
extern USART_STRUCT usartPara;
extern uint32_t TickCnt;
extern void osThreadFlagsSet(int tid, int f);
extern int USB_UART_ThreadId;
/* Add configuration structures as defined in config.h */
extern M_CFG_1  cfg1;
extern M_CFG_2  cfg2;
extern M_CFG	cfg;

/* Global status */
extern int		gnDevState;
extern void *		gpCurCfg;
extern PSTD_IF_DSCR	gpCurIf[M_MAXIFS];
extern BYTE		gbyCurIfVal[M_MAXIFS];


/* Local functions */
void USB_Endpoint0_Command (PM_EP0_STATUS, PCOMMAND);
void USB_Endpoint0_Tx (PM_EP0_STATUS);
void USB_StdDev_Req (PM_EP0_STATUS, PCOMMAND);
int ConfigureIfs (void);
void USB_Endpoint0_Rx(PM_EP0_STATUS pep0state);

CLASS_HID_DATA	c_hid_data = {0};

uint32_t temp = 0;
uint32_t hid_temp = 0;
uint8_t	usb_state = 0;


/**********************************************************************************
  * @brief  Endpoint 0 interrupt service routine
	* @param  nCallState :   
	* @note	  None
  *********************************************************************************
*/

void USB_Endpoint0(int nCallState)
{
	static M_EP0_STATUS	ep0state;
	static COMMAND	cmd;
	BYTE	byCSR0;
	

	
	/* Check for USB reset of endpoint 0 */
	if (nCallState == M_EP_RESET)
	{
		ep0state.nState = M_EP0_IDLE;
		ep0state.byFAddr = 0xFF;
		/* Clear current configuration pointer */
		gpCurCfg = NULL;
		return;
	}

	/* Read CSR0 */
	MWRITE_BYTE(M_REG_INDEX, 0);
	byCSR0 = MREAD_BYTE(M_REG_CSR0);

	/* Check for status stage of a request */
	/*receive complete*/
	if(!(byCSR0 & M_CSR0_OUTPKTRDY)) 
	{
		/* Complete SET_ADDRESS command */
		if(ep0state.byFAddr != 0xFF)
		{
			MWRITE_BYTE(M_REG_FADDR, ep0state.byFAddr);
			if((gnDevState == DEVSTATE_DEFAULT) && ep0state.byFAddr)
			{
				gnDevState = DEVSTATE_ADDRESS;
			}
			else if((gnDevState == DEVSTATE_ADDRESS) && !ep0state.byFAddr)
			{
				gnDevState = DEVSTATE_DEFAULT;
			}
		}
	}
	/* Clear pending commands */
	ep0state.byFAddr = 0xFF;

	/* Check for SentStall */
	if(byCSR0 & M_CSR0_SENTSTALL)
	{
		/*clear STALL bit*/
		MWRITE_BYTE(M_REG_CSR0, (byCSR0 & ~M_CSR0_SENDSTALL));
		ep0state.nState = M_EP0_IDLE;
	}

	/* Check for SetupEnd */
	if(byCSR0 & M_CSR0_SETUPEND)
	{
		/*clear setupend bit*/
		MWRITE_BYTE(M_REG_CSR0, (byCSR0 | M_CSR0_SVDSETUPEND));
		ep0state.nState = M_EP0_IDLE;
	}

	/* Call relevant routines for endpoint 0 state */
	if(ep0state.nState == M_EP0_IDLE) 
	{
		/* If no packet has been received, */
		/* assume that this was a STATUS phase complete. */
		/* Otherwise load new command */
		if(byCSR0 & M_CSR0_OUTPKTRDY) 
		{
			/* Read the 8-byte command from the FIFO */
			/* There is no need to check that OutCount is set to 8 */
			/* as the MUSBFSFC will reject SETUP packets that are not 8 bytes long. */
			FIFORead(0, 8, &cmd);
			USB_Endpoint0_Command(&ep0state, &cmd);
		}
	}
	
	if(ep0state.nState == M_EP0_TX)
	{
		USB_Endpoint0_Tx(&ep0state);
	}
	else if(ep0state.nState == M_EP0_RX)
	{
			USB_Endpoint0_Rx(&ep0state);
	}
}



/**********************************************************************************
  * @brief  HID Requests
	* @param  pep0state :    Endpoint 0 status structure
	* @param  pcmd :         Command structure
	* @note	  None
  *********************************************************************************
*/
void HID_SetReportEvent(PM_EP0_STATUS pep0state,PCOMMAND pcmd)
{
	if ((pcmd->bmRequestType == HID_REQUEST_SET) && (pcmd->wIndex == 0))// Interface 0
	{
		switch(pcmd->wValue >>8)
		{
			case	HID_REPORT_INPUT:break;
			case	HID_REPORT_OUTPUT:
				/*Save request type*/
				c_hid_data.wHID_Status = HID_REQUEST_SET_REPORT;
			
				pep0state->nBytesLeft = pcmd->wLength;
				pep0state->nState = M_EP0_RX;
				pep0state->pData = (void*)&c_hid_data.wHID_SetRptByte[0];
				MWRITE_BYTE(M_REG_CSR0,M_CSR0_SVDOUTPKTRDY);			
				
				break;
			case	HID_REPORT_FEATURE:break;
		}
	}	
}

/**********************************************************************************
  * @brief  USB HID Get Report Event
	* @param  pep0state :    Endpoint 0 status structure
	* @param  pcmd :         Command structure
	* @note	  None
  *********************************************************************************
*/
void HID_GetReportEvent(PM_EP0_STATUS pep0state,PCOMMAND pcmd)
{
	if(gnDevState > DEVSTATE_ADDRESS)
	{
		switch(pcmd->wValue >>8)
		{
			case	HID_REPORT_INPUT:
			case	HID_REPORT_FEATURE:
			case	HID_REPORT_OUTPUT:
				/*Save request type*/
				c_hid_data.wHID_Status = HID_REQUEST_GET_REPORT;
			
				pep0state->nBytesLeft = 0x01;
				pep0state->pData = (void*)&c_hid_data.wHID_SetRptByte[0];
				pep0state->nState = M_EP0_TX;
				MWRITE_BYTE(M_REG_CSR0,M_CSR0_SVDOUTPKTRDY);
				break;
		}
	}	
}

/**********************************************************************************
  * @brief  USB HID Get Idle Event
	* @param  pep0state :    Endpoint 0 status structure
	* @param  pcmd :         Command structure
	* @note	  None
  *********************************************************************************
*/
void HID_GetIdleEvent(PM_EP0_STATUS pep0state,PCOMMAND pcmd)
{
	/*Save request type*/
	c_hid_data.wHID_Status = HID_REQUEST_GET_IDLE;
	
	pep0state->nBytesLeft = 0x01;
	pep0state->pData = (void*)&c_hid_data.wHID_IdleTimeIf0ID;
	pep0state->nState = M_EP0_TX;
	MWRITE_BYTE(M_REG_CSR0,M_CSR0_SVDOUTPKTRDY);	
}

/**********************************************************************************
  * @brief  USB HID Set Idle Event
	* @param  pep0state :    Endpoint 0 status structure
	* @param  pcmd :         Command structure
	* @note	  None
  *********************************************************************************
*/
void HID_SetIdleEvent(PM_EP0_STATUS pep0state,PCOMMAND pcmd)
{
	/*Save request type*/
	c_hid_data.wHID_Status = HID_REQUEST_SET_IDLE;

	pep0state->nState = M_EP0_IDLE;
	c_hid_data.wHID_IdleTimeIf0ID = (pcmd->wValue>>8);
	MWRITE_BYTE(M_REG_CSR0,M_CSR0_SVDOUTPKTRDY);		
}

/**********************************************************************************
  * @brief  USB HID Get Protocol Event
	* @param  pep0state :    Endpoint 0 status structure
	* @param  pcmd :         Command structure
	* @note	  None
  *********************************************************************************
*/
void HID_GetProtocolEvent(PM_EP0_STATUS pep0state,PCOMMAND pcmd)
{
	/*Save request type*/
	c_hid_data.wHID_Status = HID_REQUEST_GET_PROTOCOL;
	
	pep0state->nBytesLeft = 0x01;
	pep0state->pData = (void*)&c_hid_data.wHID_Protocol;
	pep0state->nState = M_EP0_TX;
	MWRITE_BYTE(M_REG_CSR0,M_CSR0_SVDOUTPKTRDY);		
}

/**********************************************************************************
  * @brief  USB Set Protocol Event
	* @param  pep0state :    Endpoint 0 status structure
	* @param  pcmd :         Command structure
	* @note	  None
  *********************************************************************************
*/

void HID_SetProtocolEvent(PM_EP0_STATUS pep0state,PCOMMAND pcmd)
{
	/*Save request type*/
	c_hid_data.wHID_Status = HID_REQUEST_SET_PROTOCOL;

	pep0state->nState = M_EP0_IDLE;
	c_hid_data.wHID_Protocol = (pcmd->wValue & 0x00ff);
	MWRITE_BYTE(M_REG_CSR0,M_CSR0_SVDOUTPKTRDY);		
}
/**********************************************************************************
  * @brief  USB HID CLASS REQUIRE
	* @param  pep0state :    Endpoint 0 status structure
	* @param  pcmd :         Command structure
	* @note	  None
  *********************************************************************************
*/
void USB_HID_Class_Req(PM_EP0_STATUS pep0state,PCOMMAND pcmd)
{
	switch(pcmd->bRequest)
	{
		case	HID_REQUEST_GET_REPORT:
		{
			HID_GetReportEvent(pep0state,pcmd);
			return;
		}
		case	HID_REQUEST_GET_IDLE:
		{
			HID_GetIdleEvent(pep0state,pcmd);
			return;
		}
		case	HID_REQUEST_GET_PROTOCOL:
		{
			HID_GetProtocolEvent(pep0state,pcmd);
			return;
		}
		case	HID_REQUEST_SET_REPORT:
		{
			HID_SetReportEvent(pep0state,pcmd);
			return;
		}
		case	HID_REQUEST_SET_IDLE:
		{
			HID_SetIdleEvent(pep0state,pcmd);
			return;
		}
		case	HID_REQUEST_SET_PROTOCOL:
		{
			HID_SetProtocolEvent(pep0state,pcmd);
			return;
		}
		default:
		{
			break;
		}
	}
	MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY | M_CSR0_SENDSTALL);
}   

/**************************************************/
/* CDC Requests                                   */
/**************************************************/
#define SEND_ENCAPSULATED_COMMAND               0x00
#define GET_ENCAPSULATED_RESPONSE               0x01
#define SET_COMM_FEATURE                        0x02
#define GET_COMM_FEATURE                        0x03
#define CLEAR_COMM_FEATURE                      0x04
#define SET_LINE_CODING                         0x20
#define GET_LINE_CODING                         0x21
#define SET_CONTROL_LINE_STATE                  0x22
#define SEND_BREAK                              0x23
#define NO_CMD                                  0xFF

typedef struct
{
  uint32_t bitrate;
  uint8_t  format;
  uint8_t  paritytype;
  uint8_t  datatype;
}LINE_CODING;

LINE_CODING linecoding =
{
    115200, /* baud rate*/
    0x00,   /* stop bits-1*/
    0x00,   /* parity - none*/
    0x08    /* nb. of bits 8*/
};  

uint8_t  CmdBuf[8] = {0};

/**********************************************************************************
  * @brief  USB CDC CLASS REQUIRE
	* @param  pep0state :    Endpoint 0 status structure
	* @param  pcmd :         Command structure
	* @param  Buf  :         Buffer
	* @note	  None
  *********************************************************************************
*/

void USB_CDC_Class_Req(PM_EP0_STATUS pep0state,PCOMMAND pcmd,uint8_t* Buf)
{      
    switch (pcmd->bRequest)
    {
        case SEND_ENCAPSULATED_COMMAND:
        /* Not  needed for this driver */
        break;

        case GET_ENCAPSULATED_RESPONSE:
        /* Not  needed for this driver */
        break;

        case SET_COMM_FEATURE:
        /* Not  needed for this driver */
        break;

        case GET_COMM_FEATURE:
        /* Not  needed for this driver */
        break;

        case CLEAR_COMM_FEATURE:
        /* Not  needed for this driver */
        break;

        case SET_LINE_CODING:
            linecoding.bitrate = (uint32_t)(Buf[0] | (Buf[1] << 8) | (Buf[2] << 16) | (Buf[3] << 24));
            linecoding.format = Buf[4];
            linecoding.paritytype = Buf[5];
            linecoding.datatype = Buf[6];
            pep0state->nBytesLeft = pcmd->wLength;
            pep0state->pData = (void*)Buf;
            pep0state->nState = M_EP0_RX;
            MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);
						SetSerialPara = 1;
						RecCnt = 0;
        return;

        case GET_LINE_CODING:			
						Buf[0] = SerialPortPara[0];
						Buf[1] = SerialPortPara[1];
						Buf[2] = SerialPortPara[2];
						Buf[3] = SerialPortPara[3];
						Buf[4] = SerialPortPara[4];
						Buf[5] = SerialPortPara[5];
						Buf[6] = SerialPortPara[6];
            pep0state->nBytesLeft = pcmd->wLength;
            pep0state->pData = (void*)Buf;
            pep0state->nState = M_EP0_TX;
            MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY);
        return;

        case SET_CONTROL_LINE_STATE:
            MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY| M_CSR0_DATAEND);
        return;

        case SEND_BREAK:
            /* Not  needed for this driver */
        break; 

        default:
        break;
    } 
}


/**********************************************************************************
  * @brief  USB Endpoint 0 command
	* @param  pep0state :    Endpoint 0 status structure
	* @param  pcmd :         Command structure
	* @note	  This routine is called when a command has been received in the SETUP phase.
  *********************************************************************************
*/

void USB_Endpoint0_Command(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
	switch(pcmd->bmRequestType & M_CMD_TYPEMASK)
	{
		/***Host to device,standard request command, receiver to device***/
		case M_CMD_STDREQ:
			USB_StdDev_Req(pep0state, pcmd);
			break;

		/***Host to device,class request command,receiver is device***/
		case M_CMD_CLASSREQ:
            /* Add call to external routine for handling class requests */
            if((pcmd->bRequest& 0x20) == 0)
            {
			
                USB_HID_Class_Req(pep0state,pcmd);
            }
            else
            {
                USB_CDC_Class_Req(pep0state,pcmd,CmdBuf);
            }
            
			break;
		/***Host to device, user-defined command, receiver to device***/
		case M_CMD_VENDREQ:
		/* Add call to external routine for handling vendor requests */
			
		/***reserved***/
		default:
			/* Stall the command if a reserved request is received */
			MWRITE_BYTE(M_REG_CSR0, M_CSR0_SVDOUTPKTRDY | M_CSR0_SENDSTALL);
		break;
    }
	return;
}



/**********************************************************************************
  * @brief  Standard Device Request
	* @param  pep0state :    Endpoint 0 status structure
	* @param  pcmd :         Command structure
	* @note	  this routine is called when a standard device request has been received.
  The bRequest field of the received command is decoded.
  *********************************************************************************
*/

void USB_StdDev_Req(PM_EP0_STATUS pep0state, PCOMMAND pcmd)
{
	int	bError = FALSE, bNoData = FALSE, n;
	BYTE	by;
	BYTE	byConfig;
	PSTD_CFG_DSCR	pcfg;
	
	switch(pcmd->bRequest) 
	{
		/***Used by the host to assign an address to the device***/
		case SET_ADDRESS:
        {
			/* Store device function address until status stage of request */
			if (pcmd->bmRequestType != M_CMD_STDDEVOUT)
			{
				bError = TRUE;
			}
			else if (gnDevState <= DEVSTATE_ADDRESS) 
			{
				pep0state->byFAddr = (BYTE)pcmd->wValue;
				bNoData = TRUE;
			}
			else
			{
				bError = TRUE;
			}
        }
		break;
			
            /***Used by the host to get the specific descriptor of the device***/		
		case GET_DESCRIPTOR:
        {
			/* Decode the required descriptor from the command */
      /***The receiver of the report descriptor is the interface cmd->bmRequestType is 0x81 and transmit hid report descriptor***/
			if(!(pcmd->bmRequestType & M_CMD_STDDEVIN))
			{
				bError = TRUE;
			}
			else 
			{
				switch(pcmd->wValue & M_CMD_DESCMASK) 
				{
					/***Device descriptor***/
					case M_CMD_DEVICE:
						/* Prepare to return Standard Device Descriptor */
						pep0state->nBytesLeft = USB_FSDeviceDescriptorSize;
						/* Check host is allowing a descriptor this long */
						if(pcmd->wLength < pep0state->nBytesLeft)
						{
							pep0state->nBytesLeft = pcmd->wLength;
						}
						
						pep0state->pData = (void*)USB_FSDeviceDescriptor;
						pep0state->nState = M_EP0_TX;
						break;
					/***Device Qualifier***/	
					case M_CMD_DEVQUAL://
						/* No alternate speed supported */
					/* Prepare to return Standard Device Descriptor */
						pep0state->nBytesLeft = USB_DEVQUALDescriptor_Size;
						/* Check host is allowing a descriptor this long */
						if(pcmd->wLength < pep0state->nBytesLeft)
						{
							pep0state->nBytesLeft = pcmd->wLength;
						}
						
						pep0state->pData = (void*)USB_DEVQUALDescriptor;
						pep0state->nState = M_EP0_TX;
						
//						bError = TRUE;
						break;
					/***configure descriptor***/					
					case M_CMD_CONFIG:
						byConfig = (BYTE)(pcmd->wValue & 0x00FF);
						if(byConfig>USB_FSDeviceDescriptor[USB_FSDeviceDescriptorSize-1])
						{
							bError = TRUE;
						}
						else
						{
							/* Get pointer to requested configuration descriptor */
							/* Prepare to return Configuration Descriptors */
							pep0state->nBytesLeft = USB_FSConfigDescriptorSize;
							pep0state->pData = (void*)USB_FSConfigDescriptor;
							pep0state->nState = M_EP0_TX;
							/* Check host is allowing a descriptor this long */
							if (pcmd->wLength < pep0state->nBytesLeft)
							{
								pep0state->nBytesLeft = pcmd->wLength;
							}
						}
						break;
					/***Other speed configuration descriptors***/						
					case M_CMD_OTHERSPEED:
						/* No alternate speed supported */
						bError = TRUE;
						break;
					/***HID report descriptor***/
					case	M_CMD_HID_REPORT_DESC:		//		0x2200
						/* Prepare to return Standard Device Descriptor */
						switch(pcmd->wIndex)
						{
							case	0:
								//port 0
								pep0state->nBytesLeft = USB_HID_FSReportDescriptor_1Size;		//report descriptor length
								/* Check host is allowing a descriptor this long */
								if (pcmd->wLength < pep0state->nBytesLeft)
								{
									pep0state->nBytesLeft = pcmd->wLength;
								}
								pep0state->pData = (void*)USB_HID_FSReportDescriptor_1;
								pep0state->nState = M_EP0_TX;
								osThreadFlagsSet(USB_UART_ThreadId, 0x81U);
								break;
							case	1:
								//port 1
								pep0state->nBytesLeft = USB_HID_FSReportDescriptor_2Size;		//report descriptor length
								/* Check host is allowing a descriptor this long */
								if (pcmd->wLength < pep0state->nBytesLeft)
								{
									pep0state->nBytesLeft = pcmd->wLength;
								}
								pep0state->pData = (void*)USB_HID_FSReportDescriptor_2;
								pep0state->nState = M_EP0_TX;
								break;	
							default:
							
								bError = TRUE;
								break;							
						}
						break;
					case	M_CMD_STRING:	//String descriptor
						switch(pcmd->wValue & 0xff)
						{
							case	0:	//Get language ID
								pep0state->nBytesLeft = LanguageIdSize;		
								/* Check host is allowing a descriptor this long */
								if (pcmd->wLength < pep0state->nBytesLeft)
								{
									pep0state->nBytesLeft = pcmd->wLength;
								}
								pep0state->pData = (void*)LanguageId;
								pep0state->nState = M_EP0_TX;
							break;
							case	1:	//Since the manufacturer string index value is 1,write 1 here.
								pep0state->nBytesLeft = ManufacturerStringDescriptorSize;		
								/* Check host is allowing a descriptor this long */
								if (pcmd->wLength < pep0state->nBytesLeft)
								{
									pep0state->nBytesLeft = pcmd->wLength;
								}
								pep0state->pData = (void*)ManufacturerStringDescriptor;
								pep0state->nState = M_EP0_TX;
							break;	
							case	2:	//Since the index value of the product string is 2, write 2 here.
								pep0state->nBytesLeft = ProductStringDescriptorSize;		
								/* Check host is allowing a descriptor this long */
								if (pcmd->wLength < pep0state->nBytesLeft)
								{
									pep0state->nBytesLeft = pcmd->wLength;
								}
								pep0state->pData = (void*)ProductStringDescriptor;//company descripttion
								pep0state->nState = M_EP0_TX;
								osThreadFlagsSet(USB_UART_ThreadId, 0x81U);

							break;			
							case	3:	//Since the index value of product serial number string is 3, 3 is written here.
								pep0state->nBytesLeft = SerialNumberSringDescriptorSize;		
								/* Check host is allowing a descriptor this long */
								if (pcmd->wLength < pep0state->nBytesLeft)
								{
									pep0state->nBytesLeft = pcmd->wLength;
								}
								pep0state->pData = (void*)SerialNumberSringDescriptor;
								pep0state->nState = M_EP0_TX;
							break;
								
								case	4:	
								pep0state->nBytesLeft = SerialNumberSringDescriptorSize;		
								/* Check host is allowing a descriptor this long */
								if (pcmd->wLength < pep0state->nBytesLeft)
								{
									pep0state->nBytesLeft = pcmd->wLength;
								}
								pep0state->pData = (void*)SerialNumberSringDescriptor;
								pep0state->nState = M_EP0_TX;
							break;
								case	5:	
								pep0state->nBytesLeft = SerialNumberSringDescriptorSize;		
								/* Check host is allowing a descriptor this long */
								if (pcmd->wLength < pep0state->nBytesLeft)
								{
									pep0state->nBytesLeft = pcmd->wLength;
								}
								pep0state->pData = (void*)SerialNumberSringDescriptor;
								pep0state->nState = M_EP0_TX;
							break;
							default:
								bError = TRUE;								
							break;
						}
					break;
					/***reserved***/
					default: bError = TRUE;
				}
			}
        }
		break;

            /***Used to specify the required configuration for the host device***/
		case SET_CONFIGURATION:
        {
			byConfig = (BYTE)(pcmd->wValue & 0x00FF);
			if(gnDevState == DEVSTATE_DEFAULT)
			{
				bError = TRUE;
			}
			/* Assumes configurations are numbered 1 to NumConfigurations */
			else if(byConfig >USB_FSDeviceDescriptor[USB_FSDeviceDescriptorSize-1])
			{
				bError = TRUE;
			}
			else if(!byConfig)
			{
				gnDevState = DEVSTATE_ADDRESS;
			}
			else
			{
				/* Get pointer to requested configuration descriptor */
				gpCurCfg = (void *)USB_FSConfigDescriptor;
				/* Set all alternate settings to zero */
				for (n=0; n<M_MAXIFS; n++)
					gbyCurIfVal[n] = 0;
				/* Configure endpoints */
				ConfigureIfs();
				gnDevState = DEVSTATE_CONFIG;
				MWRITE_BYTE(M_REG_INDEX, 0);
				bNoData = TRUE;
			}
        }
	    break;

            /***It is used for the host to require the device to use a descriptor to describe the interface**/		
		case SET_INTERFACE:
        {
			if(gnDevState > DEVSTATE_ADDRESS)
			{
				n = (int)pcmd->wIndex;
				pcfg = (PSTD_CFG_DSCR)gpCurCfg;
				if((BYTE)n >= pcfg->bNumInterfaces)
				{
					bError = TRUE;
				}
				else 
				{
					gbyCurIfVal[n] = (BYTE)pcmd->wValue;
					if (ConfigureIfs() == FALSE)
					{
						bError = TRUE;
					}
					
					MWRITE_BYTE(M_REG_INDEX, 0);
					bNoData = TRUE;
				}
			}
			else
			{
				bError = TRUE;
			}
        }
		break;

            /***Used to obtain the current device configuration value of the host device***/		
		case GET_CONFIGURATION:
        {
			if ((gnDevState == DEVSTATE_ADDRESS) && (!pcmd->wValue))
			{
				/* Prepare to return zero */
				pep0state->nBytesLeft = 1;
				pep0state->pData = (void*)&pcmd->wValue;
				pep0state->nState = M_EP0_TX;
			}
			else if(gnDevState > DEVSTATE_ADDRESS)
			{
				/* Prepare to return configuration value */
				pcfg = (PSTD_CFG_DSCR)gpCurCfg;
				pep0state->nBytesLeft = 1;
				pep0state->pData = (void*)&pcfg->bConfigurationValue;
				pep0state->nState = M_EP0_TX;
			}
			else
			{
				bError = TRUE;
			}
        }
		break;

		/***Used to get the number of a ***/	
		case GET_INTERFACE:
        {
			if(gnDevState > DEVSTATE_ADDRESS)
			{
				n = (int)pcmd->wIndex;
				pcfg = (PSTD_CFG_DSCR)gpCurCfg;
				if((BYTE)n >= pcfg->bNumInterfaces)
				{
					bError = TRUE;
				}
				else 
				{
					/* Prepare to return interface value */
					pep0state->nBytesLeft = 1;
					pep0state->pData = (void*)&gpCurIf[n]->bInterfaceNumber;
					pep0state->nState = M_EP0_TX;
				}
			}
			else
            {
				bError = TRUE;
            }
        }
		break;

		/***Some features used to enable or activate the command receiver***/
		case SET_FEATURE:
        {
			switch (pcmd->bmRequestType) 
			{
					case M_CMD_STDDEVOUT:
						switch (pcmd->wValue) 
						{
							case M_FTR_DEVREMWAKE:
							default:
							bError = TRUE;
							break;
						}
					break;
					case M_CMD_STDIFIN:
					/* Add interface features here */
					case M_CMD_STDEPIN:
					/* Add endpoint features here */
					default:
					bError = TRUE;
					break;
			}
        }
		break;
            /***Unrecognized request***/
		default:
			/* Stall the command if an unrecognized request is received */
			bError = TRUE;
		break;
    }

	/* Stall command if an error occurred */
	if (bError)
	{
		/***Clear the OUTPUKTRDY bit to terminate the current transmission***/
		by = M_CSR0_SVDOUTPKTRDY | M_CSR0_SENDSTALL;
	}
	else if(bNoData)
	{
		/***Clear the OUTPUKTRDY bit. When sending, it means that the last packed of data will be sent,and when receiving, it means that the last packet of data will be received***/
		by = M_CSR0_SVDOUTPKTRDY | M_CSR0_DATAEND;
	}
	else
	{
		/***Clear OUTPKTRDY bit***/
		by = M_CSR0_SVDOUTPKTRDY;
	}
	
	MWRITE_BYTE(M_REG_CSR0, by);

	return;
	
}


/**********************************************************************************
  * @brief  USB Endpoint 0 Tx
	* @param  pep0state :    Endpoint 0 status structure
	* @note	  This routine is called when data is to be sent to the host.
  *********************************************************************************
*/

void USB_Endpoint0_Tx(PM_EP0_STATUS pep0state)
{
	int	nBytes;
	BYTE	by;

	/* Determine number of bytes to send */
	if(pep0state->nBytesLeft <= M_EP0_MAXP)
	{
		nBytes = pep0state->nBytesLeft;
		pep0state->nBytesLeft = 0;
	}
	else
	{
		nBytes = M_EP0_MAXP;
		pep0state->nBytesLeft -= M_EP0_MAXP;
	}

	FIFOWrite(0, nBytes, pep0state->pData);
	pep0state->pData = (BYTE *)pep0state->pData + nBytes;
	if(nBytes < M_EP0_MAXP)
	{
		/***The data has been written into FIFO,and the last packet of data will be sent**/
		by = M_CSR0_INPKTRDY | M_CSR0_DATAEND;
		pep0state->nState = M_EP0_IDLE;
	}
	else
	{
		/***data wriTten to FIFO***/
		by = M_CSR0_INPKTRDY;
	}
	
	MWRITE_BYTE(M_REG_CSR0, by);
}


/**********************************************************************************
  * @brief  USB_Endpoint0_Rx
	* @param  pep0state :    Endpoint 0 status structure
	* @note	  
  *********************************************************************************
*/
void USB_Endpoint0_Rx(PM_EP0_STATUS pep0state)
{
	BYTE    byOutCSR;
	
	/* Set index register */
	MWRITE_BYTE(M_REG_INDEX, 0);
	/* Read OUT CSR register */
	byOutCSR = MREAD_BYTE(M_REG_CSR0);
	
	if(byOutCSR & M_CSR0_OUTPKTRDY)
	{
		pep0state->nState = M_EP0_IDLE;
		FIFORead(0, 7, SerialPortPara);
		MWRITE_BYTE(M_REG_CSR0,M_CSR0_SVDOUTPKTRDY);
//		/* Check for another packet */
		byOutCSR = MREAD_BYTE(M_REG_CSR0);
	}
		
}


/**********************************************************************************
  * @brief  ConfigureIfs
  * @param  None
	* @note	  This routine is called when a SET_CONFIGURATION or SET_INTERFACE
  command is received and will set the gpCurIF pointers
  to the selected interface descriptors and will set the
  maximum packet size and operating mode for the endpoints in
  the selected interfaces
  *********************************************************************************
*/

int ConfigureIfs(void)
{
	PSTD_CFG_DSCR	pcfg;
	BYTE		byIf, byAltIf, byEP, byNumEPs, by;
	BYTE		*pby;
	BYTE		*pbyIfVal;
	PSTD_IF_DSCR	pif;
	PSTD_EP_DSCR	pep;

	/* Set pointer to first interface descriptor in current configuration */
	pby = (BYTE *)(gpCurCfg) + sizeof(STD_CFG_DSCR) + 8;
	pif = (PSTD_IF_DSCR)pby;

	/* Loop through all interfaces in the current configuration */
	pcfg = (PSTD_CFG_DSCR)gpCurCfg;
	pbyIfVal = (BYTE*)&gbyCurIfVal;	
	for (byIf=0; byIf<pcfg->bNumInterfaces; byIf++, pbyIfVal++) 
	{
		/* Advance pointer to selected alternate interface descriptor */
 		if (*pbyIfVal) 
		{
			for (byAltIf=0; byAltIf<*pbyIfVal; byAltIf++) 
			{
				byNumEPs = pif->bNumEndpoints;
				pby += sizeof(STD_IF_DSCR) + byNumEPs * sizeof(STD_EP_DSCR) + sizeof(STD_HID_DSCR);
				pif = (PSTD_IF_DSCR)pby;
				/* Check an alternate setting > number of alternates not specified */
				if (!pif->bAlternateSetting)
				{
					return FALSE;
				}
			} 
		}

		/* Store pointer to interface in global array */
		gpCurIf[byIf] = pif;

		/* Loop through all endpoints in interface */
		byNumEPs = pif->bNumEndpoints;
        
		if(byIf == 0)
		{
            pby += sizeof(STD_IF_DSCR) + 9;
		}
		else if(byIf == 1)
		{
			pby += sizeof(STD_IF_DSCR)+19 + 8 ;
		}
        else
		{
			pby += sizeof(STD_IF_DSCR);
		}
        
		for (byEP=0; byEP<byNumEPs; byEP++)
		{
			pep = (PSTD_EP_DSCR)pby;

			/* Configure the endpoint */
			MWRITE_BYTE(M_REG_INDEX, (pep->bEndpointAddress & 0x0F));
			/* Round up max packet size to a multiple of 8 for writing to MaxP registers */
			by = (BYTE)((pep->wMaxPacketSize+7)>>3);
			if(pep->bEndpointAddress & 0x80)
			{
				MWRITE_BYTE(M_REG_INMAXP, by);
				by = MREAD_BYTE(M_REG_INCSR2);
				switch (pep->bmAttributes & M_EP_TFMASK)
				{
					case M_EP_ISO:
						by |= M_INCSR2_ISO;
					break;
					case M_EP_BULK:
					case M_EP_INTR:						
						by &= ~M_INCSR2_ISO;
					break;
				}
				/* Set mode bit high (only strictly necessary if sharing a FIFO) */
				by |= M_INCSR2_MODE;
				MWRITE_BYTE(M_REG_INCSR2, by);
				/* Other configuration for an IN endpoint */
				/* e.g. AutoClr, DMA */
				/* should be added here */
			}
			else
			{
				MWRITE_BYTE(M_REG_OUTMAXP, by);
				by = MREAD_BYTE(M_REG_OUTCSR2);
				switch (pep->bmAttributes & M_EP_TFMASK)
				{
					case M_EP_ISO:
						by |= M_OUTCSR2_ISO;
					break;
					case M_EP_BULK:
					case M_EP_INTR:
						by &= ~M_OUTCSR2_ISO;
					break;
				}
				MWRITE_BYTE(M_REG_OUTCSR2, by);
				/* Other configuration for an OUT endpoint */
				/* e.g. AutoSet, DMA */
				/* should be added here */
			}
			
			pby += sizeof(STD_EP_DSCR);	
		}

		if(*pby != USB_FSConfigDescriptor[USB_FSConfigDescriptorSize-1])
		{
			/* Skip any alternate setting interfaces */
            if(byIf == 0)
            {
                pif = (PSTD_IF_DSCR)(pby+8);
            }
            else if(byIf == 1)
            {
                pif = (PSTD_IF_DSCR)pby;
            }
		}
	}

	return TRUE;
}

/**********************************************************************************
  * @brief  Send data through port
  * @param    Ep     :  port number  
  * @param  *ptr     :  data pointer  
  * @param  data_len :  data length  
	* @note	  
  * @retval None
  *********************************************************************************
*/

ErrorStatus USB_EP_Tx(uint8_t Ep,uint8_t *ptr,uint8_t data_len)
{
	uint16_t timeout = 0;
	
	if(gnDevState > DEVSTATE_ADDRESS)      
	{
		gepbin1.byEP = Ep;
		gepbin1.nBytesLeft = data_len;
		gepbin1.pData = ptr;
		
		EndpointBulkIn(&gepbin1, M_EP_NORMAL);
		while(MREAD_BYTE(M_REG_INCSR1) & 0x01)
		{
			timeout++;
			if(timeout>=0xffff)
			{
				return ERROR;
			}
		}
		return SUCCESS;
	}
	
	return ERROR;
}


/**********************************************************************************
  * @brief  Read data through port and save buffer.
  * @param  nEP :  port number  
	* @note	  
  * @retval None
  *********************************************************************************
*/

void USB_EP_RRx(uint8_t Ep)
{
	static uint8_t ReceIndex = 0;
    
		gepbout1.byEP = Ep;
    gepbout1.pData = USB_buffer[ReceIndex].recvBuf;
    gepbout1.nBuffLen = 128;
    gepbout1.nBytesRecv = 0;
    EndpointBulkOut(&gepbout1,M_EP_NORMAL);
    USB_buffer[ReceIndex].recvNum = gepbout1.nBytesRecv;
    USB_buffer[ReceIndex].recvStatus = 1;
    ReceIndex++;
    if(ReceIndex>=4)
        ReceIndex = 0;
}


