/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : USBD_User_HID_0.c
* @author    : FMD AE
* @brief     : USBD_User_HID_0 program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/

#include <stdint.h>
#include <string.h>
#include "cmsis_os2.h"
#define   osObjectsExternal
#include "osObjects.h"
#include "rl_usb.h"

#include "uart.h"
#include "main.h"


#define COM_DELAY_TIME          5000


#define USB_UART_PACKET_COUNT        4U
#define USB_UART_PACKET_SIZE         64U



uint16_t USBcnt;
uint32_t TickCnt;
extern  M_EPBOUT_BUF gBulkoutData2;
extern M_EPBOUTIN_BUF USB_buffer[4];

extern uint8_t SetSerialPara;
extern uint8_t SerialPortPara[7];
uint8_t ConnectedFlg = 0;
uint16_t RecCnt;
volatile uint16_t NumDataRead = 0;
#define BUSY           1
#define IDLE           0
#if (USBD_HID0_OUT_REPORT_MAX_SZ != USB_UART_PACKET_SIZE)
    #error "USB HID0 Output Report Size must match DAP Packet Size"
#endif
#if (USBD_HID0_IN_REPORT_MAX_SZ != USB_UART_PACKET_SIZE)
    #error "USB HID Input Report Size must match DAP Packet Size"
#endif

/**********************************************************************************
  * @brief USB_UART_Thread
    * @param  *argument
    * @note   main loop program
  * @retval None
  *********************************************************************************
*/
void USB_UART_Thread(void *argument)
{
    static uint8_t RecDelay = 2;

    uint8_t i, k;
    uint8_t SendBufferSerial[MAX_COM_BUFFER_SIZE];
    uint16_t SendCntSerial;

    uint8_t SendBufferUsb[MAX_COM_BUFFER_SIZE];
    uint16_t SendCntUsb;
    uint16_t usartDelayTime = 0;

    uint32_t bitrate;
    uint8_t format, paritytype, datatype;
    (void)   argument;
    (void)   k;

    usartPara.IniFlg = 1;
    for (;;)
    {
        //maintain seamless periodic flashing
        usartPara.saveTime = TickCnt;
        if (!(usartPara.saveTime % 500))
            break;
    }
    usartPara.saveTime = TickCnt;
    SetSerialPara = 1;
    for (;;)
    {
        if ((SetSerialPara) && (RecCnt > 10))
        {
            //Reconfigure serial port parameters
            SetSerialPara = 0;
            bitrate = (uint32_t)(SerialPortPara[0] | (SerialPortPara[1] << 8) | (SerialPortPara[2] << 16) | (SerialPortPara[3] << 24));
            format = SerialPortPara[4];
            paritytype = SerialPortPara[5];
            datatype = SerialPortPara[6];
            if (bitrate > 9600)
                RecDelay = 1;
            else
            {
                i = 9600 / bitrate;
                RecDelay = 1 + i;
            }

            ReIniUartByUSB(bitrate, format, paritytype, datatype);
        }

        if ((usartPara.RecFlg) && (RecCnt > RecDelay))
        {
            SendCntSerial = usartPara.recevCnt;
            memcpy(SendBufferSerial, usartPara.recevBuffer, SendCntSerial);
            RecCnt = 0;

            usartPara.recevCnt = 0;
            usartPara.RecFlg   = 0;
            usb_send_data(SendBufferSerial, SendCntSerial, 4); //Receive data from the serial port and send it directly to the USB virtual serial port

            usartPara.ComFlg = 1;
            usartPara.saveTime = TickCnt;

            NumDataRead = SendCntSerial;
        }

        if ((gBulkoutData2.recvStatus == 1) && (USBcnt > 2))
        {
            SendCntUsb = gBulkoutData2.recvNum;
            memcpy(SendBufferUsb, gBulkoutData2.recvBuf, SendCntUsb);
            gBulkoutData2.recvStatus = 0;
            SendSerialData(SendBufferUsb, SendCntUsb);
            usartPara.ComFlg = 1;
            usartPara.saveTime = TickCnt;
        }

        //LED process
        if (usartPara.ComFlg | usartPara.IniFlg)
        {
            if (usartPara.saveTime <= (0xffffffff - COM_DELAY_TIME))
            {
                usartDelayTime = TickCnt - usartPara.saveTime;
            }
            else if (TickCnt >= usartPara.saveTime)
            {
                usartDelayTime = TickCnt - usartPara.saveTime;
            }
            else
            {
                usartDelayTime = 0xffffffff - usartPara.saveTime + TickCnt;
            }

            if (usartDelayTime >= COM_DELAY_TIME)
            {
                usartPara.ComFlg = 0;
                usartPara.IniFlg = 0;
            }
        }


        for (i = 0; i < USB_UART_PACKET_COUNT; i++)
        {
            if (USB_buffer[i].recvStatus == BUSY)
            {
                SendCntUsb = USB_buffer[i].recvNum;
                memcpy(SendBufferUsb, USB_buffer[i].recvBuf, SendCntUsb);
                USB_buffer[i].recvStatus = IDLE;
                SendSerialData(SendBufferUsb, SendCntUsb);//send usb data by uart
            }
        }
    }
}
