/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_core.h
* @author    : FMD AE
* @brief     : Header for usb_core.c module
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

#ifndef _USB_CORE_H_
#define _USB_CORE_H_

/* Includes ---------------------------------------------------------------------*/
#include "ft32f0xx.h"
#include "usb_desc.h"
#include "usb_ep0.h"
#include "usb_ep.h"
/* Public Constant prototypes----------------------------------------------------*/

/* Public typedef ---------------------------------------------------------------*/
/* Define MUSBFSFC register addresses */
#define BASE_ADDR 0x40005c00

#define M_REG_FADDR     BASE_ADDR
#define M_REG_POWER     BASE_ADDR+1
#define M_REG_INTRIN1   BASE_ADDR+2
#define M_REG_INTRIN2   BASE_ADDR+3
#define M_REG_INTROUT1  BASE_ADDR+4
#define M_REG_INTROUT2  BASE_ADDR+5
#define M_REG_INTRUSB   BASE_ADDR+6
#define M_REG_INTRIN1E  BASE_ADDR+7
#define M_REG_INTRIN2E  BASE_ADDR+8
#define M_REG_INTROUT1E BASE_ADDR+9
#define M_REG_INTROUT2E BASE_ADDR+10
#define M_REG_INTRUSBE  BASE_ADDR+11
#define M_REG_FRAME1    BASE_ADDR+12
#define M_REG_FRAME2    BASE_ADDR+13
#define M_REG_INDEX     BASE_ADDR+14

#define M_REG_CSR0      BASE_ADDR+17
#define M_REG_INMAXP    BASE_ADDR+16
#define M_REG_INCSR1    BASE_ADDR+17
#define M_REG_INCSR2    BASE_ADDR+18
#define M_REG_OUTMAXP   BASE_ADDR+19
#define M_REG_OUTCSR1   BASE_ADDR+20
#define M_REG_OUTCSR2   BASE_ADDR+21
#define M_REG_OUTCOUNT1 BASE_ADDR+22
#define M_REG_OUTCOUNT2 BASE_ADDR+23

#define M_FIFO_EP0      BASE_ADDR+32


/* Interrupt register bit masks */
#define M_INTR_SUSPEND  0x01
#define M_INTR_RESUME   0x02
#define M_INTR_RESET    0x04
#define M_INTR_EP0      0x0001

/* CSR0 bit masks */
#define M_CSR0_OUTPKTRDY        0x01
#define M_CSR0_INPKTRDY         0x02
#define M_CSR0_SENTSTALL        0x04
#define M_CSR0_DATAEND          0x08
#define M_CSR0_SETUPEND         0x10
#define M_CSR0_SENDSTALL        0x20
#define M_CSR0_SVDOUTPKTRDY     0x40
#define M_CSR0_SVDSETUPEND      0x80

/* Endpoint CSR register bits */
#define M_INCSR2_ISO    0x40
#define M_INCSR2_MODE   0x20
#define M_INCSR_CDT     0x40
#define M_INCSR_FF      0x08
#define M_INCSR_IPR     0x01
#define M_OUTCSR2_ISO   0x40
#define M_OUTCSR_CDT    0x80
#define M_OUTCSR_FF     0x10
#define M_OUTCSR_OPR    0x01


#define IS_USB_ALL_PERIPH(PERIPH)       ((PERIPH) == USB)

#define USB_Register_FADDR              ((uint8_t)0x00)
#define USB_Register_POWER              ((uint8_t)0x01)
#define USB_Register_INTRIN             ((uint8_t)0x02)
#define USB_Register_INTROUT            ((uint8_t)0x04)
#define USB_Register_INTRUSB            ((uint8_t)0x06)
#define USB_Register_INTRINE            ((uint8_t)0x07)
#define USB_Register_INTROUTE           ((uint8_t)0x09)
#define USB_Register_INTRUSBE           ((uint8_t)0x0B)
#define USB_Register_FRAM1              ((uint8_t)0x0C)
#define USB_Register_FRAM2              ((uint8_t)0x0D)
#define USB_Register_INDEX              ((uint8_t)0x0E)
#define USB_Register_PDCTRL             ((uint8_t)0x0F)
#define USB_Register_INMAXP             ((uint8_t)0x10)
#define USB_Register_INCSR0             ((uint8_t)0x11)
#define USB_Register_INCSR1             ((uint8_t)0x11)
#define USB_Register_INCSR2             ((uint8_t)0x12)
#define USB_Register_OUTMAXP            ((uint8_t)0x13)
#define USB_Register_OUTCSR1            ((uint8_t)0x14)
#define USB_Register_OUTCSR2            ((uint8_t)0x15)
#define USB_Register_OUTCOUNTER         ((uint8_t)0x16)
#define USB_Register_FIFO0              ((uint8_t)0x20)
#define USB_Register_FIFO1              ((uint8_t)0x24)
#define USB_Register_FIFO2              ((uint8_t)0x28)
#define USB_Register_FIFO3              ((uint8_t)0x2C)
#define USB_Register_FIFO4              ((uint8_t)0x30)
#define USB_Register_FIFO5              ((uint8_t)0x34)
#define USB_Register_FIFO6              ((uint8_t)0x38)
#define USB_Register_FIFO7              ((uint8_t)0x3C)

#define IS_USB_REGISTER(REGISTER)       (((REGISTER) == USB_Register_FADDR      ) || \
                                                                                ((REGISTER) == USB_Register_POWER     ) || \
                                                                                ((REGISTER) == USB_Register_INTRIN    ) || \
                                                                                ((REGISTER) == USB_Register_INTROUT   ) || \
                                                                                ((REGISTER) == USB_Register_INTRUSB   ) || \
                                                                                ((REGISTER) == USB_Register_INTRINE   ) || \
                                                                                ((REGISTER) == USB_Register_INTROUTE  ) || \
                                                                                ((REGISTER) == USB_Register_INTRUSBE  ) || \
                                                                                ((REGISTER) == USB_Register_FRAM1     ) || \
                                                                                ((REGISTER) == USB_Register_FRAM2     ) || \
                                                                                ((REGISTER) == USB_Register_INDEX     ) || \
                                                                                ((REGISTER) == USB_Register_PDCTRL    ) || \
                                                                                ((REGISTER) == USB_Register_INMAXP    ) || \
                                                                                ((REGISTER) == USB_Register_INCSR0    ) || \
                                                                                ((REGISTER) == USB_Register_INCSR1    ) || \
                                                                                ((REGISTER) == USB_Register_INCSR2    ) || \
                                                                                ((REGISTER) == USB_Register_OUTMAXP   ) || \
                                                                                ((REGISTER) == USB_Register_OUTCSR1   ) || \
                                                                                ((REGISTER) == USB_Register_OUTCSR2   ) || \
                                                                                ((REGISTER) == USB_Register_OUTCOUNTER) || \
                                                                                ((REGISTER) == USB_Register_FIFO0     ) || \
                                                                                ((REGISTER) == USB_Register_FIFO1     ) || \
                                                                                ((REGISTER) == USB_Register_FIFO2     ) || \
                                                                                ((REGISTER) == USB_Register_FIFO3     ) || \
                                                                                ((REGISTER) == USB_Register_FIFO4     ) || \
                                                                                ((REGISTER) == USB_Register_FIFO5     ) || \
                                                                                ((REGISTER) == USB_Register_FIFO6     ) || \
                                                                                ((REGISTER) == USB_Register_FIFO7     ))


/**
  * @brief USB_EP
  */
#define USB_EP_0    ((uint8_t)0x00)
#define USB_EP_1    ((uint8_t)0x01)
#define USB_EP_2    ((uint8_t)0x02)
#define USB_EP_3    ((uint8_t)0x03)
#define USB_EP_4    ((uint8_t)0x04)
#define USB_EP_5    ((uint8_t)0x05)
#define USB_EP_6    ((uint8_t)0x06)
#define USB_EP_7    ((uint8_t)0x07)

#define IS_USB_EP(EPn)          (((EPn) >= 0x00) && ((EPn) < 0x08))


/**
  * @brief USB_PDCTRL
  */
#define USB_PDCTRL_FLAG         ((uint8_t)(0x03))

#define USB_PDCTRL_PD_DISABLE   ((uint8_t)(0x00))
#define USB_PDCTRL_PD_ENABLE    ((uint8_t)(0x02))

#define USB_PDCTRL_PU_DISABLE   ((uint8_t)(0x00))
#define USB_PDCTRL_PU_ENABLE    ((uint8_t)(0x01))

#define IS_USB_PDCTRL(PDCTRL)   (((PDCTRL) == USB_PDCTRL_PD_DISABLE) || \
                                ((PDCTRL) == USB_PDCTRL_PD_ENABLE) || \
                                ((PDCTRL) == USB_PDCTRL_PU_DISABLE) || \
                                ((PDCTRL) == USB_PDCTRL_PU_ENABLE))

/**
  * @brief   USB_FLAG
  */
#define USB_FADDR_UPDATE_FLAG       ((uint32_t) 0x00000180)

#define USB_POWER_RESET_FLAG        ((uint32_t) 0x00000208)
#define USB_POWER_SUSPENDM_FLAG     ((uint32_t) 0x00000202)

/**
  * @brief   USB_IT_FLAG
  */
#define USB_IT_IN_EP0_FLAG          ((uint32_t) 0x00000101)
#define USB_IT_IN_EP1_FLAG          ((uint32_t) 0x00000102)
#define USB_IT_IN_EP2_FLAG          ((uint32_t) 0x00000104)
#define USB_IT_IN_EP3_FLAG          ((uint32_t) 0x00000108)
#define USB_IT_IN_EP4_FLAG          ((uint32_t) 0x00000110)
#define USB_IT_IN_EP5_FLAG          ((uint32_t) 0x00000120)
#define USB_IT_IN_EP6_FLAG          ((uint32_t) 0x00000140)
#define USB_IT_IN_EP7_FLAG          ((uint32_t) 0x00000180)

#define USB_IT_OUT_EP1_FLAG         ((uint32_t) 0x00000202)
#define USB_IT_OUT_EP2_FLAG         ((uint32_t) 0x00000204)
#define USB_IT_OUT_EP3_FLAG         ((uint32_t) 0x00000208)
#define USB_IT_OUT_EP4_FLAG         ((uint32_t) 0x00000210)
#define USB_IT_OUT_EP5_FLAG         ((uint32_t) 0x00000220)
#define USB_IT_OUT_EP6_FLAG         ((uint32_t) 0x00000240)
#define USB_IT_OUT_EP7_FLAG         ((uint32_t) 0x00000280)

#define USB_IT_USB_SUSPEND_FLAG     ((uint32_t) 0x00000401)
#define USB_IT_USB_RESUME_FLAG      ((uint32_t) 0x00000402)
#define USB_IT_USB_RESET_FLAG           ((uint32_t) 0x00000404)
#define USB_IT_USB_SOF_FLAG             ((uint32_t) 0x00000408)
/* Register read/write macros */
#define MREAD_BYTE(addr) *((BYTE *)(addr))
#define MWRITE_BYTE(addr,data) *((BYTE *)(addr)) = data

/* Define endpoint call status values */
#define M_EP_NORMAL     0
#define M_EP_RESET      1

/* Define device states */
#define DEVSTATE_DEFAULT        0
#define DEVSTATE_ADDRESS        1
#define DEVSTATE_CONFIG         2

/* Define standard constants */
#define TRUE  1
#define FALSE 0
#ifndef NULL
    #define NULL 0
#endif

/*USE EPN */
#define USE_EP_N(X)     X

/*ep direction ,excepet ep0*/
#define EP_DIREC_IN 0x20
#define EP_DIREC_OUT    0x00
/* Public variables prototypes --------------------------------------------------*/
/*BULK CBW BUFF*/
/* Public function prototypes----------------------------------------------------*/
void FIFORead(int, int, void *);
void FIFOWrite(int, int, void *);
uint8_t USB_ReadRegister(uint8_t USB_Refister);
void USB_PDCTRLConfig(uint8_t PDCT);
void USB_ITConfig(uint32_t USB_IT, FunctionalState NewState);
ITStatus USB_GetITStatus(uint32_t USB_IT);

#endif
/************************* (C) COPYRIGHT FMD *****END OF FILE*********************/

