/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : usb_ep.c
* @author    : FMD AE
* @brief     : usb_ep program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "usb_ep.h"
#include "usb_scsi.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
uint8_t recieved_buff[512] = {0};
/*BULK CBW BUFF*/
USBC_CBW_TYPE   usb_cbw;
USBC_CSW_TYPE   usb_csw;
/* Public variables --------------------------------------------------------------*/
uint32_t recieved_len = 0;
/* Status for 2 Bulk In endpoints */
extern M_EPBIN_STATUS  gepbin1, gepbin2, gepbin3;
/* Private function prototypes ---------------------------------------------------*/
void Data_Copy(uint8_t *old_buff, uint8_t *new_buff, uint32_t data_len);
void Pro_Scsi_Out_Data(PM_EPBOUT_STATUS pbostate);
/* Public function ---------------------------------------------------------------*/

/******************************************************************************
  * @brief  USB_Ep_Direc program
  * @param  ep
  *         Direction
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_Ep_Direc(uint8_t ep, uint8_t Direction)
{
    /*set ep*/
    USB->INDEX = ep;
    /*clear direction*/
    *((BYTE *)M_REG_INCSR2) &= ~(0x01 << 5);
    /*set direction*/
    *((BYTE *)M_REG_INCSR2) |= Direction;
}
/******************************************************************************
  * @brief  Endpoint_Tx_data program
  * @param  pbistate
  * @note
  * @retval None
  *****************************************************************************
*/
void Endpoint_Tx_data(PM_EPBIN_STATUS pbistate)
{
    int     nBytes;
    BYTE    byInCSR;

    /* Check whether there is any data to send */
    if (pbistate->nBytesLeft != M_EP_NODATA)
    {
        /* Determine number of bytes to send */
        if (pbistate->nBytesLeft <= M_EP_MAXP)
        {
            nBytes = pbistate->nBytesLeft;
            pbistate->nBytesLeft = M_EP_NODATA;

            /* Load FIFO */
            FIFOWrite((int)pbistate->byEP, nBytes, pbistate->pData);
            pbistate->pData = epn_send_buf;

            MWRITE_BYTE(M_REG_INDEX, pbistate->byEP);
            byInCSR = M_INCSR_IPR;
            MWRITE_BYTE(M_REG_INCSR1, byInCSR);

            //?1??y?Y?????????????D-?䨬????
            if (USB_CBW_STATUS == CBW_DATA_STAGE)
            {
                USB_CBW_STATUS = CBW_STATE_STAGE;
                USB_SCSI_CMD_STAGE(pbistate, 0, 0);
            }
            else if (USB_CBW_STATUS == CBW_STATE_STAGE) //?1??䨬????????????????
            {
                USB_CBW_STATUS = CBW_CMD_STAGE;  //?????????
            }
        }
        else
        {
            nBytes = M_EP_MAXP;
            pbistate->nBytesLeft -= M_EP_MAXP;

            /* Load FIFO */
            FIFOWrite((int)pbistate->byEP, nBytes, pbistate->pData);
            pbistate->pData = (BYTE *)pbistate->pData + nBytes;

            MWRITE_BYTE(M_REG_INDEX, pbistate->byEP);

            byInCSR = M_INCSR_IPR;
            MWRITE_BYTE(M_REG_INCSR1, byInCSR);

            //?1??READ(10)???2??y?Y?????Da?????y?Y
            if ((usb_cbw.CBWCB[0] == READ10_CMD) && (USB_CBW_STATUS == CBW_DATA_STAGE))
            {
#if defined USB_DEBUG
                printf("read10_left:%x\r\n", pbistate->nBytesLeft);
#endif
                Get_Disk_Data(pbistate); //?????y?Y
            }
        }
    }
    else
    {
        if (USB_CBW_STATUS == CBW_DATA_STAGE) //2?|y?Y???
        {
            //??????䨬????
            USB_CBW_STATUS = CBW_STATE_STAGE;
            USB_SCSI_CMD_STAGE(pbistate, 0, 0);
        }
    }
}

/******************************************************************************
  * @brief  Endpoint_Rx_Data program
  * @param  pbistate
  * @note
  * @retval None
  *****************************************************************************
*/
void Endpoint_Rx_Data(PM_EPBOUT_STATUS pbostate)
{
    int     nBytes;
    BYTE    byOutCSR;

    /* Set index register */
    MWRITE_BYTE(M_REG_INDEX, pbostate->byEP);

    /* Read OUT CSR register */
    byOutCSR = MREAD_BYTE(M_REG_OUTCSR1);

    /* Get packet, */
    /* may need to unload more than one packet if double buffering enabled */
    /*˵FIFO */
    while (byOutCSR & M_OUTCSR_OPR)
    {
        /* Get packet size */
        nBytes = (int)MREAD_BYTE(M_REG_OUTCOUNT2);
        nBytes <<= 8;
        nBytes |= (int)MREAD_BYTE(M_REG_OUTCOUNT1);

//      /* Check there is room in the buffer */
//      if(pbostate->nBytesRecv + nBytes > pbostate->nBuffLen)
//      {
//          /* Call to function to handle buffer over run */
//          MWRITE_BYTE(M_REG_OUTCSR1, 0);
//          return;
//      }

        /* Unload FIFO */
        FIFORead((int)pbostate->byEP, nBytes, pbostate->pData);

        /* Update status */
        pbostate->nBytesRecv += nBytes;
        pbostate->pData = (BYTE *)pbostate->pData + nBytes;

        /* Check for end of transfer */
        if (nBytes <= M_EP_MAXP)
        {
            /* Call function to handle received data */
            pbostate->pData = epn_receive_buf;
        }

        /* Clear OutPktRdy */
        MWRITE_BYTE(M_REG_OUTCSR1, 0);

        /* Check for another packet */
        byOutCSR = MREAD_BYTE(M_REG_OUTCSR1);
    }
}

/******************************************************************************
  * @brief  EndpointBulkIn  program
  * @param  pbistate
  *         nCallState
  * @note   EndpointBulkIn service routine
  *         This routine provides simple handling for bulk in endpoints.
  *         Endpoint MaxP is defined in endpoint.h.
  *         While the nBytesLeft field of the pbostate structure is >= 0
  *         the endpoint will send packets of MaxP, with the residue in the last packet.
  *         If the data to send is an exact multiple of MaxP then a null data packet
  *         will be sent.
  * @retval None
  *****************************************************************************
*/
void EndpointBulkIn(PM_EPBIN_STATUS pbistate, int nCallState)
{
    BYTE    byInCSR;

    /* Reset endpoint */
    if (nCallState == M_EP_RESET)
    {
        /*л˵*/
        MWRITE_BYTE(M_REG_INDEX, pbistate->byEP);
        /*Ӧ˵ݷת״̬FIFOҪ͵*/
        byInCSR = M_INCSR_CDT | M_INCSR_FF;
        MWRITE_BYTE(M_REG_INCSR1, byInCSR);
    }
    else
    {
        Endpoint_Tx_data(pbistate);
    }
}



/******************************************************************************
  * @brief  EndpointBulkOut program
  * @param  pbistate
  *         nCallState
  * @note   EndpointBulkOut service routine
  *         This routine provides simple handling for bulk out endpoints.
  *         Endpoint MaxP is defined in endpoint.h.
  *         It assumes a transfer is complete when a packet of less than MaxP is received.
  * @retval None
  *****************************************************************************
*/
void EndpointBulkOut(PM_EPBOUT_STATUS pbostate, int nCallState)
{
    BYTE    byOutCSR;

    /* Reset endpoint */
    if (nCallState == M_EP_RESET)
    {
        /***???????o?***/
        MWRITE_BYTE(M_REG_INDEX, pbostate->byEP);
        byOutCSR = M_OUTCSR_CDT | M_OUTCSR_FF;
        /***???????y?Ya䨬?,?????? FIFO ?D?y?Y***/
        MWRITE_BYTE(M_REG_OUTCSR1, byOutCSR);
    }
    else
    {
        /**/
        if (USB_CBW_STATUS == CBW_CMD_STAGE)
        {
            Endpoint_Rx_Data(pbostate);

            if ((epn_receive_buf[0] == 0X55) && (epn_receive_buf[1] == 0X53) && \
                    (epn_receive_buf[2] == 0X42) && (epn_receive_buf[3] == 0X43))
            {
                Data_Copy((void *)&usb_cbw, epn_receive_buf, sizeof(usb_cbw));
                USB_SCSI_CMD(&gepbin2, &usb_cbw);
            }
        }
        else if (USB_CBW_STATUS == CBW_DATA_STAGE)
        {
            Pro_Scsi_Out_Data(pbostate);
        }
    }
}
/******************************************************************************
  * @brief  Data_Copy program
  * @param  old_buff
  *         new_buff
  *         data_len
  * @note
  * @retval None
  *****************************************************************************
*/
void Data_Copy(uint8_t *old_buff, uint8_t *new_buff, uint32_t data_len)
{
    uint32_t i = 0;

    for (i = 0; i < data_len; i++)
    {
        old_buff[i] = new_buff[i];
    }
}
/******************************************************************************
  * @brief  Pro_Scsi_Out_Data program
  * @param  pbostate
  * @note
  * @retval None
  *****************************************************************************
*/
void Pro_Scsi_Out_Data(PM_EPBOUT_STATUS pbostate)
{
    Endpoint_Rx_Data(pbostate);

    recieved_len -= M_EP_MAXP;

    Data_Copy(recieved_buff, pbostate->pData, M_EP_MAXP);

    if (recieved_len == 0)
    {
        USB_CBW_STATUS = CBW_CMD_STAGE;
        USB_SCSI_CMD_STAGE(&gepbin2, 0, 0);
        Endpoint_Tx_data(&gepbin2);
    }
}
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/

