/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : bsp_usb.c
* @author    : FMD AE
* @brief     : bsp_usb program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ----------------------------------------------------------------------*/
#include "bsp_usb.h"
/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
uint8_t usb_state = 0;

/* Public variables --------------------------------------------------------------*/
VarBit  flag1;
uint8_t ep1_send_buf[64] = {0};
uint8_t ep3_send_buf[64] = {0};
uint8_t epn_send_buf[64] = {0};
uint8_t epn_receive_buf[64] = {0};
/* Status for 2 Bulk Out endpoints */
M_EPBOUT_STATUS gepbout1, gepbout2, gepbout3, gepbout4;
M_EPBIN_STATUS  gepbin1, gepbin2, gepbin3;
/* Private function prototypes ---------------------------------------------------*/

/* Public function ---------------------------------------------------------------*/
/******************************************************************************
  * @brief  USB_Init    program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_Init(void)
{
    NVIC_InitTypeDef NVIC_InitStructure;

    if (RCC_GetFlagStatus(RCC_FLAG_HSI48RDY) != SET)
    {
        RCC_HSI48Cmd(ENABLE);
        while (RCC_GetFlagStatus(RCC_FLAG_HSI48RDY) != SET);
    }
    RCC_USBCLKConfig(RCC_USBCLK_HSI48);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USB, ENABLE);

    USB_ITConfig(USB_IT_USB_RESUME_FLAG, DISABLE);
    USB_ITConfig(USB_IT_USB_SUSPEND_FLAG, DISABLE);
    USB_ITConfig(USB_IT_USB_SOF_FLAG, DISABLE);
    USB_ITConfig(USB_IT_USB_RESET_FLAG, ENABLE);

    USB_PDCTRLConfig(USB_PDCTRL_PU_ENABLE);

    NVIC_InitStructure.NVIC_IRQChannel = USB_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_CRS, ENABLE);
    CRS_FrequencyErrorCounterCmd(ENABLE);
    CRS_AutomaticCalibrationCmd(ENABLE);
}

/******************************************************************************
  * @brief  USB_FS_IRQHandler   program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_FS_IRQHandler(void)
{
    uint8_t usb_intrin = 0, usb_intrusb = 0, usb_introut = 0;

    usb_intrin = USB->INTRIN;
    usb_intrusb = USB->INTRUSB;
    usb_introut = USB->INTROUT;
    /*Check for resume from suspend mode, Add call to resume routine here */
    if ((usb_intrusb & 0X02) != RESET)
    {
        usb_state = USB_RESUME_FLAG;
    }

    if (usb_intrusb & USB_IT_USB_SOF_FLAG)
    {
        USB->INTRUSB &= ~0x08;//sof
        f_sof_flag = 1;
    }
    /* Check for system interrupts */
    if ((usb_intrusb & USB_IT_USB_RESET_FLAG) != RESET)
    {
        USB_Reset();
    }

    /* Check for endpoint 0 interrupt */
    if ((usb_intrin & USB_IT_IN_EP0_FLAG) != RESET)
    {
        USB_Endpoint0(M_EP_NORMAL);
    }

    if ((usb_intrin & USB_IT_IN_EP1_FLAG) != RESET)
    {
        EndpointBulkIn(&gepbin1, M_EP_NORMAL);
    }

    if ((usb_intrin & USB_IT_IN_EP3_FLAG) != RESET)
    {
        EndpointBulkIn(&gepbin3, M_EP_NORMAL);
    }

    if ((usb_introut & USB_IT_OUT_EP2_FLAG) != RESET)
    {
        EndpointBulkOut(&gepbout1, M_EP_NORMAL);
    }


    /* Check for suspend mode */
    /* Add call to suspend routine here*/
    if ((usb_intrusb & 0x01) != RESET)
    {
        if (gnDevState > DEVSTATE_ADDRESS)
        {
            usb_state = USB_SUSPEND_FLAG;       //usb suspend
        }
    }
}

/******************************************************************************
  * @brief  USB_Reset   program
  * @param  None
  * @note
  * @retval None
  *****************************************************************************
*/
void USB_Reset(void)
{
    /* Set device into default state */
    gnDevState = DEVSTATE_DEFAULT;

    /* Reset any state machines for each endpoint */
    USB_Endpoint0(M_EP_RESET);

    /* Initialise endpoint status structures with endpoint numbers */
    gepbin1.byEP = USE_EP_N(1);
    gepbin1.nBytesLeft = M_EP_NODATA;
    gepbin1.pData = epn_send_buf;

    gepbout1.byEP = USE_EP_N(2);
    gepbout1.nBuffLen = M_EP_MAXP;
    gepbout1.nBytesRecv = M_EP_NODATA;
    gepbout1.pData = epn_receive_buf;

    gepbin3.byEP = USE_EP_N(3);
    gepbin3.nBytesLeft = M_EP_NODATA;
    gepbin3.pData = ep3_send_buf;

    EndpointBulkIn(&gepbin1, M_EP_RESET);
    EndpointBulkOut(&gepbout1, M_EP_RESET);
    EndpointBulkIn(&gepbin3, M_EP_RESET);
}
/************************** (C) COPYRIGHT FMD **********************END OF FILE****/


