/********** (C) COPYRIGHT 2021 Fremont Micro Devices (SZ) Corporation *********
* @file      : main.c
* @author    : FMD AE
* @brief     : main program body
* @version   : V1.0.0
* @data      : 2021-09-14
******************************************************************************
* @attention
* COPYRIGHT (C) 2021 Fremont Micro Devices (SZ) Corporation All rights reserved.
* This software is provided by the copyright holders and contributors,and the
*   software is believed to be accurate and reliable. However, Fremont Micro
*   Devices (SZ) Corporation assumes no responsibility for the consequences of
*   use of such software or for any infringement of patents of other rights
*   of third parties, which may result from its use. No license is granted by
*   implication or otherwise under any patent rights of Fremont Micro Devices (SZ)
*   Corporation.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private Constant --------------------------------------------------------------*/
/* Public Constant ---------------------------------------------------------------*/
/* Private typedef ---------------------------------------------------------------*/
/* Private define ----------------------------------------------------------------*/
/* Private variables -------------------------------------------------------------*/
/* Public variables --------------------------------------------------------------*/
/* Private function prototypes ---------------------------------------------------*/
static void  Buttons_Initialize(void);
/* Public function ------ --------------------------------------------------------*/
/* Private function ------ -------------------------------------------------------*/
/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
    /*!< At this stage the microcontroller clock setting is already configured,
    this is done through SystemInit() function which is called from startup
    file (startup_ft32f0xx.s) before to branch to application main.
    To reconfigure the default setting of SystemInit() function, refer to
    system_ft32f0xx.c file
    */

    /* LED and key2 configuration */
    LedConfig(LED1);
    LedConfig(LED2);
    Buttons_Initialize();

    /* Check if the system has resumed from WWDG reset */
    if (RCC_GetFlagStatus(RCC_FLAG_WWDGRST) != RESET)
    {
        /* WWDGRST flag set */
        /* Turn on LED1 */
        LedOn(LED1);

        /* Clear reset flags */
        RCC_ClearFlag();
    }
    else
    {
        /* WWDGRST flag is not set */
        /* Turn off LED1 */
        LedOff(LED1);
    }

    /* Setup SysTick Timer for 1 msec interrupts  */
    SystickInit();

    /* Configure WWDG */
    WWDG_Config();

    while (1)
    {
        /* Toggle LED2 */
        LedToggle(LED2);

        /* Insert 33 ms delay */
        Delay(33);

        /* Update WWDG counter */
        WWDG_SetCounter(127);
    }
}
/**
  * @brief  Buttons_Initialize program.
  * @param  None
  * @retval None
  */
static void  Buttons_Initialize(void)
{
    GPIO_InitTypeDef    GPIO_InitStructure;
    EXTI_InitTypeDef    EXTI_InitStructure;
    NVIC_InitTypeDef    NVIC_InitStructure;

    /* BUTTONS GPIO Periph clock enable */
    RCC_AHBPeriphClockCmd(RCC_AHBENR_GPIOCEN, ENABLE);

    /* Configure BUTTON1_GPIO_Pin in input pull down mode */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;

    GPIO_Init(GPIOC, &GPIO_InitStructure);

    /* SYSCFG Config */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
    SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOC, EXTI_PinSource13);

    /* EXTI INIT */
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_InitStructure.EXTI_Line = EXTI_Line13;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;

    EXTI_Init(&EXTI_InitStructure);

    /* NVIC Init */
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannel = EXTI4_15_IRQn;
    NVIC_Init(&NVIC_InitStructure);
}

/************************ (C) COPYRIGHT Fremont Micro Devices *****END OF FILE****/
